/*
 *  ADDNOISE
 *
 *      This routine accepts an float image of size (nsize x nsize)
 *      pointed to by (imgptr), adds pseudo-Gaussian or
 *      pseudo-uniform noise of mean (mean) and variance
 *      (sigma squared) to the input to create the output image
 *      ( = addnoise() ).
 *          option = 0 <--> don't add any noise
 *          option = 1 <--> add float gaussian rv's
 *          option = 2 <--> add truncated int's gaussian rv's
 *          option = 3 <--> add rounded int's gaussian rv's
 *          option = 4 <--> add float uniform rv's
 *          option = 5 <--> add truncated int's uniform rv's
 *          option = 6 <--> add rounded int's uniform rv's
 *
 *  Author:     Paul J. Besl    (2-25-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           addnoise(float_img_ptr,n_size,mean,sigma,option)
 *  Modules used:   gaussrand()     unifrand()
 *
 */

#include <math.h>

void addnoise(fltimgptr,nsize,mean,sigma,option)
float   *fltimgptr;
int     nsize,option;
float   mean,sigma;
{
    float               gaussrand(),unifrand();
    register int        i,nsqr;
    register float      *fltptr,temp;
    float               avg,avgsqr,stddev,fmax,fmin;
    int                 tsecs;

    if( option == 0 )
        return;
    tsecs = tstart();
    nsqr = nsize*nsize;
    fltptr = fltimgptr;
    avg = 0.0;
    avgsqr = 0.0;
    fmax = mean - 1000.0*sigma;
    fmin = mean + 1000.0*sigma;
    switch( option)
    {
        case 1: for(i=0; i<nsqr; ++i)
                {
                    temp = (sigma*gaussrand()+mean);
                    *fltptr += temp;
                    fltptr++;
                    avg += temp;
                    avgsqr += temp*temp;
                    if( temp > fmax )
                        fmax = temp;
                    if( temp < fmin )
                        fmin = temp;
                }
                break;
        case 2: for(i=0; i<nsqr; ++i)
                {
                    temp = (int)(sigma*gaussrand()+mean);
                    *fltptr += temp;
                    fltptr++;
                    avg += temp;
                    avgsqr += temp*temp;
                    if( temp > fmax )
                        fmax = temp;
                    if( temp < fmin )
                        fmin = temp;
                }
                break;
        case 3: mean += 0.5;
                for(i=0; i<nsqr; ++i)
                {
                    temp = (int)(sigma*gaussrand()+mean);
                    *fltptr += temp;
                    fltptr++;
                    avg += temp;
                    avgsqr += temp*temp;
                    if( temp > fmax )
                        fmax = temp;
                    if( temp < fmin )
                        fmin = temp;
                }
                break;
        case 4: for(i=0; i<nsqr; ++i)
                {
                    temp = (sigma*unifrand()+mean);
                    *fltptr += temp;
                    fltptr++;
                    avg += temp;
                    avgsqr += temp*temp;
                    if( temp > fmax )
                        fmax = temp;
                    if( temp < fmin )
                        fmin = temp;
                }
                break;
        case 5: for(i=0; i<nsqr; ++i)
                {
                    temp = (int)(sigma*unifrand()+mean);
                    *fltptr += temp;
                    fltptr++;
                    avg += temp;
                    avgsqr += temp*temp;
                    if( temp > fmax )
                        fmax = temp;
                    if( temp < fmin )
                        fmin = temp;
                }
                break;
        case 6: mean += 0.5;
                for(i=0; i<nsqr; ++i)
                {
                    temp = (int)(sigma*unifrand()+mean);
                    *fltptr += temp;
                    fltptr++;
                    avg += temp;
                    avgsqr += temp*temp;
                    if( temp > fmax )
                        fmax = temp;
                    if( temp < fmin )
                        fmin = temp;
                }
                break;
        default:
                printf("addnoise: Bad Option=%d\n",option);
                exit(1);
    }
    avg    /= (float)nsqr;
    avgsqr /= (float)nsqr;
    stddev  = avgsqr - avg*avg;
    stddev  = sqrt(stddev);
    printf("Noise_Mean=%g | Noise_Std_Dev=%g\n",avg,stddev);
    printf("Noise_Max=%g | Noise_Min=%g\n",fmax,fmin);
    tstop(tsecs,"Time_to_Add_Noise");
}
