/*
 *  BLKBDRCI
 *
 *      This routine put black border into unsigned char image.
 *      Some routines are affected by extraneous
 *      data in outer frame region of image where
 *      there is not enough data to compute output.
 *      This routine sets pixels equal to zero.
 *
 *  Author:     Paul J. Besl    (6-27-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           blkbdrci(unsig_char_image,x_size,y_size,window_size)
 *  Modules used:   none
 *
 */
 
void blkbdrci(fltimg,nxsize,nysize,wsize)
unsigned char   *fltimg;
int             nxsize,nysize,wsize;
{
    register int            i,j;
    register unsigned char  *lptr,*rptr;
    int                     whalf,offset,stop;
    unsigned char           *start;

    whalf = wsize/2;
    stop = nysize - whalf;
    start = fltimg + whalf;
    offset = nxsize - wsize;

    /* fixup first whalf and last two scanlines of image */

    for(i=0; i<whalf; ++i)       
    {
        lptr = fltimg + i*nxsize;
        rptr = fltimg + (i+stop)*nxsize;
        for(j=0; j<nxsize; ++j)
            *lptr++ = *rptr++ = 0;
    }

    /* fixup two pixels at beginning and end of each scanline */
    for(i=whalf; i<stop; ++i)
    {
        lptr = start + i*nxsize;
        rptr = lptr + offset;
        for(j=0; j<whalf; ++j)
            *--lptr = *++rptr = 0;
    }
}
