/*
 *  CMPMENU
 *
 *      This routine processes menu commands with compute options.
 *
 *  Author:     Paul J. Besl    (2-13-85)
 *
 *  Revised:    Arnold. H. Chiu (8-22-88)
 *  
 *  Call:           cmpmenu(cmp_options,string,string_size,default_filenames)
 *  Modules used:   savstring()     ttyclrhom()
 *
 */

#include <stdio.h>

#define MAXMENU     32
#define NUMCMDS     23
#define LIST        20
#define RETURN      21
#define GOBACK      22

int cmpmenu(cmpoptions,string,strsize,dfnames)
int     *cmpoptions,strsize;
char    *string,*dfnames[];
{
    /* default switch settings for output files */
    static int cmpinit[NUMCMDS] =
        {
            7,  7,  5,  0, 1000, 1000,  3,  3,  0,  3,
            0,  2, 10, 10,  2,  3,  1, 10,  0,  0,
            0,  0,  0
        };

    /* 2-char mnemonic commands */
    static char mnemonic[NUMCMDS][4] =
        {
            "ps","dw","hk","nf","zh","zk","sc","dz","mc","sm",
            "an","sd","l0","l1","l2","l3","l4","dt","ip","pc",
            "ls","co","gb"
        };

    /* Menu List */
    static char menubuf[NUMCMDS][MAXMENU] =
    {
        "Pre-Smooth Window_Size(1=off) ",
        "Derivative Window_Size(1=off) ",
        "HK-Smooth Window_Size(1=off)  ",
        "Display_But_Don't_Write_Files ",
        "Zero_Band_Millionths_for_H    ",
        "Zero_Band_Millionths_for_K    ",
        "HK-Smooth_Computation_Option  ",
        "Deanza_Display_of_Each_Image  ",
        "Deanza_Memory_Channel(0123)   ",
        "Pre-Smooth_Computation_Option ",
        "Additive_Noise_Flag(0,1-3,4-6)",
        "Standard_Deviation_of_Noise   ",
        "Low_Threshold_for_Edge_Op_0_eg",
        "Low_Threshold_for_Edge_Op_1_ah",
        "Low_Threshold_for_Edge_Op_2_dp",
        "Low_Threshold_for_Edge_Op_3_pg",
        "Low_Threshold_for_Edge_Op_4_qv",
        "Diffrnc_Threshold_for_Edge_Ops",
        "Interactive_Prompts_for_Edges ",
        "Print_Curvature_Statistics    ",
        "List_Options                  ",
        "Continue                      ",
        "Go_Back                       "
    };

    int             nargs,icase,defcase,param;
    char            junk[8];
    register int    i,j;

    /* set up default switch settings */

    for(i=0; i<NUMCMDS; ++i)
        cmpoptions[i] = cmpinit[i];
    icase = -1;
    defcase = LIST;

    /* Enter Command Prompt Loop */

    enter_command:

        printf("\nEnter Compute Option(%s)__ ",mnemonic[defcase]);
        if( fgets(string,strsize,stdin) == NULL )
        {
            printf("\n");
            exit(1);
        }

    /* Handle Default Case  (nargs=0 --> mnemonic entered) */

    if( strlen(string) <= 1 )
    {
        icase = defcase;
        nargs = 1;
    }
    else
        nargs = sscanf(string,"%d",&icase);

    /* Number Specification Case and Default Case */

    /* printf("nargs=%d icase=%d\n",nargs,icase); */
    if( nargs == 1 && ( icase > -1 && icase < NUMCMDS ) )
    {
        if( icase == LIST )
            goto list_menu;
        savstring(string,0,dfnames[0]);
        if( icase == RETURN )
            return(0);
        if( icase == GOBACK )
            return(-1);
        nargs = sscanf(string,"%d %d",&icase,&param);
        if(nargs == 2)
            cmpoptions[icase] = param;
        else
        {
            printf("\nEnter Parameter(%d)__ ",cmpoptions[icase]);
            if( fgets(string,strsize,stdin) == NULL )
            {
                printf("\n");
                exit(1);
            }
            sscanf(string,"%d",&cmpoptions[icase]);
            savstring(string,0,dfnames[0]);
        }
        printf("%s = %d\n",menubuf[icase],cmpoptions[icase]);
        goto enter_command;
      }

    /* Mnemonic Specification */
    icase = -1;
    if( substr(string,"?") > -1 )
        goto list_menu;
    for(i=0; i< NUMCMDS; ++i)
    {
        if( substr(string,mnemonic[i]) > -1 ) 
        {
            icase = i; /* 13 lines of code duplicated below */
            if( icase == LIST )
                goto list_menu;
            savstring(string,0,dfnames[0]);
            if( icase == RETURN )
                return(0);
            if( icase == GOBACK )
                return(-1);
            nargs = sscanf(string,"%s %d",junk,&param);
            if(nargs == 2)
                cmpoptions[icase] = param;
            else
            {
                printf("\nEnter Parameter(%d)__ ",cmpoptions[icase]);
                if( fgets(string,strsize,stdin) == NULL ) 
                {
                    printf("\n");
                    exit(1);
                }
                sscanf(string,"%d",&cmpoptions[icase]);
                savstring(string,0,dfnames[0]);
            }
            printf("%s = %d\n",menubuf[icase],cmpoptions[icase]);
            goto enter_command;
          }
      }
    fprintf(stderr,"Unknown Command: %s",string);
    goto enter_command;

    /* List Out Commands and Status */

    list_menu:

        ttyclrhom();
        printf("Enter All Compute Options by Number or Mnemonic\n");
        for(i=0; i < NUMCMDS; ++i )
        {
            if( i < LIST )
                printf("%2d:%s-%s = %d\n",i,mnemonic[i],menubuf[i],cmpoptions[i]);
          else
                printf("%2d:%s-%.14s",i,mnemonic[i],menubuf[i]);
        }
        printf("\n");
        goto enter_command;
}
