/*
 *  DERIVTIVIS
 *
 *  This subroutine computes intermediate (first and
 *  second) derivative images using separable
 *  convolution operation (with DEANZA DISPLAY options)
 *
 *  Author:     Paul J. Besl    (4-28-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           derivtivs(fltimg,iximg,iyimg,ixyimg,ixximg,iyyimg,tmpimg,nx,ny,
 *                            vec0,vec1,vec2,dwsize,cmpoptions,outoptions,rangimg,string,dfnames)
 *  Modules used:   frowcnvl()      fcolcnvl()      dwfltimg()
 *
 */
 
void derivtivs(fltimg,iximg,iyimg,ixyimg,ixximg,iyyimg,tmpimg,nx,ny,
               vec0,vec1,vec2,dwsize,cmpoptions,outoptions,rangimg,string,dfnames)
float   *fltimg,*iximg,*iyimg,*ixyimg,*ixximg,*iyyimg,*tmpimg;
float   *vec0,*vec1,*vec2;
int     *cmpoptions,*outoptions;
char    *rangimg,*string,*dfnames[];
int     nx,ny,dwsize;
{
    float fmax,fmin;
    int totime,tsecs,tstart();

    totime = tstart();

    /* compute 1st and 2nd and mixed partials
       using separable convolution              */

    /* Ix */

    tsecs = tstart();
    frowcnvl(tmpimg,vec1,fltimg,dwsize,nx,ny); 
    fcolcnvl( iximg,vec0,tmpimg,dwsize,nx,ny);
    tstop(tsecs,"Ix Derivative Convolution");

    /* Ixy */

    tsecs = tstart();
    fcolcnvl(ixyimg,vec1,tmpimg,dwsize,nx,ny);
    tstop(tsecs,"Ixy Derivative Convolution");

    /* Iy */

    tsecs = tstart();
    frowcnvl(tmpimg,vec0,fltimg,dwsize,nx,ny); 
    fcolcnvl( iyimg,vec1,tmpimg,dwsize,nx,ny);
    tstop(tsecs,"Iy Derivative Convolution");

    /* Iyy */

    tsecs = tstart();
    fcolcnvl(iyyimg,vec2,tmpimg,dwsize,nx,ny);
    tstop(tsecs,"Iyy Derivative Convolution");

    /* Ixx */

    tsecs = tstart();
    frowcnvl(tmpimg,vec2,fltimg,dwsize,nx,ny); 
    fcolcnvl(ixximg,vec0,tmpimg,dwsize,nx,ny);
    tstop(tsecs,"Ixx Derivative Convolution");

    /* Display Results as Required */

    dwfltimg( iximg,tmpimg,nx,outoptions,cmpoptions,2,
              rangimg,string,&fmax,&fmin,dfnames);
    dwfltimg( iyimg,tmpimg,nx,outoptions,cmpoptions,3,
              rangimg,string,&fmax,&fmin,dfnames);
    dwfltimg( ixximg,tmpimg,nx,outoptions,cmpoptions,4,
              rangimg,string,&fmax,&fmin,dfnames);
    dwfltimg( iyyimg,tmpimg,nx,outoptions,cmpoptions,5,
              rangimg,string,&fmax,&fmin,dfnames);
    dwfltimg( ixyimg,tmpimg,nx,outoptions,cmpoptions,6,
              rangimg,string,&fmax,&fmin,dfnames);
    tstop(totime,"Derivative Computation Time");
}
