/*
 *  ================================================================
 *  DEPTH MAP SURFACE CHARACTERIZATION AND OBJECT RECOGNITION SYSTEM
 *  ================================================================
 *
 *  SURFACE CHARACTERIZATION MODULE (DMORSYS)
 *
 *      This program accepts an unknown range_image and an object
 *      database as input.  It computes a rough surface characterization
 *      of the range_image first.  Then it computes a refined geometric
 *      structure representation.  This representation is used to match
 *      objects in the object database.
 *      Various output options for file output are available.
 *      Various compute options are also available.
 *
 *  Author:     Paul J. Besl    (2-7-85)
 *
 *  Revised:    Arnold. H. Chiu (8-22-88)
 *  
 *  Call:           dmorsys()
 *  Modules used:   getfildir()     getrifn()   cmpmenu()
 *                  outmenu()       surfchar()
 *
 */

#define DATAFILE        "dmorsys.data"
#define NUMOUTOPTNS     64
#define NUMCMPOPTNS     24
#define STRNGSIZE       128
#define NUMFILDIR       4

main(argc,argv)
int     argc;
char    *argv[];
{
    int     outoptions[NUMOUTOPTNS];    /* output option switches */
    int     cmpoptions[NUMCMPOPTNS];    /* compute options */
    char    string[STRNGSIZE];          /* scratch string */
    char    rangimg[STRNGSIZE];         /* range_image filename */
    int     iret;                       /* return_code */
    char    *names[NUMFILDIR];          /* file and directory names */

    /* Announce Program */

    printf("\nDEPTH MAP OBJECT RECOGNITION SYSTEM\n");
    printf("  Surface Characterization Module\n");
    printf("-----------------------------------\n");
    getfildir(DATAFILE,names,NUMFILDIR);

    /* Prompt for Valid Range_image File Name */

    get_file_name:
        getrifn(rangimg,string,argc,argv,names);

    /* Specify Compute Options */
    get_cmp_optns:
        iret = cmpmenu(cmpoptions,string,STRNGSIZE,names);
        if( iret )
            goto get_file_name;

    /* Specify output options */
    get_out_optns:
        iret = outmenu(outoptions,string,STRNGSIZE,names);
        if( iret )
            goto get_cmp_optns;

    /* Perform surface characterization operations */
    surfchar(cmpoptions,outoptions,rangimg,string,names);

    /* Perform Cleanup Operations */
    /* not implemented yet */

    /* Extract refined geometric structure for matching */
    /* not implemented yet */

    /* Perform matching on object database */
    /* not implemented yet */

    /* Check matching with selected object */
    /* not implemented yet */

    /* Report results */
    /* not implemented yet */
}
