/*
 *  DWFLTIMG
 *
 *     This routine displayS and/or writes float image.
 *     It accepts a pointer to a float image (fltimg)
 *     and converts it to an unsigned char image (imgptr)
 *     in the space provided.  The images are of size
 *     (nsize x nsize) with maximum value fmax and
 *     minimum value fmin.
 *
 *  Author:     Paul J. Besl    (2-26-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           dwfltimg(fltimg,imgptr,nsize,outoptns,cmpoptns,optn,rangimg,string,
 *                           fltmax,fltmin,dfnames)
 *  Modules used:   fmaxmin()       dwuscimg()      savstring()     outmenucmd()
 *
 */

#include <stdio.h>

#define  RETMSG "dwfltimg: Constant Image Ignored for\n"

int dwfltimg(fltimg,imgptr,nsize,outoptns,cmpoptns,optn,rangimg,string,
              fltmax,fltmin,dfnames)
int             nsize,outoptns[],cmpoptns[],optn;
char            *rangimg,*string,*dfnames[];
unsigned char   *imgptr;
float           *fltimg,*fltmax,*fltmin;
{
    int                     tstart();
    char                    *outmenucmd();
    register int            i,nsqr;
    register unsigned char  *ptr;
    register float          scale;
    float                   fmax,fmin;
    int                     tsecs;
    char                    astring[90];

    /* compute maximum and minimum of image */

    tsecs = tstart();
    nsqr = nsize * nsize;
    fmaxmin(fltimg,nsqr,&fmax,&fmin);
    printf("Max= %g | Min= %g | ",fmax,fmin);
    *fltmax = fmax;
    *fltmin = fmin;

    /* determine if image is flagged for output to file or deanza */

    if( outoptns[optn] == 0 )
        return(0);
    if(fmax == fmin)
    {
        printf(RETMSG);
        printf("Image (%s)\n",outmenucmd(optn,1));
        return(-1);
    }

    /* compute scaling factor & convert float image to byte image */

    scale = 255.0/(fmax-fmin);
    ptr = imgptr;
    for(i=0; i<nsqr; ++i)
        *ptr++ = (scale * ( (*fltimg++) - fmin)) + 0.5;

    /* display and/or write unsigned char image */

    dwuscimg(imgptr,nsize,outoptns,cmpoptns,optn,rangimg,string,dfnames);
    sprintf(astring,"Max= %g | Min= %g | File= %s\n",fmax,fmin,string);
    savstring(astring,0,dfnames[1]);
    tstop(tsecs,"Time to Convert and Write Float Image");
}
