/*
 *  FCIPRTI
 *
 *      This subroutine does triangular image-row smoothing
 *      inhibiting smoothing over discontinuities larger
 *      than the given threshold (thres). (inner-product)
 *      (fltptr) points to pixel in input image where output
 *      is to be computed.  This pixel is in the (row)
 *      row of (lastrow + 1) rows.  Ideal Step Edges are 
 *      invariant to this filter.  Impulses are removed.
 *      Other noise (Gaussian, uniform) is smoothed.
 *
 *  Author:     Paul J. Besl    (7-9-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           fciprti(float_pointer,row,last_row,row_incr,
 *                          lhaf,thres)
 *  Modules used:   none
 *
 */

#include <math.h>

float fciprti(fltptr,row,lastrow,rowincr,lhaf,thres)
int     row,lastrow,lhaf; 
float   *fltptr,thres;
{
    register double     sum;
    register float      *ptr1,*ptr2;
    register int        i,wtsum,istop;
    int                 istart;

    /* First, Find Upper Stopping Point */

    sum = 0.0;
    wtsum = 0;
    ptr1 = fltptr;
    ptr2 = ptr1 - rowincr;
    istop = row;
    for(i=lhaf; i>0; --i)
    {
        if( istop == 0 )
            break;
        if( fabs( (*ptr1) - (*ptr2) ) > thres )
            break;
        ptr1 -= rowincr;
        ptr2 -= rowincr;
        sum += i*( *ptr1 );
        wtsum += i;
        istop--; 
    }
    istart = istop;

    /* Next, Compute Lower Stopping Point */

    ptr1 = fltptr;
    ptr2 = ptr1 + rowincr;
    istop = row;
    for(i=lhaf; i>0; --i)
    {
        if( istop == lastrow )
            break;
        if( fabs( (*ptr1) - (*ptr2) ) > thres )
            break;
        ptr1 += rowincr;
        ptr2 += rowincr;
        sum += i*( *ptr1 );
        wtsum += i;
        istop++; 
    }

    /* Remove Impulse-Like Spikes;  WARNING: A PRIORI ASSUMPTION */

    if( istop == istart )
    {
        if( istop == 0 || istop == lastrow )
            return(*fltptr);
        else
        {
            sum = 0.5 * ( *(fltptr+1) + *(fltptr-1) );
            return((float)sum);
        }
    }

    /* Do Final Calculations to Get Weighted Inner Product Sum */

    wtsum += (++lhaf);
    sum += lhaf*(*fltptr);
    sum /= (double)wtsum;
    return((float)sum);
}
