/*
 *  FCOLCNVL
 *
 *      This routine performs column vector convolution for
 *      floating point numbers (32-bits) (New Version)
 *      It is assumed that 'frowcnvl' has already operated
 *      on the image, and that the first and last 'lhaf'
 *      columns have been zeroed out.
 *
 *  Author:     Paul J. Besl    (7-2-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           fcolcnvl(out_image,vector,in_image,l_vector,
 *                           x_size,y_size)
 *  Modules used:   none
 *
 */
 
void fcolcnvl(outimg,vector,inimg,lvec,nx,ny)
int     nx,ny,lvec;
float   *outimg,*inimg,*vector;
{
    register float      *iptr,*vptr,*optr;
    register int        i,j,k;
    register double     sum;
    int                 istop,nstop,lhaf,decr;

    lhaf = lvec/2;
    decr = (lvec-1)*nx;
    nstop = ny - lhaf;

    /* convolve vector with input image creating output */

    for(k=0; k<nx; ++k)
    {
        /* for each non-zero column of input image... */
        iptr = inimg + k;
        optr = outimg + k;
        for(i=0; i<lhaf; ++i)   /* zero out first rows */
        {
            *optr = 0.0;
            optr += nx; 
        }
        for(i=lhaf; i<nstop; ++i)
        {
            sum = 0.0;
            vptr = vector;
            for(j=0; j<lvec; ++j)
            {
                sum += (*vptr++) * (*iptr);
                iptr += nx;
            }
            iptr -= decr;
            *optr = sum;
            optr += nx;
        }
        for(i=nstop; i<ny; ++i)   /* zero out last rows */
        {
            *optr = 0.0;
            optr += nx;
        }
    }
}
