/*
 *  FISLICE
 *
 *      This routine creates closed contours by intersecting
 *      an image with a plane at a particular height.
 *
 *  Author:     Paul J. Besl    (10-7-84, 7-13-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           fislice(float_image_ptr,image_ptr,
 *                          x_size,y_size,f_level)
 *  Modules used:   imgset()
 *
 */

void fislice(fimgptr,imgptr,nxsize,nysize,flevel)
unsigned char   *imgptr;
int             nxsize,nysize;
float           *fimgptr,flevel;
{
    int                     i,nsqr,nstop,np1;
    register float          *p1,*p2,*p3;
    float                   num1,num2;
    register unsigned char  *outptr;

    /* initialize some values */

    np1=nxsize+1;
    nsqr=nxsize*nysize;
    nstop=nsqr-(2*np1);

    /* set up output array memory pointer */

    imgset(imgptr,0,np1,1);
    imgset(imgptr+nstop+np1,0,np1,1);
    outptr = imgptr + np1;

    /* initialize window pointers */
    p1=fimgptr+np1;
    p2=p1-nxsize;
    p3=p1+nxsize;

/*     p2         Layout of pointers to image
  p1-- p1 p1++
       p3
 */

    /* process input image array generating output image array */
    for(i=0;i<nstop;++i)
    {
        num1 = *p1;
        if( (*p2) > num1 )
            num2= *p2;
        else
        {
            num2= num1;
            num1= *p2;
        }
        if( num1 <= flevel && flevel <= num2 )
        {
            if( num1 != num2 )
                *outptr++ = 255;
            else
                *outptr++ = 0;
        }
        else
        {
            if( (*p3) > num1 )
                num2= *p3;
            else
            {
                num2=num1;
                num1= *p3;
            }
            if( num1 <= flevel && flevel <= num2)
            {
                if( num1 != num2 )
                    *outptr++ = 255;
                else
                    *outptr++ = 0;
            }
            else
            {
                --p1;
                if( (*p1) >num1 )
                    num2= *p1;
                else
                {
                    num2=num1;
                    num1= *p1;
                }
                ++p1;
                if( num1 <= flevel && flevel <= num2)
                {
                    if( num1 != num2 )
                        *outptr++ = 255;
                    else
                        *outptr++ = 0;
                   }
                else
                {
                    ++p1;
                    if( (*p1) >num1 )
                        num2= *p1;
                    else
                    {
                        num2=num1;
                        num1= *p1;
                    }
                    --p1;
                    if( num1 <= flevel && flevel <= num2)
                    {
                        if( num1 != num2 )
                            *outptr++ = 255;
                        else
                            *outptr++ = 0;
                    }
                    else
                        *outptr++ = 0;  /* no crossing */
                }
            }
        }
        ++p1;
        ++p2;
        ++p3;
    }
}
