/*
 *  FNMSUPRES2
 *
 *      This routine performs non-maxima suppression in
 *      float image (w/o using gradient direction as in
 *      Canny operator).
 *      ptr1 image is converted to become ptr2 image
 *      Edge pixels are labeled as such using ranking information.
 *      Neighbors in 3x3 window are numbered as follows:
 *      (Parentheses indicate next pixel numbering.)
 * 
 *           0(-) | 1(0) | 2(1)     p1 used for this row
 *          ------+------+------
 *           7(-) | *(7) | 3(*)     p2 used for this row
 *          ------+------+------
 *           6(-) | 5(6) | 4(5)     p3 used for this row
 *
 *  Author:     Paul J. Besl    (1-25-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           fnmsupres2(pointer1,pointer2,n_size,f_max,
 *                             low_thresh,num_bigger,diff_thresh)
 *  Modules used:   blkbdrci()
 *
 */

#define NUMEQUAL    5

void fnmsupres2(ptr1,ptr2,nsize,fmax,lowth,numbigger,difth)
float           *ptr1;          /* input image */
unsigned char   *ptr2;          /* output image */
int             lowth;          /* threshold percentage */
int             difth;          /* threshold percentage */
int             nsize;          /* image size */
float           fmax;           /* max image value */
int             numbigger;      /* number of neighbor pixels bigger */
{
    register int            i,j,k;
    register float          *p1,*p2,*p3;
    register unsigned char  *po;
    int                     n1,ngreater,nequal,tsecs;
    int                     tstart();
    float                   lowval,difval,sfactr,pixel,winmax;
    float                   nabor0, nabor1, nabor2, nabor3;
    float                   nabor4, nabor5, nabor6, nabor7;

    /* Blacken Border of Output Image */

    tsecs = tstart();
    blkbdrci(ptr2,nsize,nsize,3);

    /* Scale Thresholds as Percentage of Maximum */

    sfactr = 255.0/fmax;
    lowval = lowth*fmax/100.0;
    difval = difth*fmax/100.0;
    /*printf("lowth=%d numbigger= %d difth=%d\n",lowth,numbigger,difth);*/

    /* Initialize Each Row; Process By Ranking */

    n1 = nsize - 1;
    for(i=1; i<n1; ++i)   /* for each row in image */
    {
        p2 = ptr1 + i*nsize;
        p1 = p2 - nsize;
        p3 = p2 + nsize;
        po = ptr2 + i*nsize + 1;
        nabor0 = *p1++;     nabor7 = *p2++;
        nabor6 = *p3++;     nabor1 = *p1;
        pixel  = *p2;       nabor5 = *p3;
        for(j=1; j<n1; ++j)  /* for each pixel in row */
        {
            /* Rank Pixel Against its Neighbors and Compute Max/Min */

            nabor2 = *++p1;     nabor3 = *++p2;     nabor4 = *++p3;
            if( pixel <= lowval )
            {
                *po = 0;
                goto update_window;
            }
            ngreater = 0;
            nequal = 0;
            winmax = pixel;
            if( pixel < nabor0 )
                ++ngreater;
            else if( pixel == nabor0 )
                nequal++;
            if( nabor0 > winmax )
                winmax = nabor0;
            if( pixel < nabor1 )
                ++ngreater;
            else if( pixel == nabor1 )
                nequal++;
            if( nabor1 > winmax )
                winmax = nabor1;
            if( pixel < nabor2 )
                ++ngreater;
            else if( pixel == nabor2 )
                nequal++;
            if( nabor2 > winmax )
                winmax = nabor2;
            if( pixel < nabor3 )
                ++ngreater;
            else if( pixel == nabor3 )
                nequal++;
            if( nabor3 > winmax )
                winmax = nabor3;
            if( pixel < nabor4 )
                ++ngreater;
            else if( pixel == nabor4 )
                nequal++;
            if( nabor4 > winmax )
                winmax = nabor4;
            if( pixel < nabor5 )
                ++ngreater;
            else if( pixel == nabor5 )
                nequal++;
            if( nabor5 > winmax )
                winmax = nabor5;
            if( pixel < nabor6 )
                ++ngreater;
            else if( pixel == nabor6 )
                nequal++;
            if( nabor6 > winmax )
                winmax = nabor6;
            if( pixel < nabor7 )
                ++ngreater;
            else if( pixel == nabor7 )
                nequal++;
            if( nabor7 > winmax )
                winmax = nabor7;

            /* process window to create output pixel */

            if(   nequal > NUMEQUAL  )
            {
                *po = 0;
                goto update_window;
            }
            if( winmax - pixel >= difval ) 
            {
                *po = 0;
                goto update_window;
            }
            if( ngreater < numbigger ) 
                *po = (int)(pixel*sfactr+0.5);
            else
                *po = 0;

            /* update window and output image for next pixel */

        update_window:
            nabor0 =  nabor1;   nabor7 =  pixel;
            nabor6 =  nabor5;   nabor1 =  nabor2;
            pixel  =  nabor3;   nabor5 =  nabor4;
            ++po;
        }
    }
    tstop(tsecs,"Non_Maxima_Suppression_Operation");
}
