/*
 *  FRIPRTI
 *
 *      This subroutine does triangular image-row smoothing
 *      inhibiting smoothing over discontinuities larger
 *      than the given threshold (thres). (inner product)
 *      (fltptr) points to pixel in input image where output
 *      is to be computed.  This pixel is in the (column)
 *      column of (lastcol + 1) columns.  Ideal Step Edges are
 *      invariant to this filter.  Impulses are removed.
 *      Other noise (Gaussian, uniform) is smoothed.
 *
 *  Author:     Paul J. Besl    (7-9-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           friprti(float_pointer,column,last_column,
 *                          lhaf,thres)
 *  Modules used:   none
 *
 */
#include <math.h>

float friprti(fltptr,column,lastcol,lhaf,thres)
int     column,lastcol,lhaf; 
float   *fltptr,thres;
{
    register double     sum;
    register float      *ptr1,*ptr2;
    register int        i,wtsum,istop;
    int                 istart;

    /* First, Find Left Stopping Point */

    sum = 0.0;
    wtsum = 0;
    ptr1 = fltptr;
    ptr2 = ptr1 - 1;
    istop = column;
    for(i=lhaf; i>0; --i)
    {
        if( istop == 0 )
            break;
        if( fabs( (*ptr1--) - (*ptr2--) ) > thres )
            break;
        sum += i*( *ptr1 );
        wtsum += i;
        istop--; 
    }
    istart = istop;

    /* Next, Find Right Stopping Point */

    ptr1 = fltptr;
    ptr2 = ptr1 + 1;
    istop = column;
    for(i=lhaf; i>0; --i)
    {
        if( istop == lastcol )
            break;
        if( fabs( (*ptr1++) - (*ptr2++) ) > thres )
            break;
        sum += i*( *ptr1 );
        wtsum += i;
        istop++; 
    }

    /* Remove Impulse-Like Spikes;  WARNING: A PRIORI ASSUMPTION */

    if( istop == istart )
    {
        if( istop == 0 || istop == lastcol )
            return(*fltptr);
        else
        {
            sum = 0.5 * ( *(fltptr+1) + *(fltptr-1) );
            return((float)sum);
        }
    }

    /* Do Final Calculations to Get Weighted Inner Product Value */

    wtsum += (++lhaf);
    sum += lhaf*(*fltptr);
    sum /= (double)wtsum;
    return((float)sum);
}
