/*
 *  FSMOOTHER2
 *
 *      This routine performs image smoothing for
 *      creating floating point images.
 *      32-bit float image is smoothed in-place
 *      "nimg" is the square image size.
 *      "lvec" is the smoothing window size.
 *      "opt" is the option flag for different smoothing
 *          opt = 0  <-->  Triangle Smoother
 *          opt = 1  <-->  Gaussian Smoother (1% threshold)
 *          opt = 2  <-->  Gaussian Smoother (5% threshold)
 *          opt = 3  <-->  Gaussian Smoother (10% threshold)
 *          opt = 4  <-->  Gaussian Smoother (15% threshold)
 *          opt = 5  <-->  Gaussian Smoother (20% threshold)
 *          opt = 6  <-->  Gaussian Smoother (25% threshold)
 *          opt = 7  <-->  Gaussian Smoother (30% threshold)
 *          opt = 8  <-->  Gaussian Smoother (40% threshold)
 *          opt = 9  <-->  Gaussian Smoother (50% threshold)
 *          For opt >= 10: interpret opt as a threshold for fsmtinhb
 *
 *  Author:     Paul J. Besl    (9-29-84)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           fsmoother2(float_img,x_size,y_size,lvec,opt,temp_image)
 *  Modules used:   trivectr()      gsmvectr()      frowcnvl()
 *                  fcolcnvl()      fsmtinhb()
 */

int fsmoother2(fltimg,nx,ny,lvec,opt,tmpimg)
int     nx,ny,lvec,opt;
float   *fltimg,*tmpimg;
{
    float   vecptr[49];
    float   sigma;

    if( lvec == 1 )
        return(0);
    if( lvec > 49 )
    {
        printf("fsmoother: lvec= %d\n",lvec);
        exit(1);
    }

    /* build smoothing vector */

    if( opt > 9 )  goto special_smooth;
    switch( opt )
    {
        case  0:
                trivectr(lvec,vecptr);
                break;
        case  1:
                gsmvectr(lvec,0.01,vecptr,&sigma);
                break;
        case  2:
                gsmvectr(lvec,0.05,vecptr,&sigma);
                break;
        case  3:
                gsmvectr(lvec,0.10,vecptr,&sigma);
                break;
        case  4:
                gsmvectr(lvec,0.15,vecptr,&sigma);
                break;
        case  5:
                gsmvectr(lvec,0.20,vecptr,&sigma);
                break;
        case  6:
                gsmvectr(lvec,0.25,vecptr,&sigma);
                break;
        case  7:
                gsmvectr(lvec,0.30,vecptr,&sigma);
                break;
        case  8:
                gsmvectr(lvec,0.40,vecptr,&sigma);
                break;
        case  9:
                gsmvectr(lvec,0.50,vecptr,&sigma);
                break;
        default:
                printf("Bad Smoothing Option=%d\n",opt);
                trivectr(lvec,vecptr);
                break;
    }

    /* perform row and column convolutions for smoothing */
    frowcnvl(tmpimg,vecptr,fltimg,lvec,nx,ny);
    fcolcnvl(fltimg,vecptr,tmpimg,lvec,nx,ny);
    return(0);

    /* perform special smoothing */

special_smooth:
    fsmtinhb(tmpimg,fltimg,nx,ny,lvec,(float)opt,'r');
    fsmtinhb(fltimg,tmpimg,nx,ny,lvec,(float)opt,'c');
}
