/*
 *  FSMTINHB
 *
 *      This routine performs row/column convolution for
 *      floating point numbers (32-bits) using triangular
 *      smoothing window weights.
 *      New Version for Inhibiting Smoothing over
 *      Zeroth-Order Discontinuities.
 *      The nx X ny input image (inimg) smoothed to create
 *      output image (outimg).
 *          lvec = maximum length of triangular smoothing window.
 *          thres = threshold for smoothing inhibition
 *          rowcol = 'c' for column smoothing.
 *          rowcol = 'r' for row smoothing.
 *
 *  Author:     Paul J. Besl    (7-8-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           fsmtinhb(out_image,in_image,x_size,y_size,
 *                           l_vector,threshold,row_column)
 *  Modules used:   friprti()       fciprti()
 *
 */

void fsmtinhb(outimg,inimg,nx,ny,lvec,thres,rowcol)
float   *outimg,*inimg,thres;
int     nx,ny,lvec;
char    rowcol;
{
    float   *iptr,*optr;
    float   friprti(),fciprti();
    int     i,ntot,lhaf,nx1,ny1;

    /* apply either row or column smoothing */

    nx1 = nx - 1;
    ny1 = ny - 1;
    ntot = nx*ny;
    lhaf = lvec/2; 
    optr = outimg;
    iptr = inimg;
    if( rowcol == 'r' ) 
    {
        for(i=0; i<ntot; ++i)
            *optr++ = friprti(iptr++,i%nx,nx1,lhaf,thres);
    }
    else
    {
        for(i=0; i<ntot; ++i)
            *optr++ = fciprti(iptr++,i/nx,ny1,nx,lhaf,thres);
    }
}
