/*
 *  GFOPEN
 *
 *      Guaranteed file open routine, which opens file, returning a
 *      pointer to the file, or exits.  (See options below.)
 *      User is allowed NUM_TRIALS times to open a valid file.
 *
 *  Author:     Paul J. Besl    (7-26-84)
 *
 *  Revised:    Arnold. H. Chiu (7-13-88)
 *  
 *  Call:           gfopen(file_name,access_code)
 *  Modules used:   none
 *
 */
 
/* skip listing <stdio.h> */
/* #nolist */
#include <stdio.h>
/* #list */
  
#define NUM_TRIALS      10          /* number of I/O trials         */
#define BUFSIZE         256         /* input buffer size            */
#define NO_MORE_TRIALS  0           /* flag for # of trials check   */

FILE *gfopen(filnam,code)
char    *filnam,*code;
{
    FILE    *filptr,*fopen();
    int     ntimes = NUM_TRIALS;
    char    string[BUFSIZE];

    /* open file **/
    filptr = fopen(filnam,code);
    ntimes--;
    strcpy(string,filnam);

    /* retry if unsuccessful first time and prompt user for:
       1. return [CR] to abort
       2. system command preceded with '!'
       3. new file name                         */

    while (filptr == NULL)
    {
        if( ntimes == NO_MORE_TRIALS )
            exit(1);
        fprintf(stderr,"fopen: File Open Error for %s\n",string);

    try_again:

        fprintf(stderr,"Enter New Filename__ ");
        if( gets(string) == NULL )
            exit(1);
        if( string[0] == '!' )
        {
            system(&string[1]);
            goto try_again;
        }
        filptr = fopen(string,code);
        ntimes--;
    }
    return(filptr);
}
