/*
 *  GSMVECTR
 *
 *      This routine computes the vector needed for the
 *      separable convolution gaussian smoothing operation.
 *      The only input parameters are the window size (wsize)
 *      and the threshold for the smallest coefficient.
 *
 *  Author:     Paul J. Besl    (2-25-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           gsmvectr(w_size,threshold,vec_gauss,sigma)
 *  Modules used:   none
 *
 */

#include <stdio.h>
#include <math.h>

void gsmvectr(wsize,threshold,vecgauss,sigma)
int     wsize;
float   threshold;
float   *vecgauss,*sigma;
{
    register int    i,whalf;   
    double          temp,cfactr,sigma2;
    float           sum;

    /* initialize values */

    if( threshold < 0.0 || threshold > 1.0 ) 
    {
        printf("gsmvectr: Bad Threshold=%g\n",threshold);
        exit(1);
    }
    whalf = wsize/2;
    temp  = -2.0*log(threshold);
    sigma2 = (float)(whalf*whalf)/temp;
    for(i=1,temp=0.0; i<=whalf; ++i)
        temp += exp( -0.5*(float)(i*i)/(sigma2) );
    cfactr = 1.0 + 2.0*temp;
    cfactr = 1.0/cfactr;

    /* fill vector with needed data */

    sum = 0.0;
    for(i=0; i<wsize; ++i)
    {
        temp = i - whalf;
        vecgauss[i] = cfactr*exp( -0.5*temp*temp/sigma2 );
        sum += vecgauss[i];
      /*printf("  vecgauss[%d]=%g\n",i,vecgauss[i]);*/
    }
    *sigma = sqrt(sigma2);
    temp = vecgauss[whalf]/vecgauss[0];
  /*printf("Window-Size=%d | Sigma=%g | Vector_Sum=%g | Ratio=%g\n",
            wsize,*sigma,sum,temp);*/
}
