/*
 *  OUTMENUCMD
 *
 *  This subroutine for menu commands returns the pointer
 *  to the two letter mnemonic or the word description.
 *    flag = 0 <--> mnemonic
 *    flag = 1 <--> word string
 *
 *  Author:     Paul J. Besl    (2-13-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           outmenucmd(command,flag)
 *  Modules used:   none
 *
 */
 
#define NUMCMDS   56
#define MAXMENU   22

char *outmenucmd(cmd,flag)
int cmd,flag;
{
    /* 2-char mnemonic commands */
    static char mnemonic[NUMCMDS][4] =
    {   "oi","si","ix","iy","xx","yy","xy","ai","i2","lp",
        "zx","zy","z1","z2","z3","a2","zl","a1","o1","cp",
        "qv","eq","cc","ac","fe","ef","sg","eg","ph","pg",
        "ep","hk","ah","sh","h0","eh","ak","sk","k0","ek",
        "dp","ed","k1","k2","s1","s2","p1","p2","re","se",
        "rs","au","ad","ls","co","gb" };

    /* Menu List */
    static char menubuf[NUMCMDS][MAXMENU] =
    {   "Original Image      ", "Smoothed Image      ",
        "Ix Partial Deriv.   ", "Iy Partial Deriv.   ",
        "Ixx Partial Deriv.  ", "Iyy Partial Deriv.  ",
        "Ixy Partial Deriv.  ", "Average Image       ",
        "Squared Image       ", "Laplacian Image     ",   /* 10 */
        "Zeros(Ix)           ", "Zeros(Iy)           ",
        "Zeros(Ixx)          ", "Zeros(Iyy)          ",
        "Zeros(Ixy)          ", "AND0s(Ixx,Iyy,Ixy)  ",
        "Zeros of Laplacian  ", "AND0s(Ix,Iy)        ",
        "OR0s(Ix,Iy)         ", "Critical Pts.(NDCP) ",   /* 20 */
        "Quadratic Var.(QV)  ", "Edges(QV)           ",
        "COS(Coord. Angle)   ", "|COS(Coord. Angle)| ",
        "Fit Error           ", "Edges(Fit Error)    ",
        "SQRT(g)             ", "Edges(SQRT(g))      ",
        "Phase Image         ", "Phase Gradient      ",   /* 30 */
        "Edges(Phase)        ", "HK-Sign Map         ",
        "|H|                 ", "sgn(H)              ",
        "Zeros(H)            ", "Edges(H)            ",
        "|K|                 ", "sgn(K)              ",
        "Zeros(K)            ", "Edges(K)            ",   /* 40 */
        "|k1-k2|             ", "Edges(|k1-k2|)      ",
        "|Princ. Curv. 1|    ", "|Princ. Curv. 2|    ",
        "sgn(Princ. Curv. 1) ", "sgn(Princ. Curv. 2) ",
        "Princ. Dir. 1       ", "Princ. Dir. 2       ",
        "Roof Edges          ", "Step Edges          ",   /* 50 */
        "Roof&Step Edges     ", "AllSwitches Up(on)  ",
        "AllSwtchs Down(off) ", "List_Options        ",
        "Continue            ", "Go_Back             " };

    if( cmd < 0 || cmd >= NUMCMDS )
    {
        printf("_menucmd: Bad Command Option=%d\n",cmd);
        exit(1);
    }
    if( flag < 0 || flag > 1 )
    {
        printf("_menucmd: Bad Flag Option=%d\n",cmd);
        exit(1);
    }
    switch( flag )
    {
        case 0: return(mnemonic[cmd]);
        case 1: return(menubuf[cmd]);
    }
}
