/*
 *  PHASGRAD
 *
 *      This subroutine computes the phase image of the
 *      gradient vector at each point and encodes the angle
 *      as a gray level.
 *
 *  Author:     Paul J. Besl    (4-28-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           phaseimg(phase_image,mag_image,ix_image,iy_image,
 *                           x_size,y_size)
 *  Modules used:   none
 *
 */

#include <math.h>

void phaseimg(phsimg,magimg,iximg,iyimg,nxsize,nysize)
float   *phsimg,*magimg,*iximg,*iyimg;
int     nxsize,nysize;
{
    float               *aptr,*xptr,*yptr,*mptr;
    register double     x,y,a;
    register int        i,ntot;
    double              pio2,pi = 3.14159265358979;

    /* Setup for Computation Loop */

    ntot = nxsize * nysize;
    aptr = phsimg;
    xptr = iximg;
    yptr = iyimg;
    mptr = magimg;
    pio2 = pi/2.0;

    /* Compute Angle at Each Point */

    for(i=0; i<ntot; ++i)
    {
        x = *xptr++;
        y = *yptr++;
        if( x == 0.0 )
        {
            if( y == 0.0 )
                a = 0.0;  /* x = y = 0 */
            else  /* y != 0.0 */
            {
                if( y > 0.0 )
                    a = pio2;
                else
                    a = -pio2;
            }
        } 
        else
            a = atan2(y,x);     /* x != 0 */
        *aptr++ = a;
        *mptr++ = fabs(x) + fabs(y);
    }
}
