/*
 *  PHASGRAD
 *
 *      This subroutine computes the phase image gradient
 *      vector magnitude at each point and encodes the
 *      angle as a gray level.
 *
 *  Author:     Paul J. Besl    (4-28-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           phasgrad(phase_grad,phase_image,x_size,y_size)
 *  Modules used:   none
 *
 */

#include <math.h>

void phasgrad(phsgrd,phsimg,nxsize,nysize)
float   *phsimg,*phsgrd;
int     nxsize,nysize;
{
    register double     h1,h2,h3,v1,v2,v3;
    float               *p1,*p2,*p3,sumh,sumv;
    register int        i,ntot;
    double              twopi,mpi,pi = 3.14159265358979;
    int                 tsecs,tstart();

    /* Setup for Computation Loop */

    tsecs = tstart();
    ntot = nxsize*nysize;
    p1 = phsimg;    p2 = p1 + nxsize;   p3 = p2 + nxsize;
    p1++;           p2++;               p3++;
    twopi = pi*2.0;
    mpi = -pi;
    ntot = ntot - 2*(nxsize + 1);
    phsgrd += (nxsize+1);

    /* Compute Angle at Each Point */

    for(i=0; i<ntot; ++i)
    {
        p1--;   p2--;   p3--;
        v1 = (*p1 - *p3);
        h1 = -(*p1);    h2 = -(*p2);    h3 = -(*p3);
        p1++;   p2++;   p3++;
        v2 = (*p1 - *p3);
        p1++;   p2++;   p3++;
        v3 = (*p1 - *p3);
        h1 += (*p1);    h2 += (*p2);    h3 += (*p3);
        if( h1 >= pi )
            h1 -= twopi;
        if( h1 < mpi )
            h1 += twopi;
        if( h2 >= pi )
            h2 -= twopi;
        if( h2 < mpi )
            h2 += twopi;
        if( h3 >= pi )
            h3 -= twopi;
        if( h3 < mpi )
            h3 += twopi;
        if( v1 >= pi )
            v1 -= twopi;
        if( v1 < mpi )
            v1 += twopi;
        if( v2 >= pi )
            v2 -= twopi;
        if( v2 < mpi )
            v2 += twopi;
        if( v3 >= pi )
            v3 -= twopi;
        if( v3 < mpi )
            v3 += twopi;
        sumh = fabs(h1) + fabs(h2) + fabs(h3);
        sumv = fabs(v1) + fabs(v2) + fabs(v3);
        *phsgrd++ = sumh + sumv;
    }
    tstop(tsecs,"Time to Compute Phase Gradient");
}
