/*
 *  PRINCDIR
 *
 *      This routine computes principal directions using
 *      quadratic solution equation.  (More stable methods exist)
 *
 *  Author:     Paul J. Besl    (2-21-85)
 *
 *  Revised:    Arnold. H. Chiu (8-22-88)
 *  
 *  Call:           princdir(pifact,piover2,E,F,G,L,M,N,prdir1,prdir2)
 *  Modules used:   none
 *
 */

#include <math.h>

void princdir(pifact,piover2,E,F,G,L,M,N,prdir1,prdir2)
float   pifact,piover2;
float   E,F,G,L,M,N;
int     *prdir1,*prdir2;
{
    register double     A,B,C,D,slope1,slope2;

    A =  E*M - F*L;
    B = (E*N - G*L)/2.0;
    C =  F*N - G*M;
    D = sqrt( B*B - A*C );
    if( C != 0 )
    {
        slope1 =  ( D - B ) / C;
        slope2 = -( D + B ) / C;
        *prdir1 = (int)pifact*(atan(slope1)+piover2);
        *prdir2 = (int)pifact*(atan(slope2)+piover2);
    }
    else /* C = 0 */
    {
        if( A == 0 )
        {
            *prdir1 = 128;
            *prdir2 = 255;
        }
        else /* A != 0 */
        {
            slope1 = -A/(2.0*B);
            *prdir1 = (int)pifact*(atan(slope1)+piover2);
            *prdir2 = 255;
        }
    }
}
