/*
 *  PRINCURV
 *
 *      This routine computes the two principal curvatures
 *      from the mean and Gaussian curvature.
 *      k1 has the larger magnitude of the two curvatures.
 *      (k1ptr) and (k2ptr) point to absolute value images.
 *      (sgnk1) and (sgnk2) point to sign function images.
 *      (flag) zero indicates sgn images not required.
 *
 *  Author:     Paul J. Besl    (7-3-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           princurv(Hptr,Kptr,k1ptr,k2ptr,
 *                           sgnk1,sgnk2,nx,ny,epsilon,flag)
 *  Modules used:   none
 *
 */

#include <math.h>

int princurv(Hptr,Kptr,k1ptr,k2ptr,sgnk1,sgnk2,nx,ny,epsilon,flag)
float           *Hptr,*Kptr,*k1ptr,*k2ptr,epsilon;
unsigned char   *sgnk1,*sgnk2;
int             nx,ny,flag; 
{
    register float          *k1p,*k2p,*H,*K;
    register unsigned char  *sk1,*sk2;
    float                   negepsilon = -1.0e-5;
    int                     i,ntot,negcount;
    double                  discr;

    /* Initialize Everything */

    negcount = 0;
    ntot = nx * ny;
    H = Hptr;
    K = Kptr;
    k1p = k1ptr;
    k2p = k2ptr;
    sk1 = sgnk1;
    sk2 = sgnk2;

    /* Loop over all Pixels Computing Magnitude and Sign */

    for(i=0; i<ntot; ++i)
    {
        discr = (*H) * (*H) - (*K++);
        if( discr < negepsilon )
            negcount++;
        discr = sqrt( discr );
        if( (*H) >= 0.0 )
        {
            *k1p = (*H) + discr;
            *k2p = (*H++) - discr;
        }
        else
        {
            *k1p = (*H) - discr;
            *k2p = (*H++) + discr;
        }
        if( flag ) /* if sign images are required */
        {
            if( fabs( (*k1p) ) < epsilon )
                *sk1++ = 128;
            else if( (*k1p) > 0.0 )
                    *sk1++ = 255;
                 else
                    *sk1++ = 0;
            if( fabs( (*k2p) ) < epsilon )
                    *sk2++ = 128;
            else if( (*k2p) > 0.0 )
                    *sk2++ = 255;
                 else
                    *sk2++ = 0;
        }
        *k1p++ = fabs( *k1p );
        *k2p++ = fabs( *k2p );
    }
    return(negcount);
}
