/*
 *  QSFVECTRS
 *
 *      This routine compute the vectors needed for the
 *      separable convolution least squares quadratic surface
 *      fit operation.
 *      The only input parameter is the window size (wsize).
 *
 *  Author:     Paul J. Besl    (2-19-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           qsfvectrs(w_size,vec_const,vec1_stdf,
 *                            vec2_nddf,P,Q,R)
 *  Modules used:   none
 *
 */

#include <stdio.h>
#include <math.h>

void qsfvectrs(wsize,vecconst,vec1stdf,vec2nddf,P,Q,R)
int     wsize;
float   *vecconst,*vec1stdf,*vec2nddf,*P,*Q,*R;
{
    register int    i;   
    int             w21,w33,wsumsqr,whalf;
    float           cofconst,cof1stdf,cof2nddf,wftmp;

    /* initialize window values */
    whalf = wsize/2;
    w21 =whalf*whalf + whalf;
    wsumsqr=(w21*wsize)/6;
    w33 = (3*whalf*whalf + 3*whalf - 1);

    /* set up the differential operator coefficient arrays */

    cofconst = wsize;
    cof1stdf = 2*wsumsqr;
    wftmp = (18*wsumsqr*w33/5)+(w21*w21*wsize)-(12*w21*wsumsqr);
    cof2nddf = (wftmp/6.0);
    *P = wftmp/36.0;
    *Q = cof1stdf;
    *R = cofconst;
    *P = (*P)/(*R);
    *Q  = (*Q)/(*R);
    for(i=0; i<wsize; ++i)
    {
        vecconst[i] = 1.0/cofconst;
        vec1stdf[i] = (i - whalf)/cof1stdf;
        vec2nddf[i] = (3.0*(i-whalf)*(i-whalf) - w21)/cof2nddf;
      /*printf("  vecconst[%d]=%g",i,vecconst[i]);
        printf("  vec1stdf[%d]=%g",i,vec1stdf[i]);
        printf("  vec2nddf[%d]=%g\n",i,vec2nddf[i]);*/
    }
}
