/*
 *  REFINEDGE
 *
 *      This subroutine refines the output of the non-maxima
 *      suppression operator to create one pixel wide
 *      8-connected edges, using precomputed masks.
 *
 *  Author:     Paul J. Besl    (4-29-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           refinedge(image_ptr,output_ptr,x_size,y_size,
 *                            options)
 *  Modules used:   map3wndw()
 *
 */

void refinedge(imgptr,outptr,nx,ny,opt)
unsigned char   *imgptr,*outptr;
int             nx,ny,opt;
{
    int                     mask = 0x0010;
    register int            sum,val;
    register unsigned char  *p1,*p2,*p3,*po;
    int                     ntot,i,count;
    int                     map3wndw();

    /* Initialize Values and Pointers */

    ntot = nx*ny;

    count = nx+1;
    po = outptr;
    sum = ntot - count;
    p1 = outptr + sum;
    for(i=0; i<count; ++i)
        *po++ = *p1++ = 0;

    ntot = ntot - 2*(nx+1);
    p1 = imgptr;    p2 = p1 + nx;   p3 = p2 + nx;
    p1++;           p2++;           p3++;
    po = outptr + nx + 1;

    /* For Each Pixel in the Output Image */

    for(i=0; i<ntot; ++i)
    {
        p1--;   p2--;   p3--;
        val = 0;
        count = 0;
        sum = 0;
        if( *p1 )
        {
            sum++;
            val += *p1;
            count++;
        }
        if( *p2 )
        {
            sum += 8;
            val += *p2;
            count++;
        }
        if( *p3 )
        {
            sum += 64;
            val += *p3;
            count++;
        }
        p1++;   p2++;   p3++;
        if( *p1 )
        {
            sum += 2;
            val += *p1;
            count++;
        }
        if( *p2 )
        {
            sum += 16;
            val += *p2;
            count++;
        }
        if( *p3 )
        {
            sum += 128;
            val += *p3;
            count++;
        }
        p1++;   p2++;   p3++;
        if( *p1 )
        {
            sum += 4;
            val += *p1;
            count++;
        }
        if( *p2 )
        {
            sum += 32;
            val += *p2;
            count++;
        }
        if( *p3 )
        {
            sum += 256;
            val += *p3;
            count++;
        }
        if( sum != 0 )
        {
            /*printf("i= %d premap= %d val= %d count= %d\n",i,sum,val,count);*/
            if( opt == 0 )
                sum = map3wndw( sum );
            else
                sum = map3wndwf( sum );
            val /= count;
            /*printf("postmap= %d postval= %d\n",sum,val);*/
        }
        sum &= mask;
        if( sum != 0 )
            *po++ = val;
        else
            *po++ = 0;
    }
}
