/*
 *  SGNFLTIMG
 *
 *      This routine computes signum function of
 *      floating point image.
 *      Output image has 255 (white) for positive values of image.
 *      Output image has 128 (gray)  for zero values of image.
 *      Output image has 0   (black) for negative values of image.
 *      Zero is determined to within (epsilon) tolerance.
 *
 *  Author:     Paul J. Besl    (7-1-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           sgnfltimg(float_image,image_pointer,
 *                            x_size,y_size,epsilon,num_pos,
 *                            num_neg,num_zero,S_plus,S_minus)
 *  Modules used:   none
 *
 */

#include <math.h>

#define PLUS 255
#define ZERO 128
#define MINUS  0

void sgnfltimg(fltimg,imgptr,nx,ny,epsilon,npos,nneg,nzero,Splus,Sminus)
float           *fltimg,epsilon,*npos,*nneg,*nzero,*Splus,*Sminus;
unsigned char   *imgptr;
int             nx,ny;
{
    register int            i,ntot;
    register unsigned char  *outptr;
    register float          *fptr,eps;
    float                   sumplus,suminus;
    int                     np,nn,n0;

    ntot = nx*ny;
    eps = epsilon;
    np = nn = n0 = 0;
    sumplus = suminus = 0.0;
    outptr = imgptr;
    fptr = fltimg;
    for(i=0;i<ntot;++i)
    {
        if( fabs(*fptr) <= eps ) 
        {
            *outptr++ = ZERO;
            ++n0;
        }
        else
        {
            if( *fptr >  0.0) 
            {
                *outptr++ = PLUS;
                ++np;
                sumplus += *fptr;
            }
            else
            {
                *outptr++ = MINUS;
                ++nn;
                suminus -= *fptr;
            }
        }
        fptr++;
    }
    *Splus = sumplus;
    *Sminus = suminus;
    *npos = (float)np;
    *nneg = (float)nn;
    *nzero = (float)n0;
}
