/*
 *  SURFACE CURVATURE SUBROUTINE (with DEANZA DISPLAY option)
 *  
 *      This routine reads a (nsize x nsize) square image
 *      pointed to by imgptr and computes a floating point
 *      array (nsize x nsize) which contains the computed
 *      Gaussian curvature Values at each point. 
 *      This version stores intermediate derivative images 
 *      computed using separable convolution operation.
 *
 *  Author:     Paul J. Besl    (2-19-85)
 *
 *  Revised:    Arnold. H. Chiu (8-12-88)
 *  
 *  Call:           surfchar(cmp_options,out_options,range_image,
 *                           string,default_filenames)
 *  Modules used:   addnoise()      fsmoother2()    qsfvectrs()
 *                  fnmsupres2()    okedges2()      fwindopr()
 *                  refinedge()     derivtivs()     phaseimg()
 *                  phasgrad()      fislice()       surfcolr()
 *                  printout()      savstring()     princurv()
 *                  imgand3()       imgor()         imgand()
 *                  imgnot()        blkbdrci()      substr() 
 *                  matsub()        matcpy()        matadd()
 *                  getimage()      floatimage()    fabsimg()
 *                  sgnfltimg()     dwfltimg()      dwuscimg()
 *                  ldcolr()        ds()            dslut0()
 */

#include <stdio.h>
#include <math.h>

#define LAPLTHRS    0.1

surfchar(cmpoptions,outoptions,rangimg,string,dfnames)
int     *cmpoptions,*outoptions;
char    *rangimg,*string,*dfnames[];
{
    register int    i,j;
    int             okedges2();
    unsigned char   *malloc(),*getimage();
    unsigned char   *imgptr,*ucimg1,*ucimg2,*ucimg3,*ucimg4,*ucimg5,*ucsav;
    int             mem,pswsize,dwsize,hkwsize,tsecs,totime;
    int             nsize,nsqr,numimgs,total,nsoption,smoption,dzdisp,numbig;
    int             ifiterr,ifprcrv;
    float           vecconst[49],vec1stdf[49],vec2nddf[49];
    float           *iximg,*iyimg,*ixximg,*iyyimg,*ixyimg,*tmpimg,*fltimg;
    float           *ixximgsav,*iyyimgsav,*iximgsav,*iyimgsav,*ixyimgsav;
    float           *ptrsav,*fltimgsav,*avgimgsav,*avgimg,*phsimg,*phsimgsav;
    float           *Kptr,*Kptrsav,*gptr,*bptr,*Hptr;
    float           *bptrsav,*gptrsav,*Hptrsav,*sqrimgsav,*sqrimg;
    float           p,q,r,s,t,g,gsqrt,b,E,F,G,L,M,N,K,H,P,Q,R;
    float           a,f2,piover2,pi,pifact;
    float           area,avgH,avgb,avgK,avginvK;
    float           Kplus,Kminus,Hplus,Hminus,Kabs,Habs,quadvar,babs;
    float           epsilonH,epsilonK,nposH,nnegH,nzeroH,nposK,nnegK,nzeroK;
    float           Kmax,Hmax,mean,sigma,fmax,fmin;
    int             pd1,pd2,edgprmpt,scstats;
    int             lowth[7],difth[7];

    totime = tstart();

    /* Get Input Image from File "rangimg" */

    imgptr = getimage(rangimg,&nsize);
    strcpy(string,"Results Log File for --> ");
    strcat(string,rangimg);

    /* initialize values */

    dwsize   = cmpoptions[1];   hkwsize  = cmpoptions[2];
    edgprmpt = cmpoptions[18];  scstats  = cmpoptions[19];  
    lowth[0] = cmpoptions[12];  difth[0] = cmpoptions[17];  /* eg */
    lowth[1] = cmpoptions[13];  difth[1] = cmpoptions[17];  /* eh */
    lowth[2] = cmpoptions[14];  difth[2] = cmpoptions[17];  /* ed */
    lowth[3] = cmpoptions[15];  difth[3] = cmpoptions[17];  /* ep */
    lowth[4] = cmpoptions[16];  difth[4] = cmpoptions[17];  /* eq */
    lowth[5] = cmpoptions[16];  difth[5] = cmpoptions[17];  /* ef */
    lowth[6] = cmpoptions[12];  difth[6] = cmpoptions[17];  /* ek */
    dzdisp   = cmpoptions[7];   mem      = cmpoptions[8];
/*
    if( dzdisp % 2 == 1 )
    {
        ds(mem);
        dblank(mem);
        dspsub(mem,imgptr,nsize);
    }
*/
    nsqr = nsize*nsize;
    epsilonH  = cmpoptions[4];
    epsilonK  = cmpoptions[5];
    epsilonH *= 1.0e-6;
    epsilonK *= 1.0e-6;

    /* Data for Principal Angle Calculation */

    pi = 3.14159265358979;

    pifact  = 255.0 / pi;
    piover2 = pi / 2.0;
    numbig  = 3;

    /* set up the differential operator coefficient arrays */

    qsfvectrs(dwsize,vecconst,vec1stdf,vec2nddf,&P,&Q,&R);

    /* allocate memory for output arrays and intermediate arrays */

    numimgs = 15;
    total = numimgs*4*nsqr;
    ptrsav = (float *)malloc(total);
    Hptrsav   = ptrsav          ;   Kptrsav   = ptrsav +    nsqr;
    gptrsav   = ptrsav +  2*nsqr;   bptrsav   = ptrsav +  3*nsqr;
    iximgsav  = ptrsav +  4*nsqr;   iyimgsav  = ptrsav +  5*nsqr;
    ixximgsav = ptrsav +  6*nsqr;   iyyimgsav = ptrsav +  7*nsqr;
    ixyimgsav = ptrsav +  8*nsqr;   tmpimg    = ptrsav +  9*nsqr;
    avgimgsav = ptrsav + 10*nsqr;   sqrimgsav = ptrsav + 11*nsqr;
    fltimgsav = ptrsav + 12*nsqr;   phsimgsav = ptrsav + 13*nsqr;
    ucimg1 = malloc(5*nsqr);
    ucimg2 = ucimg1 + nsqr; ucimg3 = ucimg2 + nsqr;
    ucimg4 = ucimg3 + nsqr; ucimg5 = ucimg4 + nsqr;
    total += 5*nsqr;
    printf("%d KBytes of Virtual Memory Allocated\n",total/1000);

    /* Float Image Data for All Future Computations */

    floatimage(fltimgsav,imgptr,nsize,nsize);

    /* Add Noise to Original Image */

    nsoption = cmpoptions[10];
    mean = 3*cmpoptions[11];
    sigma = cmpoptions[11];
    addnoise(fltimgsav,nsize,mean,sigma,nsoption);
    fltimg = fltimgsav;
    dwfltimg(fltimg,tmpimg,nsize,outoptions,cmpoptions,0,
             rangimg,string,&fmax,&fmin,dfnames);

    /* Pre-Smooth Image prior to Derivative Operations */

    pswsize = cmpoptions[0];
    if( pswsize != 1 )
    {
        smoption = cmpoptions[9];
        if( outoptions[50] )
            matcpy(gptrsav,fltimg,nsize,nsize);
        tsecs = tstart();
        fsmoother2(fltimg,nsize,nsize,pswsize,smoption,tmpimg);
        dwfltimg(fltimg,tmpimg,nsize,outoptions,cmpoptions,1,
                 rangimg,string,&fmax,&fmin,dfnames);
        tstop(tsecs,"Time to Smooth");
        if( outoptions[50] )
        {
            matsub(bptrsav,fltimg,gptrsav,nsize,nsize);
            fabsimg(gptrsav,bptrsav,nsize,nsize);
            dwfltimg(gptrsav,tmpimg,nsize,outoptions,cmpoptions,50,
                     rangimg,string,&fmax,&fmin,dfnames);
        rs_edges:
            fnmsupres2(gptrsav,tmpimg,nsize,fmax,lowth[4],numbig,difth[4]);
            dwuscimg(tmpimg,nsize,outoptions,cmpoptions,50,
                     rangimg,string,dfnames);
            if( edgprmpt )
            {
                if( okedges2(&lowth[4],&numbig,&difth[4],string,dfnames) )
                    goto rs_edges;
            }
            refinedge(tmpimg,bptrsav,nsize,nsize,0);
            refinedge(bptrsav,tmpimg,nsize,nsize,1);
            dwuscimg(tmpimg,nsize,outoptions,cmpoptions,50,
                     rangimg,string,dfnames);
        }
    }

    /* compute 1st and 2nd and mixed partials using separable convolution */

    iximg = iximgsav;   iyimg = iyimgsav;   phsimg = phsimgsav; 
    ixyimg = ixyimgsav; ixximg = ixximgsav; iyyimg = iyyimgsav;
    derivtivs(fltimg,iximg,iyimg,ixyimg,ixximg,iyyimg,tmpimg,nsize,nsize,
              vecconst,vec1stdf,vec2nddf,dwsize,
              cmpoptions,outoptions,rangimg,string,dfnames);

    /* Compute Phase Image, Gradient, and Edges if required */

    if( outoptions[28] || outoptions [29] || outoptions[30] )
    {
        phaseimg(phsimg,Hptrsav,iximg,iyimg,nsize,nsize);
        dwfltimg(phsimg,tmpimg,nsize,outoptions,cmpoptions,28,
                 rangimg,string,&fmax,&fmin,dfnames);
        dwfltimg(Hptrsav,tmpimg,nsize,outoptions,cmpoptions,49,
                 rangimg,string,&fmax,&fmin,dfnames);
    }
    if( outoptions [29] || outoptions[30] )
    {
        phasgrad(gptrsav,phsimg,nsize,nsize);
        dwfltimg(gptrsav,tmpimg,nsize,outoptions,cmpoptions,29,
                 rangimg,string,&fmax,&fmin,dfnames);
    }
    if( outoptions[30] )
    {
    pg_edges:
        fnmsupres2(gptrsav,tmpimg,nsize,fmax,lowth[3],numbig,difth[3]);
        dwuscimg(tmpimg,nsize,outoptions,cmpoptions,30,
                 rangimg,string,dfnames);
        if( edgprmpt )
        {
            if( okedges2(&lowth[3],&numbig,&difth[3],string,dfnames) )
                goto pg_edges;
        }
        refinedge(tmpimg,bptrsav,nsize,nsize,0);
        refinedge(bptrsav,tmpimg,nsize,nsize,1);
        dwuscimg(tmpimg,nsize,outoptions,cmpoptions,30,rangimg,string,dfnames);
    }

    /* Compute Laplacian of Digital Surface Function */

    if( outoptions[9] || outoptions[16] )
    {
        matadd(bptrsav,ixximgsav,iyyimgsav,nsize,nsize);
        dwfltimg(bptrsav,tmpimg,nsize,outoptions,cmpoptions,9,
                 rangimg,string,&fmax,&fmin,dfnames);
    }

    /* Compute Zero-Crossings of the Laplacian */

    if( outoptions[16] )
    {
        fislice(bptrsav,ucimg1,nsize,nsize,0.0);
        iximg = iximgsav;
        ucsav = ucimg1;
        iyimg = iyimgsav;
        for(i=0;i<nsqr;++i)
        {
            if( *ucimg1 != 0 )  
            {
                if( fabs(*iximg) + fabs(*iyimg) <= LAPLTHRS )
                    *ucimg1 = 0;
            }
            ucimg1++;
            iximg++;
            iyimg++;
        }
        ucimg1 = ucsav; iximg = iximgsav;
        dwuscimg(ucimg1,nsize,outoptions,cmpoptions,16,
                 rangimg,string,dfnames);
    }

    /* zero crossings of 1st x derivative */

    tsecs = tstart();
    fislice(iximgsav,ucimg1,nsize,nsize,0.0);
    dwuscimg(ucimg1,nsize,outoptions,cmpoptions,10,rangimg,string,dfnames);

    /* zero crossings of 1st y derivative */

    fislice(iyimgsav,ucimg2,nsize,nsize,0.0);
    dwuscimg(ucimg2,nsize,outoptions,cmpoptions,11,rangimg,string,dfnames);

    /* zero crossings of 2nd derivative Ixx */

    fislice(ixximgsav,ucimg3,nsize,nsize,0.0);
    dwuscimg(ucimg3,nsize,outoptions,cmpoptions,12,rangimg,string,dfnames);

    /* zero crossings of 2nd derivative Iyy */

    fislice(iyyimgsav,ucimg4,nsize,nsize,0.0);
    dwuscimg(ucimg4,nsize,outoptions,cmpoptions,13,rangimg,string,dfnames);

    /* zero crossings of 2nd derivative Ixy */

    fislice(ixyimgsav,ucimg5,nsize,nsize,0.0);
    dwuscimg(ucimg5,nsize,outoptions,cmpoptions,14,rangimg,string,dfnames);

    /* Merge Three 2nd Deriv. Zero Crossings Images into One AND image.
       These Points have quality that  Hess[Image] = 0.                  */

    imgand3(ucimg3,ucimg4,ucimg5,nsize,nsize);
    dwuscimg(ucimg3,nsize,outoptions,cmpoptions,15,rangimg,string,dfnames);

    /* Merge Two 1st Deriv. Zero Crossings Images into One:OR
       These points are or general structural interest        */

    if( outoptions[18] )
    {
        imgor(ucimg5,ucimg1,ucimg2,nsize,nsize);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,18,
                 rangimg,string,dfnames);
    }

    /* Merge Two 1st Deriv. Zero Crossings Images into One:AND 
       These points are the critical points of the Image.      */

    imgand(ucimg4,ucimg1,ucimg2,nsize,nsize);
    dwuscimg(ucimg4,nsize,outoptions,cmpoptions,17,rangimg,string,dfnames);

    /* Merge Both Merged Zero Crossing images to get non-degenerate pts. */

    imgnotip(ucimg3,nsize,nsize);
    imgand(ucimg1,ucimg3,ucimg4,nsize,nsize);
    dwuscimg(ucimg1,nsize,outoptions,cmpoptions,19,rangimg,string,dfnames);
    tstop(tsecs,"Zero Crossings Computations");

    /* compute quadratic variation */

    if( outoptions[20] || outoptions[21] )
    {
        quadvar = 0.0;
        bptr = bptrsav;
        for(i=0;i<nsqr;++i)
        {
            *bptr = (*ixximg)*(*ixximg)+(*iyyimg)*(*iyyimg)+
                    2.0*(*ixyimg)*(*ixyimg);
            quadvar += *bptr++;
            ixximg++;
            iyyimg++;
            ixyimg++;
        }
        bptr = bptrsav;  /* display quad var */
        dwfltimg(bptr,tmpimg,nsize,outoptions,cmpoptions,20,rangimg,string,
                 &fmax,&fmin,dfnames);
    }
    if( outoptions[21] )  /* get quad var edges */
    {
    qv_edges:
        fnmsupres2(bptr,ucimg5,nsize,fmax,lowth[4],numbig,difth[4]);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,21,rangimg,string,dfnames);
        if( edgprmpt )
        {
            if( okedges2(&lowth[4],&numbig,&difth[4],string,dfnames) )
                goto qv_edges;
        }
        /* do not refine edges; edges naturally ok, spaced close together */
    }
    if( outoptions[22] || outoptions[23] )
    {
        /* compute cos ( coord angle ) */
        gptr = gptrsav;
        for(i=0;i<nsqr;++i)
        {
            p = *iximg++;
            q = *iyimg++;
            E = 1.0 + p*p;
            F = p*q;
            G = 1.0 + q*q;
            *gptr++ = F / sqrt( E*G );
        }
        gptr = gptrsav;  /* display cos coord angle */
        dwfltimg(gptr,tmpimg,nsize,outoptions,cmpoptions,22,rangimg,string,
                 &fmax,&fmin,dfnames);
    }
    if( outoptions[23] )  /* display abs value cos coord angle */
    {
        for(i=0;i<nsqr;++i)
            *gptr++ = (fabs(*gptr));
        gptr = gptrsav;
        dwfltimg(gptr,tmpimg,nsize,outoptions,cmpoptions,23,
                 rangimg,string,&fmax,&fmin,dfnames);
    }

    /* compute average image for later error computation */

    if( outoptions[24] || outoptions[25] || outoptions[7] )
    {
        fwindopr(avgimgsav,fltimg,tmpimg,nsize,nsize,vecconst,vecconst,
                 dwsize,outoptions,cmpoptions,7,rangimg,string,dfnames);
    }

    /* compute squared image for later error computation */

    if( outoptions[24] || outoptions[25] || outoptions[8] )
    {
        fltimg = fltimgsav;
        sqrimg = sqrimgsav;
        for(i=0; i<nsqr; ++i)
        {
            *sqrimg++ = (*fltimg)*(*fltimg);
            ++fltimg;
        }
        sqrimg = sqrimgsav;
        fltimg = fltimgsav;
        fwindopr(sqrimg,sqrimg,tmpimg,nsize,nsize,vecconst,vecconst,
                 dwsize,outoptions,cmpoptions,8,rangimg,string,dfnames);
    }

    /* compute differential geometric quantities from derivative images *
       ================================================================ */

    area = 0.0; avgH = 0.0; avgK = 0.0; avgb = 0.0; avginvK = 0.0; babs=0.0;
    iximg = iximgsav;      iyimg = iyimgsav;
    ixximg = ixximgsav;    iyyimg = iyyimgsav;  ixyimg = ixyimgsav;
    Kptr = Kptrsav;        bptr = bptrsav;
    Hptr = Hptrsav;        gptr = gptrsav;
    avgimg = avgimgsav;    sqrimg = sqrimgsav;
    ucimg1 = ucimg1;       ucimg2 = ucimg2;
    ifiterr = ( outoptions[24] || outoptions[25] );
    ifprcrv = ( outoptions[46] || outoptions[47] );
    fprintf(stderr,"Entering Main Computation Loop\n");
    tsecs = tstart();

    /* process input image array generating output array */

    for(i=0;i<nsize;++i)
    {
        for(j=0;j<nsize;++j)
        {
            p = (*iximg++);     q = (*iyimg++);
            r = (*ixximg++);    s = (*ixyimg++);    t = (*iyyimg++);

           /*printf("i=%d j=%d p=%g q=%g r=%g s=%g t=%g\n",i,j,p,q,r,s,t);*/

            E = 1.0 + p*p;       F = p*q;           G = 1.0 + q*q;
            g = E + G - 1.0;     /*  g = 1.0 + p*p + q*q;  */
            gsqrt = sqrt(g);
            L = r/gsqrt;         M = s/gsqrt;       N = t/gsqrt; 
            b = L*N - M*M;

            /* compute Gaussian and mean curvature */
            K = b/g;
            H = (E*N + G*L - 2.0*F*M)/(2.0*g);

            area += gsqrt; avgb += b; avgH += H; avgK += K; babs += fabs(b);
            if( K != 0.0 )
                avginvK += (1.0/K);

            *Hptr++ = H;
            *Kptr++ = K; 
            *gptr++ = gsqrt;

            /* quadratic surface fit error */
            if( ifiterr )
            {
                f2 = (*sqrimg++);
                a = (*avgimg++);
                *bptr++ = f2 -(a*a + (p*p+q*q)*Q + s*s*Q*Q + (r*r+t*t)*P);
            }

            /* principal direction angles at each point */
            if( ifprcrv )
            {
                princdir(pifact,piover2,E,F,G,L,M,N,&pd1,&pd2);
                *ucimg1++ = pd1;
                *ucimg2++ = pd2;
            }
        } /* end of j-loop */
    } /* end of i-loop */
    tstop(tsecs,"Main Computation Loop");

    /* display principal direction angles */

    if( ifprcrv )
    {
        dwuscimg(ucimg1,nsize,outoptions,cmpoptions,46,rangimg,string,dfnames);
        dwuscimg(ucimg2,nsize,outoptions,cmpoptions,47,rangimg,string,dfnames);
    }

    /* display square root metric determinant */

    gptr = gptrsav;
    dwfltimg(gptr,tmpimg,nsize,outoptions,cmpoptions,26,rangimg,string,
             &fmax,&fmin,dfnames);
    if( outoptions[27] )
    {
    sg_edges:
        fnmsupres2(gptr,ucimg5,nsize,fmax,lowth[0],numbig,difth[0]);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,27,
                 rangimg,string,dfnames);
        if( edgprmpt )
        {
            if( okedges2(&lowth[0],&numbig,&difth[0],string,dfnames) )  
                goto sg_edges;
        }
        refinedge(ucimg5,ucimg4,nsize,nsize,0);
        refinedge(ucimg4,ucimg5,nsize,nsize,1);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,27,rangimg,string,dfnames);
    }

    /* display output error image */

    bptr = bptrsav;
    dwfltimg(bptr,tmpimg,nsize,outoptions,cmpoptions,24,rangimg,string,
             &fmax,&fmin,dfnames);
    if( outoptions[25] )
    {
    fe_edges:
        fnmsupres2(bptr,ucimg5,nsize,fmax,lowth[5],numbig,difth[5]);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,25,rangimg,string,dfnames);
        if( edgprmpt )
        {
            if( okedges2(&lowth[5],&numbig,&difth[5],string,dfnames) )
                goto fe_edges;
        }
        /* do not refine edges; edges naturally ok, spaced close together */
    }

    /* Compute Principal Curvature Prior to Smoothing
       (Data in bptr and gptr no longer needed.)       */

    if( outoptions[42] || outoptions[43] || 
        outoptions[44] || outoptions[45] )
    {
        bptr = bptrsav; gptr = gptrsav; Hptr = Hptrsav; Kptr = Kptrsav;
        i = (outoptions[44] || outoptions[45]);
        j=princurv(Hptr,Kptr,bptr,gptr,ucimg4,ucimg5,nsize,nsize,epsilonH,i);
        printf("%d Negative Discriminants\n",j);
        dwfltimg(bptr,tmpimg,nsize,outoptions,cmpoptions,42,rangimg,string,
                 &fmax,&fmin,dfnames);
        dwfltimg(gptr,tmpimg,nsize,outoptions,cmpoptions,43,rangimg,string,
                 &fmax,&fmin,dfnames);
        dwuscimg(ucimg4,nsize,outoptions,cmpoptions,44,
                 rangimg,string,dfnames);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,45,
                 rangimg,string,dfnames);
    }

    /* Smooth K and H output */

    Kptr = Kptrsav;  Hptr = Hptrsav;
    if( hkwsize != 1 )
    {
        smoption = cmpoptions[6];
        tsecs = tstart();
        fsmoother2(Kptr,nsize,nsize,hkwsize,smoption,tmpimg);
        fsmoother2(Hptr,nsize,nsize,hkwsize,smoption,tmpimg);
        tstop(tsecs,"Time to Smooth");
     }

    /* output absolute value mean curvature */

    Habs = Hmax = 0.0;
    Hptr = Hptrsav;
    gptr=gptrsav;
    for(i=0;i<nsqr;++i)
    {
        *gptr = (fabs(*Hptr++));   /* g = |H| */
        if( *gptr > Hmax )
            Hmax = *gptr;
        Habs += *gptr++; 
    }
    gptr = gptrsav;
    dwfltimg(gptr,tmpimg,nsize,outoptions,cmpoptions,32,rangimg,string,
             &fmax,&fmin,dfnames);
    if( outoptions[35] )
    {
    H_edges:
        fnmsupres2(gptr,ucimg5,nsize,fmax,lowth[1],numbig,difth[1]);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,35,
                 rangimg,string,dfnames);
        if( edgprmpt )
        {
            if( okedges2(&lowth[1],&numbig,&difth[1],string,dfnames) )
                goto H_edges;
        }
        refinedge(ucimg5,ucimg4,nsize,nsize,0);
        refinedge(ucimg4,ucimg5,nsize,nsize,1);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,35,rangimg,string,dfnames);
    }

    /* compute square root of absolute value Gaussian curvature
       and umbilic parameter H square minus K                   */

    Kabs = Kmax = 0.0;
    Kptr = Kptrsav;     Hptr = Hptrsav;
    bptr = bptrsav;     gptr=gptrsav;
    for(i=0;i<nsqr;++i)
    {
        *bptr++ = (fabs((*Hptr)*(*Hptr) - (*Kptr)));
        *gptr = (fabs(*Kptr++));
        if( *gptr > Kmax )
            Kmax = *gptr;
        Kabs += *gptr++; Hptr++;
    }

    /* output absolute value Gaussian curvature */

    gptr = gptrsav;
    dwfltimg(gptr,tmpimg,nsize,outoptions,cmpoptions,36,rangimg,string,
             &fmax,&fmin,dfnames);
    if( outoptions[39] )
    {
    K_maxima:
        fnmsupres2(gptr,ucimg5,nsize,fmax,lowth[6],numbig,difth[6]);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,39,rangimg,string,dfnames);
        if( edgprmpt )
        {
            if( okedges2(&lowth[6],&numbig,&difth[6],string,dfnames) )
                goto K_maxima;
        }
    }

    /* output squared difference of principal curvatures */

    bptr = bptrsav; 
    dwfltimg(bptr,tmpimg,nsize,outoptions,cmpoptions,40,rangimg,string,
             &fmax,&fmin,dfnames);
    if( outoptions[41] )
    {
    dp_edges:
        fnmsupres2(bptr,ucimg5,nsize,fmax,lowth[2],numbig,difth[2]);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,41,rangimg,string,dfnames);
        if( edgprmpt )
        {
            if( okedges2(&lowth[2],&numbig,&difth[2],string,dfnames) )
                goto dp_edges;
        }
        refinedge(ucimg5,ucimg4,nsize,nsize,0);
        refinedge(ucimg4,ucimg5,nsize,nsize,1);
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,41,
                 rangimg,string,dfnames);
    }

    /* count positive, negative, and zero Gaussian curvature pixels
       compute signum function image */

surf_curv_sign:

    printf("Kmax= %g, epsilonK=%g, %%max=%g\n",Kmax,epsilonK,
            100.0*epsilonK/Kmax);
    sgnfltimg(Kptrsav,ucimg3,nsize,nsize,epsilonK,
              &nposK,&nnegK,&nzeroK,&Kplus,&Kminus);
    blkbdrci(ucimg3,nsize,nsize,hkwsize);
    dwuscimg(ucimg3,nsize,outoptions,cmpoptions,37,rangimg,string,dfnames);

    /* count positive, negative, and zero Mean curvature pixels */

    printf("Hmax= %g, epsilonH=%g, %%max=%g\n",Hmax,epsilonH,
            100.0*epsilonH/Hmax);
    sgnfltimg(Hptrsav,ucimg4,nsize,nsize,epsilonH,
              &nposH,&nnegH,&nzeroH,&Hplus,&Hminus);
    blkbdrci(ucimg4,nsize,nsize,hkwsize);
    dwuscimg(ucimg4,nsize,outoptions,cmpoptions,33,rangimg,string,dfnames);

    /* display zero and sign images */

    if( outoptions[38] ) 
    {
        fislice(Kptrsav,ucimg1,nsize,nsize,0.0);
        /* ds(mem);  */
        dwuscimg(ucimg1,nsize,outoptions,cmpoptions,38,
                 rangimg,string,dfnames);
    }
    if( outoptions[34] )
    {
        fislice(Hptrsav,ucimg2,nsize,nsize,0.0);
        /* ds(mem); */
        dwuscimg(ucimg2,nsize,outoptions,cmpoptions,34,
                 rangimg,string,dfnames);
    }
    if( outoptions[31] )
    {
        surfcolr(ucimg4,ucimg3,ucimg5,nsize,nsize);
        /* ldcolr(0);   */
        /* dslut0(mem); */
        dwuscimg(ucimg5,nsize,outoptions,cmpoptions,31,
                 rangimg,string,dfnames);
    }

    /* print out scalar Features */

    if( scstats)
        printout(nsqr,area,avgH,Hplus,Hminus,Habs,avgK,Kplus,Kminus,Kabs,
                 avgb,babs,avginvK,nposK,nnegK,nzeroK,nposH,nnegH,nzeroH);
    tstop(totime,"Total Time");

    /* prompt for new value of epsilon for best zeroing  */

prompt:
    printf("Hmax= %g, epsilonH=%g, %%max=%g\n",Hmax,epsilonH,
            100.0*epsilonH/Hmax);
    printf("\nEnter New Value of H-Epsilon(%g)__ ",epsilonH);
    if( fgets(string,80,stdin) == NULL)
    {
        printf("\n");
        exit(0);
    }
    savstring(string,0,dfnames[0]);
    if( substr(string,"ds") > -1 )
    {
        system("ds -lut");
        goto prompt;
    }
    if( substr(string,"ex") > -1 )
        return(0);
    if( substr(string,"co") > -1 )
        return(0);
    sscanf(string,"%f",&epsilonH);
    printf("New Value of H-Epsilon= %g\n",epsilonH);
    printf("Kmax= %g, epsilonK=%g, %%max=%g\n",Kmax,epsilonK,
            100.0*epsilonK/Kmax);
    printf("\nEnter New Value of K-Epsilon(%g)__ ",epsilonK);
    if( fgets(string,80,stdin) == NULL)
    {
        printf("\n");
        exit(0);
    }
    savstring(string,0,dfnames[0]);
    if( substr(string,"ds") > -1 )
    {
        system("ds -lut");
        goto prompt;
    }
    if( substr(string,"ex") > -1 )
        return(0);
    if( substr(string,"co") > -1 )
        return(0);
    sscanf(string,"%f",&epsilonK);
    printf("New Value of K-Epsilon= %g\n",epsilonK);
    goto surf_curv_sign;
}
