#include <stdio.h>
#include <math.h>
#include "../rangelib/rangelib.h"

/* 
 * mask-refine-output
 * 5/1/90 PJF
 *
 * This program takes a range image and a labeling produced by Besl's
 * dmorsys and srefine programs, and throws out segments with nonexistent
 * 'real' range pixels.
 */

float *x,*y,*z;
int nrows,ncols;
unsigned char *label,*flag,*outlabel;
char *Progname;

main(argc,argv)
  int argc;
  char *argv[];
  {
    int nr,nc;
    int lmax=0,l,i;
    Progname=argv[0];
    if (argc!=3) {
      fprintf(stderr,"Usage: %s img label >outlabel\n",argv[0]);
      exit(-1);
      };
    if (!read_range_image_from_file(argv[1],&x,&y,&z,&flag,&nrows,&ncols))
      exit(-1);
    if (!read_label_image_from_file(argv[2],&label,&nr,&nc))
      exit(-1);
    if ((nr!=nrows)||(nc!=ncols)) {
      fprintf(stderr,"range and label images are different sizes.\n");
      exit(-1);
      };

    /* find largest label */
    for(i=0;i<nrows*ncols;i++) if (label[i]>lmax) lmax=label[i];

    /* loop through labels */
    for(l=0;l<=lmax;l++) {
      int nvrp=0, nlp=0;
      for(i=0;i<nrows*ncols;i++) 
        if (label[i]==l) {
          nlp++;
          if (flag[i])
            nvrp++;
          else
            label[i]=0;
          };
      fprintf(stderr,"segment %d: %d valid range, %d labeled.\n",l,nvrp,nlp);
      if ((1.0*nvrp/(1.0*nlp)) < 0.95) {
        int j;
        fprintf(stderr,"Clobbering this segment.\n");
        for(j=0;j<nrows*ncols;j++) if (label[j]==l) label[j]=0;
        }
      };
    if (!write_label_image(stdout,label,nr,nc))
      exit(-1);
    exit(0);
  }
