/*
 *  BCBCONVERT
 *
 *      This routine converts coeeficient vector to global form.
 *      Orthogonal Polynomials are used to compute basis
 *      function coefficients found in (cvec).  It is easier
 *      to evaluate function forms based on global polynomial
 *      coefficients which are computed by this routine.
 *
 *  Author:      Arnold. H. Chiu (9-6-88)
 *  
 *  Call:           bcbconvert(x_mean,y_mean,x_size,y_size,c_vector)
 *  Modules used:   none
 *
 */

int bcbconvert(xmean,ymean,nx,ny,cvec)
float   xmean,ymean;
double  cvec[];
int     nx,ny;
{
    double  x2,x3,y2,y3,x42,y42,x40,y40;
    double  mux2gamma,mux2delta,muy2gamma,muy2delta,xymean; 
    double  a,b,c,d,e,f,g,h,i3,j3;

    /* Recompute Orthogonal Polynomial Parameters */

    x2 = ( 1.0 - nx*nx ) / 12.0;
    y2 = ( 1.0 - ny*ny ) / 12.0;
    x3 = ( 7.0 - 3.0*nx*nx ) / 20.0;
    y3 = ( 7.0 - 3.0*ny*ny ) / 20.0;
    mux2gamma = xmean*xmean + x2;
    muy2gamma = ymean*ymean + y2;
    mux2delta = xmean*xmean + x3;
    muy2delta = ymean*ymean + y3;
    xymean = xmean*ymean;

    /* Convert Values to Polynomial Coefficients */

    a = cvec[0] - (cvec[1]*xmean + cvec[2]*ymean) +
        cvec[3]*xymean + cvec[4]*mux2gamma + cvec[5]*muy2gamma +
        -(cvec[6]*ymean*mux2gamma + cvec[7]*xmean*muy2gamma) +
        -(cvec[8]*xmean*mux2delta + cvec[9]*ymean*muy2delta);

    b = cvec[1] - (cvec[3]*ymean + 2.0*cvec[4]*xmean) +
        cvec[7]*(muy2gamma) + cvec[8]*(3.0*xmean*xmean+x3) +
        (2.0*xymean*cvec[6]);

    c = cvec[2] - (cvec[3]*xmean + 2.0*cvec[5]*ymean) +
        cvec[6]*(mux2gamma) + cvec[9]*(3.0*ymean*ymean+y3) +
        (2.0*xymean*cvec[7]);

    d = cvec[3] - (2.0*xmean*cvec[6] + 2.0*ymean*cvec[7]);

    e = cvec[4] - (cvec[6]*ymean + 3.0*cvec[8]*xmean);

    f = cvec[5] - (cvec[7]*xmean + 3.0*cvec[9]*ymean);

    g = cvec[6];        h = cvec[7];
    i3= cvec[8];        j3= cvec[9];

    /** Print Out Coefficients */
    /* printf("                     2   2   2      2   3   3   3    2 2    3   4   4\n");
    printf("z(x,y)=a+bx+cy+dxy+ex +fy +gx y +hxy +ix +jy +kx y+lx y +mxy +nx +oy\n");
    printf("a= %g\t\t",a);  printf("b= %g\t\t",b);  printf("c= %g\n",c);
    printf("d= %g\t\t",d);  printf("e= %g\t\t",e);  printf("f= %g\n",f);
    printf("g= %g\t\t",g);  printf("h= %g\t\t",h);  printf("i= %g\n",i4);
    printf("j= %g\t\t",j4);
    */

    /* Replace Coefficient Vector with Global Values */

    cvec[0] = a;    cvec[1] = b;    cvec[2] = c;
    cvec[3] = d;    cvec[4] = e;    cvec[5] = f;
    cvec[6] = g;    cvec[7] = h;    cvec[8] = i3;   cvec[9] = j3;
    return(0);
}
