/*
 *  BLKBDRFI
 *
 *      This routine put black border into float image.
 *      Some routines are affected by extraneous
 *      data in outer frame region of image where
 *      there is not enough data to compute output.
 *      This routine sets pixels equal to zero.
 *
 *  Author:     Paul J. Besl    (6-27-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           blkbdrfi(float_image,x_size,y_size,window_size)
 *  Modules used:   none
 *
 */
 
blkbdrfi(fltimg,nxsize,nysize,wsize)
float   *fltimg;
int     nxsize,nysize,wsize;
{
    register int        i,j;
    register float      *lptr,*rptr;
    int                 whalf,offset,stop;
    float               *start;

    whalf = wsize/2;
    start = fltimg + whalf;
    stop = nysize - whalf;
    offset = nxsize - wsize;

    /* fixup whalf pixels at beginning and end of each scanline */

    for(i=whalf; i<stop; ++i)
    {
        lptr = start + i*nxsize;
        rptr = lptr + offset;
        for(j=0; j<whalf; ++j)
            *--lptr = *++rptr = 0.0;
    }

    /* fixup first two and last two scanlines of image */

    for(i=0; i<whalf; ++i)       
    {
        lptr = fltimg + i*nxsize;
        rptr = fltimg + (i+stop)*nxsize;
        for(j=0; j<nxsize; ++j)
            *lptr++ = *rptr++ = 0.0;
    }
}
