/*
 *  BQ4CONVERT
 *
 *      This routine converts coeeficient vector to global form.
 *      Orthogonal Polynomials are used to compute basis
 *      function coefficients found in (cvec).  It is easier
 *      to evaluate function forms based on global polynomial
 *      coefficients which are computed by this routine.
 *
 *  Author:     Paul J. Besl    (4-21-85)
 *
 *  Revised:    Arnold. H. Chiu (8-22-88)
 *  
 *  Call:           bq4convert(x_mean,y_mean,x_size,y_size,c_vector)
 *  Modules used:   none
 *
 */

bq4convert(xmean,ymean,nx,ny,cvec)
float   xmean,ymean;
double  cvec[];
int     nx,ny;
{
    double  x2,x3,y2,y3,x42,y42,x40,y40;
    double  mux2gamma,mux2delta,muy2gamma,muy2delta,xymean; 
    double  a,b,c,d,e,f,g,h,i4,j4,k,l,m,n,o;

    /* Recompute Orthogonal Polynomial Parameters */

    x2 = ( 1.0 - nx*nx ) / 12.0;
    y2 = ( 1.0 - ny*ny ) / 12.0;
    x3 = ( 7.0 - 3.0*nx*nx ) / 20.0;
    y3 = ( 7.0 - 3.0*ny*ny ) / 20.0;
    x42 = ( 13.0 - 3.0*nx*nx ) / 14.0;
    y42 = ( 13.0 - 3.0*ny*ny ) / 14.0;
    x40 = ( 3.0*(nx*nx - 1.0)*(nx*nx - 9.0) ) / 560.0;
    y40 = ( 3.0*(ny*ny - 1.0)*(ny*ny - 9.0) ) / 560.0;
    mux2gamma = xmean*xmean + x2;
    muy2gamma = ymean*ymean + y2;
    mux2delta = xmean*xmean + x3;
    muy2delta = ymean*ymean + y3;
    xymean = xmean*ymean;

    /* Convert Values to Polynomial Coefficients */

    a = cvec[0] - (cvec[1]*xmean + cvec[2]*ymean) +
        cvec[3]*xymean + cvec[4]*mux2gamma + cvec[5]*muy2gamma +
        -(cvec[6]*ymean*mux2gamma + cvec[7]*xmean*muy2gamma) +
        -(cvec[8]*xmean*mux2delta + cvec[9]*ymean*muy2delta);
    a += cvec[10]*(xymean*mux2delta) + cvec[12]*(xymean*muy2delta);
    a += cvec[11]*(mux2gamma)*(muy2gamma);
    a += cvec[13]*(xmean*xmean*xmean*xmean+xmean*xmean*x42+x40);
    a += cvec[14]*(ymean*ymean*ymean*ymean+ymean*ymean*y42+y40);

    b = cvec[1] - (cvec[3]*ymean + 2.0*cvec[4]*xmean) +
        cvec[7]*(muy2gamma) + cvec[8]*(3.0*xmean*xmean+x3) +
        (2.0*xymean*cvec[6]);
    b -= cvec[10]*(ymean*(3.0*xmean*xmean+x3));
    b -= cvec[11]*(2.0*xmean*(muy2gamma));
    b -= cvec[12]*(ymean*(muy2delta));
    b -= cvec[13]*(4.0*xmean*xmean*xmean+2.0*xmean*x42);

    c = cvec[2] - (cvec[3]*xmean + 2.0*cvec[5]*ymean) +
        cvec[6]*(mux2gamma) + cvec[9]*(3.0*ymean*ymean+y3) +
        (2.0*xymean*cvec[7]);
    c -= cvec[10]*(xmean*(mux2delta));
    c -= cvec[11]*(2.0*ymean*(mux2gamma));
    c -= cvec[12]*(xmean*(3.0*ymean*ymean+y3));
    c -= cvec[14]*(4.0*ymean*ymean*ymean+2.0*ymean*y42);

    d = cvec[3] - (2.0*xmean*cvec[6] + 2.0*ymean*cvec[7]);
    d += cvec[11]*(4.0*xymean);
    d += cvec[10]*(3.0*xmean*xmean+x3);
    d += cvec[12]*(3.0*ymean*ymean+y3);

    e = cvec[4] - (cvec[6]*ymean + 3.0*cvec[8]*xmean);
    e += cvec[10]*(3.0*xymean);
    e += cvec[11]*(muy2gamma);
    e += cvec[13]*(6.0*xmean*xmean+x42);

    f = cvec[5] - (cvec[7]*xmean + 3.0*cvec[9]*ymean);
    f += cvec[12]*(3.0*xymean);
    f += cvec[11]*(mux2gamma);
    f += cvec[14]*(6.0*ymean*ymean+y42);

    g = cvec[6] - (2.0*ymean*cvec[11] + cvec[10]*3.0*xmean);
    h = cvec[7] - (2.0*xmean*cvec[11] + cvec[12]*3.0*ymean);
    i4= cvec[8] - (4.0*xmean*cvec[13] + cvec[10]*ymean);
    j4= cvec[9] - (4.0*ymean*cvec[14] + cvec[12]*xmean);

    k = cvec[10]; l = cvec[11]; m = cvec[12];
    n = cvec[13]; o = cvec[14];

    /** Print Out Coefficients */
    /* printf("                     2   2   2      2   3   3   3    2 2    3   4   4\n");
    printf("z(x,y)=a+bx+cy+dxy+ex +fy +gx y +hxy +ix +jy +kx y+lx y +mxy +nx +oy\n");
    printf("a= %g\t\t",a);  printf("b= %g\t\t",b);  printf("c= %g\n",c);
    printf("d= %g\t\t",d);  printf("e= %g\t\t",e);  printf("f= %g\n",f);
    printf("g= %g\t\t",g);  printf("h= %g\t\t",h);  printf("i= %g\n",i4);
    printf("j= %g\t\t",j4); printf("k= %g\t\t",k);  printf("l= %g\n",l);
    printf("m= %g\t\t",m);  printf("n= %g\t\t",n);  printf("o= %g\n",o);
    */

    /* Replace Coefficient Vector with Global Values */

    cvec[0] = a;    cvec[1] = b;    cvec[2] = c;
    cvec[3] = d;    cvec[4] = e;    cvec[5] = f;
    cvec[6] = g;    cvec[7] = h;    cvec[8] = i4;   cvec[9] = j4;
    cvec[10] = k;   cvec[11] = l;   cvec[12] = m;
    cvec[13] = n;   cvec[14] = o;
    return(0);
}
