/*
 *  BQDCONVERT
 *
 *      This routine converts coefficient vector to global form.
 *      Orthogonal Polynomials are used to compute basis function
 *      coefficients found in (cvec).  It is easier to evaluate
 *      function forms based on global polynomial coefficients
 *      which are computed by this routine.
 *
 *  Author:     Paul J. Besl    (4-21-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           bqdconvert(x_mean,y_mean,x_size,_sizey,cvec)
 *  Modules used:   
 *
 */

bqdconvert(xmean,ymean,nx,ny,cvec)
float   xmean,ymean;
double  cvec[];
int     nx,ny;
{
    double  x2,y2;
    double  mux2gamma,muy2gamma,xymean; 
    double  a,b,c,d,e,f;

    /* Recompute Orthogonal Polynomial Parameters */

    x2 = ( 1.0 - nx*nx ) / 12.0;
    y2 = ( 1.0 - ny*ny ) / 12.0;
    mux2gamma = xmean*xmean + x2;
    muy2gamma = ymean*ymean + y2;
    xymean = xmean*ymean;

    /* Convert Values to Polynomial Coefficients */

    a = cvec[0] - (cvec[1]*xmean + cvec[2]*ymean) +
        cvec[3]*xymean + cvec[4]*mux2gamma + cvec[5]*muy2gamma;
    b = cvec[1] - (cvec[3]*ymean + 2.0*cvec[4]*xmean);
    c = cvec[2] - (cvec[3]*xmean + 2.0*cvec[5]*ymean);
    d = cvec[3]; e = cvec[4]; f = cvec[5];

    /* Print Out Coefficients */

    /*
    printf("                     2   2\n");
    printf("z(x,y)=a+bx+cy+dxy+ex +fy\n"); 
    printf("a= %g\t\t",a);  printf("b= %g\t\t",b);  printf("c= %g\n",c);
    printf("d= %g\t\t",d);  printf("e= %g\t\t",e);  printf("f= %g\n",f);
    */

    /* Replace Coefficient Vector with Global Values */

    cvec[0] = a;    cvec[1] = b;    cvec[2] = c;
    cvec[3] = d;    cvec[4] = e;    cvec[5] = f;
    return(0);
}
