/*
 *  BQDSURFIT
 *
 *      This subroutine takes the (x,y,z) data arrays and
 *      the mean data values (xmean,ymean) and fits a
 *      biquadratic surface function with 6 coefficients
 *      which are returned in (cvec) vector.
 *
 *  Author:     Paul J. Besl    (4-7-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           bqdsurfit(x_array,y_array,z_array,num_pixels,
 *                            x_mean,y_mean,error_total,x_size,y_size,
 *                            cvec,opt,ovec)
 *  Modules used:   linlsq(),   matcpyd(),  bqdconvert()
 *
 */

#define ARSIZE 6

bqdsurfit(xray,yray,zray,npixels,xmean,ymean,errtol,nx,ny,cvec,opt,ovec)
float   *xray,*yray,*zray,xmean,ymean,*errtol; 
int     npixels,nx,ny,opt;
double  cvec[],ovec[];
{
    unsigned char       *malloc();
    double              *phiray,*zdblvec,error,x2,y2;
    float               *xptr,*yptr,*zptr;
    register double     *phiptr,xval,yval,*zdblptr;
    register int        i;
    int                 rank,arsize = ARSIZE;

    if( npixels < arsize )
    {
        printf("=> %d Data Points needed for Surface Fit\n",arsize);
        exit(1);
    }

    /* Allocate memory for npixels x arsize float array. */

    phiray  = (double *)malloc(8*arsize*npixels);
    zdblvec = (double *)malloc(8*npixels);

    /* Compute Quadratic Offset Values */

    x2 = ( 1.0 - nx*nx ) / 12.0;
    y2 = ( 1.0 - ny*ny ) / 12.0;

    /* Loop over all data points filling in PHI array */

    xptr = xray;    yptr = yray;    zptr = zray;
    phiptr = phiray;
    zdblptr = zdblvec;
    for(i=0;i<npixels;++i)
    {
        xval = (*xptr++) - xmean;
        yval = (*yptr++) - ymean;
        *zdblptr++ = *zptr++;
        *phiptr++ = 1;              /* 0 */
        *phiptr++ = xval;           /* 1 */
        *phiptr++ = yval;           /* 2 */
        *phiptr++ = xval*yval;      /* 3 */
        *phiptr++ = xval*xval + x2; /* 4 */
        *phiptr++ = yval*yval + y2; /* 5 */
    }

    /* Compute Least Squares Solution */

    rank = linlsq(cvec,phiray,zdblvec,npixels,arsize,opt,&error);
    matcpyd(ovec,cvec,arsize,1);
    bqdconvert(xmean,ymean,nx,ny,cvec);
    printf("[Biquadratic_Fit] RMS_Error= %g\n",error);
    free( zdblvec );
    free( phiray );
    *errtol = error;
}
