/*
 *  BURRTRIM
 *
 *      This subroutine refines the output of the
 *      region grower operator to cleanup isolated pixels.
 *      It can only turn on pixels that are off.
 *      Hence, it is different from previously implemented edge
 *      operations that may turn on or turn off pixels.
 *      Returns the number of changes made to image.
 *
 *  Author:     Paul J. Besl    (8-5-85)
 *
 *  Revised:    Arnold. H. Chiu (7-27-88)
 *  
 *  Call:           burrtrim(image_pointer,output_pointer,x_size,y_size,opt)
 *  Modules used:   map3wndwt()
 *
 */

#define MASK 0x0010  /* mask out center pixel of 3x3 window */

int burrtrim(imgptr,outptr,nx,ny,opt)
unsigned char   *imgptr,*outptr;
int             nx,ny,opt;
{
    register int            sum,val;
    register unsigned char  *p1,*p2,*p3,*po;
    int                     map3wndwt();
    int                     ntot,i,count,changes;

    /* Zero Out Pixels without Complete 3x3 Neighborhoods */

    ntot = nx*ny;
    count = nx+1;
    po = outptr;
    sum = ntot - count;
    p1 = outptr + sum;
    for(i=0; i<count; ++i)
        *po++ = *p1++ = 0;

    /* Initialize Values and Pointers */

    ntot = ntot - 2*(nx+1);    changes = 0;
    p1 = imgptr; p2 = p1 + nx; p3 = p2 + nx;
    p1++;        p2++;         p3++;
    po = outptr + nx + 1;

    /* For Each Pixel in the Output Image */

    for(i=0; i<ntot; ++i)
    {
        if( *p2 )
        {
            p1--;   p2--;   p3--;
            val = count = sum = 0;
            if( *p1 )
            {
                sum++;
                val += *p1;
                count++;
            }
            if( *p2 )
            {
                sum += 8;
                val += *p2;
                count++;
            }
            if( *p3 )
            {
                sum += 64;
                val += *p3;
                count++;
            }
            p1++;   p2++;   p3++;
            if( *p1 )
            {
                sum += 2;
                val += *p1;
                count++;
            }
            if( *p2 )
            {
                sum += 16;
                val += *p2;
                count++;
            }
            if( *p3 )
            {
                sum += 128;
                val += *p3;
                count++;
            }
            p1++;   p2++;   p3++;
            if( *p1 )
            {
                sum += 4;
                val += *p1;
                count++;
            }
            if( *p2 )
            {
                sum += 32;
                val += *p2;
                count++;
            }
            if( *p3 )
            {
                sum += 256;
                val += *p3;
                count++;
            }
            if( sum != 0 )
            {
                /*printf("i=%d premap=%d val=%d count=%d\n",
                       i,sum,val,count);*/
                sum = MASK & map3wndwt( sum );
                /*printf("postmap=%d postval=%d\n",sum,val);*/
                if( sum != 0 )
                {
                    *po++ = val/count;
                    changes++;
                }
                else
                    *po++ =   0;
            }
            else
                *po++ = 0;
        }
        else /* increment all pointers;  out = in */
        {
            p1++;
            p3++;
            *po++ = *p2++;
        }
    }
    return(changes);
}
