/*
 *  CMPMENU_SR
 *
 *      This routine performs menu command processing for
 *      compute options in <Surface Refinement Program>.
 *
 *  Author:     Paul J. Besl    (7-20-85)
 *
 *  Revised:    Arnold. H. Chiu (8-31-88)
 *  
 *  Call:           cmpmenu_sr(compute_options,string,string_size
 *                             default_file_names,image_size)
 *  Modules used:   savstring(),    substr(),   ttyclrhom()
 *
 */
 
#include <stdio.h>

#define MAXMENU     34
#define NUMCMDS     22
#define XUB         14
#define YUB         15
#define LIST        18
#define RETURN      19
#define GOBACK      20
#define QUIT        21

int cmpmenu_sr(cmpoptions,string,strsize,dfnames,img_size)
int     *cmpoptions,strsize,img_size;
char    *string,*dfnames[];
{
    /* default switch settings for compute options */

    static int cmpinit[NUMCMDS] =
        {   3,   0,  30, 350,   8,  30, 420,   9, 280, 150,
           50,  20,   0,   0, 128, 128,   1,   0,   0,   0,   0};

    /* 2-char mnemonic commands */

    static char mnemonic[NUMCMDS][4] =
        {   "dz","dc","is","ut","lt","mi","me","mc","el","ae",
            "mb","tv","xl","yl","xu","yu","ft","fo",
            "ls","co","gb","qt"};

    /* Menu List */

    static char menubuf[NUMCMDS][MAXMENU] = {
        "Deanza_Display_Option           ",
        "Deanza_Memory_Channel(0123)     ",
        "Minimum_Initial_Region_Size     ",
        "Upper_Contraction_Pixel_Count   ",
        "Lower_Contraction_Pixel_Count   ",
        "Fit/Grow_Iteration_Limit(max)   ",
        "RMS_Error_Limit_in_Hundredths   ",
        "Maximum_Number_of_Contractions  ",
        "Error_Limit_Factor_for_Growth   ",
        "Acceptability_Error_Factor      ",
        "Maximum_BlackBoard_Region_Size  ",
        "Test_Value_for_Regions_Test     ",
        "Min_X_for_Sub_Image_Box         ",
        "Min_Y_for_Sub_Image_Box         ",
        "Max_X_for_Sub_Image_Box         ",
        "Max_Y_for_Sub_Image_Box         ",
        "Fitting_Method [0 1(dllsq.f) 2] ",
        "Fit_Order [0(variable) 1 2 3 4] ",
        "List_Options                    ",
        "Continue                        ",
        "Go_Back                         ",
        "Quit                            " };

    int             nargs,icase,defcase,param;
    char            junk[8];
    register int    i,j;

    /* set up default switch settings */

    for(i=0; i<NUMCMDS; ++i)
    {
        if ( ((i == XUB) || (i == YUB)) && (img_size > 0) )
        {
            cmpoptions[XUB] = cmpoptions[YUB] = img_size;
            continue;
        }
        cmpoptions[i] = cmpinit[i];
    }
    icase = -1;
    defcase = LIST;

    /* Enter Command Prompt Loop */

enter_command:

    printf("\nEnter Compute Option(%s)__ ",mnemonic[defcase]);
    if( fgets(string,strsize,stdin) == NULL )
    {
        printf("\n");
        exit(1);
    }

    /* Handle Default Case  (nargs=0 --> mnemonic entered) */

    if( strlen(string) <= 1 )
    {
        icase = defcase;
        nargs = 1;
    }
    else
        nargs = sscanf(string,"%d",&icase);

    /* Number Specification Case and Default Case */

    /* printf("nargs=%d icase=%d\n",nargs,icase); */
    if( nargs == 1 && ( icase > -1 && icase < NUMCMDS ) )
    {
        if( icase == LIST )
            goto list_menu;
        savstring(string,0,dfnames[0]);
        if( icase == RETURN )
            return(0);
        if( icase == GOBACK )
            return(-1);
        if( icase == QUIT )
            exit(0);
        nargs = sscanf(string,"%d %d",&icase,&param);
        if(nargs == 2)
            cmpoptions[icase] = param;
        else
        {
            printf("\nEnter Parameter(%d)__ ",cmpoptions[icase]);
            if( fgets(string,strsize,stdin) == NULL )
            {
                printf("\n");
                exit(1);
            }
            sscanf(string,"%d",&cmpoptions[icase]);
            savstring(string,0,dfnames[0]);
        }
        printf("%s = %d\n",menubuf[icase],cmpoptions[icase]);
        goto enter_command;
    }

    /* Mnemonic Specification */

    icase = -1;
    if( substr(string,"?") > -1 )
        goto list_menu;
    for(i=0; i< NUMCMDS; ++i)
    {
        if( substr(string,mnemonic[i]) > -1 ) 
        {
            icase = i; /* 13 lines of code duplicated below */
            if( icase == LIST )
                goto list_menu;
            savstring(string,0,dfnames[0]);
            if( icase == RETURN )
                return(0);
            if( icase == GOBACK )
                return(-1);
            if( icase == QUIT )
                exit(0);
            nargs = sscanf(string,"%s %d",junk,&param);
            if(nargs == 2)
                cmpoptions[icase] = param;
            else
            {
                printf("\nEnter Parameter(%d)__ ",cmpoptions[icase]);
                if( fgets(string,strsize,stdin) == NULL ) 
                {
                    printf("\n");
                    exit(1);
                }
                sscanf(string,"%d",&cmpoptions[icase]);
                savstring(string,0,dfnames[0]);
            }
            printf("%s = %d\n",menubuf[icase],cmpoptions[icase]);
            goto enter_command;
        }
    }
    fprintf(stderr,"Unknown Command: %s",string);
    goto enter_command;
    /*===============*/

    /* List Out Commands and Status */

list_menu:

    ttyclrhom();
    printf("Enter All Compute Options by Number or Mnemonic [Type: option value]\n");
    for(i=0; i < NUMCMDS; ++i )
    {
        if( i < LIST )
            printf("%2d:%s-%s = %d\n",i,mnemonic[i],menubuf[i],cmpoptions[i]);
        else
            printf("%2d:%s-%.14s",i,mnemonic[i],menubuf[i]);
    }
    printf("\n");
    goto enter_command;
}
