/*
 *  CNTRCTEST
 *
 *      This routine performs the contraction profile test.
 *      All images are (nx) x (ny) pixels.
 *      The binary (regnimg) image contains a region that is to
 *      be contracted (maxcnt) times to see if it vanishes.
 *      (img1) and (img2) are scratch images.
 *      Contraction profile is stored in (pixlcnts) array of length (maxcnt).
 *
 *  Author:     Paul J. Besl    (9-8-85)
 *
 *  Revised:    Arnold. H. Chiu (7-27-88)
 *  
 *  Call:           cntrctest(region_image,image1,image2,x_size,y_size,
 *                            pixel_counts,max_count)
 *  Modules used:   contract()
 *
 */

int cntrctest(regnimg,img1,img2,nx,ny,pixlcnts,maxcnt)
unsigned char   *regnimg,*img1,*img2;
int             nx,ny,pixlcnts[],maxcnt;
{
    int     tstart(),contract();
    int     tsecs,icount,pxlcnt;

    /* CONTRACTION LOOP */

    tsecs = tstart();
    imgcpy(img1,regnimg,nx,ny);
    icount = 0;
    pxlcnt = 1;
    while( pxlcnt > 0 && icount < maxcnt )
    {
        if( icount % 2 == 0 )       /* icount even */
            pxlcnt = contract(img1,img2,nx,ny);
        else                        /* icount odd */
            pxlcnt = contract(img2,img1,nx,ny);
        pixlcnts[icount++] = pxlcnt;
    } 

    /* Print More Data to Screen */

    tstop(tsecs,"Contraction Test Loop Time");
    return(icount);
}
