/*
 *  CNTRCTPRF
 *
 *      This routine computes the contraction profile of a region
 *      All images are (nx) x (ny) pixels.
 *      The binary (regnimg) image contains a region
 *      that is to be contracted to provide a seed image (seedimg).
 *      (img1) and (img2) are scratch images.
 *      (t1) and (t2) are thresholds used to determine seed image.
 *      Contraction profile is stored in (pixlcnts) array of length
 *      (maxcnt).  (npxls) in region of (regnimg).
 *  
 *
 *  Author:     Paul J. Besl    (8-29-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           cntrctprf(region_image,seed_image,image_1,image_2,
 *                            x_size,y_size,pixel_counts,max_count,
 *                            t1,t2,num_pixels,dzdisp,mem)
 *  Modules used:   contractx(), rgnbrc(), imgcpy()
 *
 */

int cntrctprf(regnimg,seedimg,img1,img2,nx,ny,pixlcnts,maxcnt,
              t1,t2,npxls,dzdisp,mem)
unsigned char   *regnimg,*seedimg,*img1,*img2;
int             nx,ny,pixlcnts[],maxcnt,t1,t2,npxls,dzdisp,mem;
{
    int     tstart(),rgnbrc(),contract();
    int     tsecs;
    int     icount,ncontrcts,finalcnt,pxlcnt,limit,first,last;

    /* Determine Contraction Profile of Region */
    tsecs = tstart();
    printf("\nRegion Contraction Profile (%d pixels)\n",npxls);
    finalcnt = 0;
    ncontrcts = 0;
    icount = 0;
    pxlcnt = npxls;
    imgcpy(img1,regnimg,nx,ny);
    printf("%d: t1=%d<? %d <?t2=%d (n=%d,f=%d)\n",
            icount,t1,pxlcnt,t2,ncontrcts,finalcnt);
    limit = t1 - 1;
    first = -1;
    last = -1;
 
    /* ================
     * CONTRACTION LOOP
     * ================ */

    while( pxlcnt > limit )
    {
        if( icount % 2 == 0 )  /* icount even */
        {
            pxlcnt = contractx(img1,img2,nx,ny,&first,&last);
            if( pxlcnt )
                pxlcnt = rgnbrc(img2,nx,ny,0,0);
            /* if( dzdisp )
                dspsub2(mem,img2,nx); */
            if( t1 <= pxlcnt && pxlcnt <= t2 )
            {
                imgcpy(seedimg,img2,nx,ny);
                ncontrcts = icount + 1;
                finalcnt = pxlcnt;
            }
            if( icount + 1 >= maxcnt && finalcnt == 0 )
            {
                imgcpy(seedimg,img2,nx,ny);
                ncontrcts = icount + 1;
                finalcnt = pxlcnt;
             }
        }
        else
        {
            pxlcnt = contractx(img2,img1,nx,ny,&first,&last);
            if( pxlcnt )
                pxlcnt = rgnbrc(img1,nx,ny,0,0);
            /* if( dzdisp )
                dspsub2(mem,img1,nx); */
            if( t1 <= pxlcnt && pxlcnt <= t2 )
            {
                imgcpy(seedimg,img1,nx,ny); 
                ncontrcts = icount + 1;
                finalcnt = pxlcnt;
            }
            if( icount + 1 >= maxcnt && finalcnt == 0 )
            {
                imgcpy(seedimg,img1,nx,ny); 
                ncontrcts = icount + 1;  finalcnt = pxlcnt;
            }
        }
        pixlcnts[icount++] = pxlcnt;
        printf("%d: t1=%d<? %d <?t2=%d (n=%d,f=%d)\n",
               icount,t1,pxlcnt,t2,ncontrcts,finalcnt);
        if( icount >= maxcnt )
            pxlcnt = -1;
    } /* ==== End of Contraction Loop ==== */

    /* Print More Data to Screen */
    if( finalcnt == 0 )
    {
        imgcpy(seedimg,regnimg,nx,ny);
        finalcnt = npxls;
    }
    printf("%d Contractions to Get Seed Region (%d pixels)\n",
            ncontrcts,finalcnt);
    tstop(tsecs,"Contraction Loop Time");
    return(ncontrcts);
}
