/*
 *  DILATE
 *
 *  This routine dilates regions of binary image.
 *  Return the number of non-zero pixels.
 *  It only turns some zero pixels to nonzero and 
 *  cannot turn any nonzeros to zeros.
 *
 *  Author:     Paul J. Besl    (5-14-85)
 *
 *  Revised:    Arnold. H. Chiu (7-27-88)
 *  
 *  Call:           dilate(image_pointer,output_pointer,x_size,y_size)
 *  Modules used:   none
 *
 */

int dilate(imgptr,outptr,nx,ny)
unsigned char   *imgptr,*outptr;
int             nx,ny;
{
    int                     ntot,itemp,jtemp,count;
    register int            log1,i;
    register unsigned char  *p1,*p2,*p3,*po;

    /* Initialize Values and Pointers */

    ntot = nx*ny;
    count = 0;
    itemp = nx+1;
    po = outptr;
    jtemp = ntot - itemp;
    p1 = outptr + jtemp;
    for(i=0; i<itemp; ++i)
        *po++ = *p1++ = 0;
    ntot = ntot - 2*(nx+1);
    p1 = imgptr + 1;
    p2 = p1 + nx;
    p3 = p2 + nx;
    po = outptr + nx + 1;

    /* For Each Pixel in the Output Image */

    for(i=0; i<ntot; ++i)
    {
        if( *p2 == 0 ) 
        {
            p1--;       p2--;       p3--;
            log1 = *p1 || *p2 || *p3;
            p1++;       p2++;       p3++;
            log1 = log1 || *p1 || *p2  || *p3;
            p1++;       p2++;       p3++;
            if( log1 || *p1 || *p2  || *p3 )
            {
                *po++ = 255;
                count++;
            }
            else
                *po++ = 0;
        }
        else
        {
            p1++;
            p2++;
            p3++;
            *po++ = 255;
        }
    }
    return(count);
}
