/* dllsq.f -- translated by f2c (version of 29 May 1992  0:59:25).
   You must link the resulting object file with the libraries:
	-lF77 -lI77 -lm -lc   (in that order)
*/

#include "f2c.h"

/* NAASA  4.1.003 DLLSQ    FTN   06-24-75     THE UNIV OF MICH COMP CTR */

/*     .................................................................. */

/*        SUBROUTINE DLLSQ */

/*        PURPOSE */
/*           TO SOLVE LINEAR LEAST SQUARES PROBLEMS, I.E. TO MINIMIZE */
/*           THE EUCLIDEAN NORM OF B-A*X, WHERE A IS A M BY N MATRIX */
/*           WITH M NOT LESS THAN N. IN THE SPECIAL CASE M=N SYSTEMS OF */
/*           LINEAR EQUATIONS MAY BE SOLVED. */

/*        USAGE */
/*           CALL DLLSQ (A,B,M,N,L,X,IPIV,EPS,IER,AUX) */

/*        DESCRIPTION OF PARAMETERS */
/*           A      - DOUBLE PRECISION M BY N COEFFICIENT MATRIX */
/*                    (DESTROYED). */
/*           B      - DOUBLE PRECISION M BY L RIGHT HAND SIDE MATRIX */
/*                    (DESTROYED). */
/*           M      - ROW NUMBER OF MATRICES A AND B. */
/*           N      - COLUMN NUMBER OF MATRIX A, ROW NUMBER OF MATRIX X. */
/*           L      - COLUMN NUMBER OF MATRICES B AND X. */
/*           X      - DOUBLE PRECISION N BY L SOLUTION MATRIX. */
/*           IPIV   - INTEGER OUTPUT VECTOR OF DIMENSION N WHICH */
/*                    CONTAINS INFORMATIONS ON COLUMN INTERCHANGES */
/*                    IN MATRIX A. (SEE REMARK NO.3). */
/*           EPS    - SINGLE PRECISION INPUT PARAMETER WHICH SPECIFIES */
/*                    A RELATIVE TOLERANCE FOR DETERMINATION OF RANK OF */
/*                    MATRIX A. */
/*           IER    - A RESULTING ERROR PARAMETER. */
/*           AUX    - A DOUBLE PRECISION AUXILIARY STORAGE ARRAY OF */
/*                    DIMENSION MAX(2*N,L). ON RETURN FIRST L LOCATIONS */
/*                    OF AUX CONTAIN THE RESULTING LEAST SQUARES. */

/*        REMARKS */
/*           (1) NO ACTION BESIDES ERROR MESSAGE IER=-2 IN CASE */
/*               M LESS THAN N. */
/*           (2) NO ACTION BESIDES ERROR MESSAGE IER=-1 IN CASE */
/*               OF A ZERO-MATRIX A. */
/*           (3) IF RANK K OF MATRIX A IS FOUND TO BE LESS THAN N BUT */
/*               GREATER THAN 0, THE PROCEDURE RETURNS WITH ERROR CODE */
/*               IER=K INTO CALLING PROGRAM. THE LAST N-K ELEMENTS OF */
/*               VECTOR IPIV DENOTE THE USELESS COLUMNS IN MATRIX A. */
/*               THE REMAINING USEFUL COLUMNS FORM A BASE OF MATRIX A. */
/*           (4) IF THE PROCEDURE WAS SUCCESSFUL, ERROR PARAMETER IER */
/*               IS SET TO 0. */

/*        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED */
/*           NONE */

/*        METHOD */
/*           HOUSEHOLDER TRANSFORMATIONS ARE USED TO TRANSFORM MATRIX A */
/*           TO UPPER TRIANGULAR FORM. AFTER HAVING APPLIED THE SAME */
/*           TRANSFORMATION TO THE RIGHT HAND SIDE MATRIX B, AN */
/*           APPROXIMATE SOLUTION OF THE PROBLEM IS COMPUTED BY */
/*           BACK SUBSTITUTION. FOR REFERENCE, SEE */
/*           G. GOLUB, NUMERICAL METHODS FOR SOLVING LINEAR LEAST */
/*           SQUARES PROBLEMS, NUMERISCHE MATHEMATIK, VOL.7, */
/*           ISS.3 (1965), PP.206-216. */

/*     .................................................................. */

/* Subroutine */ int dllsq_(doublereal *a, doublereal *b, integer *m, integer 
	*n, integer *l, doublereal *x, integer *ipiv, real *eps, integer *ier,
	 doublereal *aux)
{
    /* System generated locals */
    integer i__1, i__2, i__3, i__4;

    /* Builtin functions */
    double sqrt(doublereal);

    /* Local variables */
    static doublereal beta;
    static integer iend, kpiv;
    static doublereal h;
    static integer i, j, k, id, ii, lm, ln;
    static doublereal sig, piv, tol;
    static integer ist, jst, kst;



/*     ERROR TEST */
    /* Parameter adjustments */
    --aux;
    --ipiv;
    --x;
    --b;
    --a;

    /* Function Body */
    if (*m - *n >= 0) {
	goto L1;
    } else {
	goto L30;
    }

/*     GENERATION OF INITIAL VECTOR S(K) (K=1,2,...,N) IN STORAGE */
/*     LOCATIONS AUX(K) (K=1,2,...,N) */
L1:
    piv = 0.;
    iend = 0;
    i__1 = *n;
    for (k = 1; k <= i__1; ++k) {
	ipiv[k] = k;
	h = 0.;
	ist = iend + 1;
	iend += *m;
	i__2 = iend;
	for (i = ist; i <= i__2; ++i) {
/* L2: */
	    h += a[i] * a[i];
	}
	aux[k] = h;
	if (h - piv <= 0.) {
	    goto L4;
	} else {
	    goto L3;
	}
L3:
	piv = h;
	kpiv = k;
L4:
	;
    }

/*     ERROR TEST */
    if (piv <= 0.) {
	goto L31;
    } else {
	goto L5;
    }

/*     DEFINE TOLERANCE FOR CHECKING RANK OF A */
L5:
    sig = sqrt(piv);
    tol = sig * abs(*eps);


/*     DECOMPOSITION LOOP */
    lm = *l * *m;
    ist = -(*m);
    i__1 = *n;
    for (k = 1; k <= i__1; ++k) {
	ist = ist + *m + 1;
	iend = ist + *m - k;
	i = kpiv - k;
	if (i <= 0) {
	    goto L8;
	} else {
	    goto L6;
	}

/*     INTERCHANGE K-TH COLUMN OF A WITH KPIV-TH IN CASE KPIV.GT.K */
L6:
	h = aux[k];
	aux[k] = aux[kpiv];
	aux[kpiv] = h;
	id = i * *m;
	i__2 = iend;
	for (i = ist; i <= i__2; ++i) {
	    j = i + id;
	    h = a[i];
	    a[i] = a[j];
/* L7: */
	    a[j] = h;
	}

/*     COMPUTATION OF PARAMETER SIG */
L8:
	if (k - 1 <= 0) {
	    goto L11;
	} else {
	    goto L9;
	}
L9:
	sig = 0.;
	i__2 = iend;
	for (i = ist; i <= i__2; ++i) {
/* L10: */
	    sig += a[i] * a[i];
	}
	sig = sqrt(sig);

/*     TEST ON SINGULARITY */
	if (sig - tol <= 0.) {
	    goto L32;
	} else {
	    goto L11;
	}

/*     GENERATE CORRECT SIGN OF PARAMETER SIG */
L11:
	h = a[ist];
	if (h >= 0.) {
	    goto L13;
	} else {
	    goto L12;
	}
L12:
	sig = -sig;

/*     SAVE INTERCHANGE INFORMATION */
L13:
	ipiv[kpiv] = ipiv[k];
	ipiv[k] = kpiv;

/*     GENERATION OF VECTOR UK IN K-TH COLUMN OF MATRIX A AND OF */
/*     PARAMETER BETA */
	beta = h + sig;
	a[ist] = beta;
	beta = 1. / (sig * beta);
	j = *n + k;
	aux[j] = -sig;
	if (k - *n >= 0) {
	    goto L19;
	} else {
	    goto L14;
	}

/*     TRANSFORMATION OF MATRIX A */
L14:
	piv = 0.;
	id = 0;
	jst = k + 1;
	kpiv = jst;
	i__2 = *n;
	for (j = jst; j <= i__2; ++j) {
	    id += *m;
	    h = 0.;
	    i__3 = iend;
	    for (i = ist; i <= i__3; ++i) {
		ii = i + id;
/* L15: */
		h += a[i] * a[ii];
	    }
	    h = beta * h;
	    i__3 = iend;
	    for (i = ist; i <= i__3; ++i) {
		ii = i + id;
/* L16: */
		a[ii] -= a[i] * h;
	    }

/*     UPDATING OF ELEMENT S(J) STORED IN LOCATION AUX(J) */
	    ii = ist + id;
	    h = aux[j] - a[ii] * a[ii];
	    aux[j] = h;
	    if (h - piv <= 0.) {
		goto L18;
	    } else {
		goto L17;
	    }
L17:
	    piv = h;
	    kpiv = j;
L18:
	    ;
	}

/*     TRANSFORMATION OF RIGHT HAND SIDE MATRIX B */
L19:
	i__2 = lm;
	i__3 = *m;
	for (j = k; i__3 < 0 ? j >= i__2 : j <= i__2; j += i__3) {
	    h = 0.;
	    iend = j + *m - k;
	    ii = ist;
	    i__4 = iend;
	    for (i = j; i <= i__4; ++i) {
		h += a[ii] * b[i];
/* L20: */
		++ii;
	    }
	    h = beta * h;
	    ii = ist;
	    i__4 = iend;
	    for (i = j; i <= i__4; ++i) {
		b[i] -= a[ii] * h;
/* L21: */
		++ii;
	    }
	}
    }
/*     END OF DECOMPOSITION LOOP */


/*     BACK SUBSTITUTION AND BACK INTERCHANGE */
    *ier = 0;
    i = *n;
    ln = *l * *n;
    piv = 1. / aux[*n * 2];
    i__4 = ln;
    i__3 = *n;
    for (k = *n; i__3 < 0 ? k >= i__4 : k <= i__4; k += i__3) {
	x[k] = piv * b[i];
/* L22: */
	i += *m;
    }
    if (*n - 1 <= 0) {
	goto L26;
    } else {
	goto L23;
    }
L23:
    jst = (*n - 1) * *m + *n;
    i__3 = *n;
    for (j = 2; j <= i__3; ++j) {
	jst = jst - *m - 1;
	k = *n + *n + 1 - j;
	piv = 1. / aux[k];
	kst = k - *n;
	id = ipiv[kst] - kst;
	ist = 2 - j;
	i__4 = *l;
	for (k = 1; k <= i__4; ++k) {
	    h = b[kst];
	    ist += *n;
	    iend = ist + j - 2;
	    ii = jst;
	    i__2 = iend;
	    for (i = ist; i <= i__2; ++i) {
		ii += *m;
/* L24: */
		h -= a[ii] * x[i];
	    }
	    i = ist - 1;
	    ii = i + id;
	    x[i] = x[ii];
	    x[ii] = piv * h;
/* L25: */
	    kst += *m;
	}
    }


/*     COMPUTATION OF LEAST SQUARES */
L26:
    ist = *n + 1;
    iend = 0;
    i__4 = *l;
    for (j = 1; j <= i__4; ++j) {
	iend += *m;
	h = 0.;
	if (*m - *n <= 0) {
	    goto L29;
	} else {
	    goto L27;
	}
L27:
	i__3 = iend;
	for (i = ist; i <= i__3; ++i) {
/* L28: */
	    h += b[i] * b[i];
	}
	ist += *m;
L29:
	aux[j] = h;
    }
    return 0;

/*     ERROR RETURN IN CASE M LESS THAN N */
L30:
    *ier = -2;
    return 0;

/*     ERROR RETURN IN CASE OF ZERO-MATRIX A */
L31:
    *ier = -1;
    return 0;

/*     ERROR RETURN IN CASE OF RANK OF MATRIX A LESS THAN N */
L32:
    *ier = k - 1;
    return 0;
} /* dllsq_ */

