/*
 *  EBUPDAT1
 *
 *      This subroutine takes original image and 
 *      the function definition (cvec,funeval) and computes
 *      the surface function error at each pixel.
 *      (dltimg) is the dilated image.
 *
 *  Author:     Paul J. Besl    (10-12-85)
 *
 *  Revised:    Arnold. H. Chiu (7-27-88)
 *  
 *  Call:           ebupdat1(image_pointer,dilate_image,x_size,y_size,
 *                           cvec,function_eval,error_buffer,
 *                           surface_image,region_labels,i_label)
 *  Modules used:   none
 *
 */

#include <math.h>

int ebupdat1(imgptr,dltimg,nx,ny,cvec,funeval,errbuf,surfimg,rgnlabls,ilabl)
unsigned char   *imgptr,*dltimg,*surfimg,*rgnlabls;
double          (*funeval)(),cvec[];
int             nx,ny,ilabl;
float           *errbuf;
{
    register int            i,count,ntot;
    register unsigned char  *iptr;
    double                  error,xval,yval,zval,zfun;
    unsigned char           *srfptr,*rgnptr,*dltptr;
    float                   *errptr;

    /* Loop over all data points */

    count = 0;
    ntot = nx*ny;
    iptr = imgptr;
    errptr = errbuf; 
    srfptr = surfimg;
    rgnptr = rgnlabls;
    dltptr = dltimg;
    for(i=0;i<ntot;++i)
    {
        if( *dltptr++ )
        {
            xval = i % nx;
            yval = i / nx;
            zval = *iptr++;  
            zfun = (*funeval)(xval,yval,cvec);
            error = fabs( zfun - zval );
            if( zfun < 0.5 )
                zfun = 0.5;     /* hard limit */
            else if( zfun > 255.4 )
                zfun = 255.4;
    
            /* Update Images Unconditionally */

            *errptr++ = error;
            *rgnptr++ = ilabl;
            *srfptr++ = (int)(zfun+0.5);
            count++;
        }
        else
        {
            iptr++;
            errptr++;
            srfptr++;
            rgnptr++;
        }
    }
    return(count);
}
