/*
 *  EBUPDAT2
 *
 *      This subroutine takes original image and 
 *      the function definition (cvec,funeval) and computes
 *      the surface function error at each pixel.
 *      (dltimg) is the dilated image less (basimg).
 *
 *  Author:     Paul J. Besl    (10-12-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           ebupdat2(image_pointer,dilate_image,base_image,
 *                           mark_image,x_size,y_size,cvec,function_eval,
 *                           error_buffer,surfacce_image,region_labels,
 *                           i_label,error_limit)
 *  Modules used:   imgor2(),   imgsub2(),  rgnbri()
 *
 */

#include <math.h>

int ebupdat2(imgptr,dltimg,basimg,mrkimg,nx,ny,cvec,funeval,errbuf,surfimg,rgnlabls,ilabl,errlim)
unsigned char   *imgptr,*dltimg,*surfimg,*rgnlabls,*basimg,*mrkimg;
double          (*funeval)(),cvec[];
float           *errbuf,errlim; 
int             nx,ny,ilabl;
{
    register float          *errptr;
    register int            i,count,ntot;
    register unsigned char  *iptr,*mrkptr;
    double                  error,xval,yval,zval,zfun;
    unsigned char           *srfptr,*rgnptr,*dltptr;
    int                     first,last,pxlcnt;

    /* Mark Potential Neighboring Pixels in Mrkimg */

    count = 0;
    ntot = nx*ny;
    first = 0;
    iptr = imgptr;
    dltptr = dltimg;
    errptr = errbuf;
    mrkptr = mrkimg;
    for(i=0;i<ntot;++i)
    {
        if( *dltptr++ )
        {
            if( first == 0 )
                first = i;
            xval = i % nx;
            yval = i / nx;
            zval = *iptr++;  
            zfun = (*funeval)(xval,yval,cvec);
            error = fabs( zfun - zval );

            /* Mark Potential Pixels to be Connectivity Constrained*/

            if( error < *errptr++ && error < errlim )
                *mrkptr++ = 255;
            else
                *mrkptr++ = 0;
            last = i;
        }
        else
        {
            iptr++;
            errptr++;
            *mrkptr++ = 0;
        }
    }

    /* OR in base image and find largest connected region */

    imgor2(mrkimg,basimg,nx,ny);
    pxlcnt = rgnbri(mrkimg,dltimg,nx,ny,255,1);
    imgsub2(dltimg,basimg,nx,ny);

    /* Mark Potential Neighboring Pixels in Tmpimg */

    iptr = imgptr + first;
    errptr = errbuf + first; 
    srfptr = surfimg + first;
    rgnptr = rgnlabls + first;
    dltptr = dltimg + first;
    mrkptr = mrkimg + first;
    ntot = ++last;
    for(i=first;i<ntot;++i)
    {
        if( *dltptr++ )
        {
            xval = i % nx;
            yval = i / nx;
            zval = *iptr++;  
        	zfun = (*funeval)(xval,yval,cvec);
        	error = fabs( zfun - zval );
        	if( zfun < 0.5 )
                zfun = 0.5;     /* hard limit */
        	else if( zfun > 255.4 )
                zfun = 255.4;

        	/* Update Images */

        	if( error < *errptr && error < errlim )
        	{
                *errptr++ = error;
                *rgnptr++ = ilabl;
                *srfptr++ = (int)(zfun+0.5);
                count++;
            }
            else
            {
                errptr++;
                srfptr++;
                rgnptr++;
            }
        }
        else
        {
            iptr++;
            errptr++;
            srfptr++;
            rgnptr++;
        }
    }
    printf("%d Connected_Pixels within Error_Limit %g\n",count,errlim);
    return(count);
}
