/*
 *  FLTUCIMG
 *
 *  This subroutine accepts a pointer to a float image (fltimg)
 *  and converts it to an unsigned char image (imgptr) in the
 *  space provided.  The images are of size (nx x ny) with
 *  maximum value fmax and minimum value fmin.
 *
 *  Author:     Paul J. Besl    (5-5-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           fltucimg(imgptr,fltimg,nx,ny,fltmax,fltmin)
 *  Modules used:   imgset()    fmaxmin()
 *
 */

/* stop listing of <stdio.h> */
/* #list */
#include <math.h>
#include <stdio.h>
/* #nolist */

int fltucimg(imgptr,fltimg,nx,ny,fltmax,fltmin)
float           *fltimg,*fltmax,*fltmin;
unsigned char   *imgptr;
int             nx,ny;
{
    register int            i,ntot;
    register unsigned char  *ptr;
    register float          scale;
    float                   fmax,fmin;

    /* compute maximum and minimum of image */

    ntot = nx * ny;
    fmaxmin(fltimg,ntot,&fmax,&fmin);
    *fltmax = fmax;
    *fltmin = fmin;
    if(fmax == fmin)
    {
        imgset(imgptr,128,nx,ny);
        return(1);
    }

    /* compute scaling factor & convert float image to byte image */

    scale = 255.0/(fmax-fmin);
    ptr = imgptr;
    for(i=0; i<ntot; ++i)
        *ptr++ = (int)((scale * ( (*fltimg++) - fmin)) + 0.5);

    /* Print Scaling Results */

    fprintf(stderr,"<Max= %g><Min= %g><Dif= %g>\n",fmax,fmin,fmax-fmin);
    return(0);
}
