/*
 *  FUNCHK
 *
 *      This subroutine takes the (x,y,z) data arrays and
 *      the function definition (cvec,funeval) and computes
 *      the surface function at each pixel to find
 *      the predicted surface value.
 *      Returns Flag indicates 2-D Runs Test Results.
 *
 *  Author:     Paul J. Besl    (11-29-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           funchk(x_array,y_array,z_array,num_pixels,cvec,
 *                         function_eval,e1,e2,e8,image_pointer,
 *                         x_size,y_size,dzdisp,test_value)
 *  Modules used:   contract(),     rgnbrc()
 *
 */

#include <math.h>

int funchk(xray,yray,zray,npixels,cvec,funeval,e1,e2,e8,imgptr,nx,ny,
	   dzdisp,testval)
float           *xray,*yray,*zray,testval;
double          (*funeval)(),cvec[]; 
unsigned char   *imgptr;
double          *e1,*e2,*e8;
int             npixels,nx,ny,dzdisp;
{
    unsigned char   *ucimg1,*ucimg2,*calloc();
    register float  *xptr,*yptr,*zptr;
    double          xval,yval,zval,zfun;
    double          avgerr,sqrerr,toterr,error,errmax;
    register int    i;
    int             ix,iy,tsecs,tstart(),runflg;
    int             count,poscnt,negcnt,cnthres;
    float           pct1,pct2,thres;

    /* Loop over all data points */

    tsecs = tstart();
    if( npixels <= 0 )
    {
        *e1 = *e2 = *e8=0.0;
        return(0);
    }
    errmax = toterr = sqrerr = avgerr = 0.0;
    poscnt = negcnt = 0;
    xptr = xray;
    yptr = yray;
    zptr = zray;
    ucimg1 = calloc(nx*ny,1);
    ucimg2 = calloc(nx*ny,1);
    for(i=0;i<npixels;++i)
    {
        ix = xval = (*xptr++);
        iy = yval = (*yptr++);
        zval = (*zptr++);  
        zfun = (*funeval)(xval,yval,cvec);
        error = ( zfun - zval );
        if( error > 0.0 )
            *(ucimg1+nx*iy+ix)=255;
        else
            *(ucimg2+nx*iy+ix)=255;
        avgerr += error;
        error = fabs( error );
        if( error > errmax )
            errmax = error;
        toterr += error;
        sqrerr += error*error;
        /*printf("x=%g y=%g z=%g zcmp=%g err=%g\n",xval,yval,zval,zfun,error);*/
    }
    toterr /= (float)npixels;       avgerr /= (float)npixels;
    sqrerr /= (float)npixels;       sqrerr = sqrt( sqrerr );
    *e1 = toterr;   *e2 = sqrerr;   *e8 = errmax;
    printf("e0=%g < e1=%g < e2=%g < e8= %g\n",avgerr,toterr,sqrerr,errmax);
    thres = testval;
    cnthres = thres * npixels - 1;
    
    /* if( dzdisp ) dspsub2(0,ucimg1,nx); */
    count = contract(ucimg1,imgptr,nx,ny);
    if( count > cnthres )
    {
        poscnt = rgnbrc(imgptr,nx,ny,0,0);
        pct1 = (float)poscnt/(float)npixels;
        printf("%d Cntrctd +Pxls, %d Lrg_Pos_Rgn %g\n",count,poscnt,pct1);
    }
    else
        pct1 = 0.0;

    count = contract(ucimg2,imgptr,nx,ny);
    if( count > cnthres ) 
    {
        negcnt = rgnbrc(imgptr,nx,ny,0,0);
        pct2 = (float)negcnt/(float)npixels;
        printf("%d Cntrctd -Pxls, %d Lrg_Neg_Rgn %g\n",count,negcnt,pct2);
    }
    else
        pct2 = 0.0;

    /* if( dzdisp ) dspsub2(0,ucimg2,nx); */
    free(ucimg2);
    free(ucimg1);

    tstop(tsecs,"Time_to_do_Runs_Test+Fun.Chk."); 
    if( pct1 > thres && pct1 >= pct2 ) 
    {
        printf("(+) Incr. Fit Order\n");  count = 100.0*pct1;
        return(count);
    }
    if( pct2 > thres ) 
    {
        printf("(-) Incr. Fit Order\n");  count = 100.0*pct2;
        return(count);
    }
    return(0);
}
