/*
 *  GETIMAGE
 *
 *      This subroutine reads the file by the name filename
 *      into a n x n virtual memory array created by this
 *      routine using malloc.  The pointer to the start of
 *      the array is returned to the calling program.
 *
 *  Author:     Paul J. Besl    (12-6-84)
 *
 *  Revised:    Arnold H. Chiu  (7-12-88)
 *
 *  Call:       getimage(file_name,image_size_read_in)
 *
 */

/* skip listing <stdio.h> <math.h> */
/*#nolist*/
#include <stdio.h>
#include <math.h>
/*#list*/

#define DEF_IMG_DIM     512     /* default image dimension <getimage> */
 
unsigned char *getimage(filename,nsize)
char    *filename;
int     *nsize;
{
    register unsigned char  *inptr;     /* pointer to image     */
    unsigned char           *malloc();  /* memory allocator     */
    unsigned char           *calloc();
    register int            nsqr;       /* image size constant  */
    register int            nread;      /* number of bytes read */ 
    int                     size;       /* image dimension      */
    FILE                    *filptr;    /* file pointer         */

    /* open file, exit if not successful */

    filptr = fopen(filename,"r");
    if (filptr == NULL)
    {
        fprintf(stderr,"getimage: Can't Open File '%s'\n",filename);
        return(NULL);
    }

    /* determine size of square image in file */

    fseek(filptr,0L,2);
    nsqr = ftell(filptr);
    if (nsqr < 4)
        nsqr = DEF_IMG_DIM*DEF_IMG_DIM;
    size = sqrt( (double)nsqr );

    /* fill up input image array with pixel data */

    inptr = malloc(nsqr);
    if (inptr == NULL)
    {
        fprintf (stderr,"getimage: Malloc Failed\n");
        fclose(filptr);
        return (NULL);
    }
    fseek(filptr,0L,0);
    nread = fread(inptr,1,nsqr,filptr);
    fclose(filptr);

    /* if error on read, exit and notify user of error */

    if (nread != nsqr)
    {
        fprintf(stderr,"getimage: Input Read Error (actual:%d  requested:%d)\n",nread,nsqr);
        free(inptr);
        return (NULL);
    }

    /* return pointer to calling program */

    fprintf(stderr,"getimage: %dx%d image read from '%s'\n",size,size,filename);
    *nsize = size;
    return(inptr);
}
