/*
 *  MATNRMD
 *
 *  This routine performs double precision nXn matrix euclidean norm.
 *
 *  Author:     Paul J. Besl    (4-15-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           matnrmd(a_pointer,number,off_diagonal,on_diagonal,total)
 *  Modules used:   none
 *
 */

/* stop listing <math.h> */
/* #nolist */
#include <math.h>
/* #list */

void matnrmd(aptr,n,ofdiag,ondiag,total)
double  *aptr,*ofdiag,*ondiag,*total;
int     n;
{
    double              anorm,wnorm,vnorm;
    register double     *a,aii,aij;
    register int        i,j,k;
    int                 nsqr = n*n;

    a = aptr;
    vnorm = wnorm = 0.0;

    /* compute on- and off-diagonal norms of a[][] */

    for(k=0; k<nsqr; ++k)
    {
        /* row index=i and column index=j */

        i=k/n;
        j=k%n;
        if(i == j)          /* on-diagonal */
        {
            aii = *a++;
            wnorm  += aii*aii;
        }
        else                /* off-diagonal */
        {
            aij = *a++;
            vnorm  += aij*aij;
        }
    }

    /* compute matrix norms with square roots */

    anorm = vnorm + wnorm;
    anorm = sqrt(anorm);
    vnorm = sqrt(vnorm);
    wnorm = sqrt(wnorm);
    *ofdiag = vnorm;
    *ondiag = wnorm;
    *total  = anorm;
}
