/*
 *  MATSINVD
 *
 *      This routine performs double precision matrix symmetric
 *      inverse (in-place algorithm)
 *              -1
 *      [a] = [a]     { [a]=(nxn) }
 *      Returns Matrix Rank of [a]
 *
 *  Author:     Paul J. Besl    (6-29-84)
 *
 *  Revised:    Arnold. H. Chiu (7-27-88)
 *  
 *  Call:           matsinvd(a_pointer,n)
 *  Modules used:   matmuld(),  mattpsd(),  matcpyd()
 *
 */

/* stop listing <stdio.h> <math.h> */
/* #nolist */
#include <stdio.h>
#include <math.h>
/* #list */

int matsinvd(aptr,n)
double  *aptr; int n;
{
    register int        i,n1 = n + 1;
    double              *ptr,*rptr;
    double              *mptr;
    int                 rank,nsqr = n*n;
    unsigned char       *malloc();
    double              epsilon = 1.0e-12;

    /* allocate memory for extra matrices */

    rptr = (double *)malloc(8*nsqr);
    mptr = (double *)malloc(8*nsqr);

    /* solve for eigenvalues of symmetric matrix */

    /*printf("matsinvd: now executing\n");*/
    /*matwrfd0(stdout,aptr,n,n,"org");*/
    /*printf("Calling Eigend\n");*/

    eigend(aptr,rptr,n);

    /*printf("Back From Eigend\n");*/
    /*matwrfd0(stdout,aptr,n,n,"diag");*/
    /*matwrfd0(stdout,rptr,n,n,"rot");*/

    /* invert diagonal matrix */

    rank = 0;
    for(i=0;i<n;++i)
    {
        ptr =  aptr + (i * n1);
        if(fabs(*ptr) < epsilon)
        {
            /* zero eigenvalue - perform pseudo-inverse operation */

            printf("0_Eigenvalue_Encountered!(%d)(%g)\n",i,*ptr);
            *ptr = 0.0;
        }
        else
        {
            *ptr = 1.0/(*ptr);
            rank++;
        }
    }
    if( rank != n )
        printf("Matrix_Rank= %d (%d)\n",rank,n);
    /*matwrfd0(stdout,aptr,n,n,"invdiag");*/

    /* carry out matrix multiplication to get inverse matrix */

    matmuld(mptr,rptr,aptr,n,n,n);      /* [m] = [r][a] */
    mattpsd(aptr,rptr,n,n);             /* [a] = [r]T   */
    matcpyd(rptr,aptr,n,n);             /* [r] = [a]    */
    matmuld(aptr,mptr,rptr,n,n,n);      /* [a] = [m][r] = [r][a][r]T */
    /*matwrfd0(stdout,aptr,n,n,"inv");*/
    free(mptr);
    free(rptr);
    return(rank);
}
