/*
 *  OKTOFIT
 *
 *  This subroutine incorporates all checks that are necessary to
 *  decide if it is OK to continue fitting and growing.
 *  Returns 0 if time to stop and 1 if ok to continue.
 *
 *  Author:     Paul J. Besl    (8-12-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           oktofit(lastccnt,thisccnt,lastscnt,thisscnt,errtol,errmax,
 *                          numiter,maxiter,thisnpts,lastnpts,fitmode,fitlast)
 *  Modules used:   none
 *
 */

/* skip listing <math.h> */
/* #nolist */
#include <math.h>
/* #list */

int oktofit(lastccnt,thisccnt,lastscnt,thisscnt,errtol,errmax,
            numiter,maxiter,thisnpts,lastnpts,fitmode,fitlast)
int     lastccnt,thisccnt,lastscnt,thisscnt,numiter,maxiter;
int     thisnpts,lastnpts,fitmode,fitlast;
float   errtol,errmax;
{
    float pct;

    if( fitmode > 3 )
        return(0);
    if( fitmode != fitlast )
        return(1);
    if( thisccnt <= 3 )
        return(0);
    if( thisscnt <= 3 )
        return(0);
    if( thisnpts <= 3 )
        return(0);

    /* Check to See if All Tests Can Be Passed */

    if( numiter >= maxiter )
        return(0);
    if( errtol  >  errmax )
        return(0);
    if( lastccnt == thisccnt )
        return(0);
    if( lastscnt == thisscnt )
        return(0);
    if( lastnpts == thisnpts )
        return(0);
    if( numiter > 3 )
    {
        if( abs(thisccnt-lastccnt) < 4 )
            return(0);
        if( abs(thisscnt-lastscnt) < 4 )
            return(0);
        if( abs(thisccnt-thisscnt) < 4 )
            return(0);
        if( abs(thisccnt-thisscnt) < 2 )
            return(0);
    }

    /* New Percentage Tests for Extremely Large Regions */
    if( numiter > 5 )
    {
        pct = 2.0*abs( thisccnt - lastccnt )/(float)(thisccnt+lastccnt);
        printf("PctC= %g  ",pct);
        /* one-tenth of one percent */
        if( pct < 0.002 )
            return(0);

        pct = 2.0*abs( thisscnt - lastscnt )/(float)(thisscnt+lastscnt);
        printf("PctS= %g\n",pct);
        /* one-tenth of one percent */
        if( pct < 0.002 )
            return(0);
    }
    /* All Tests Have Been Passed;  OK to Keep on Fitting */
    return(1);
}
