/*
 *  OUTMENU
 *
 *      This routine performs menu command processing for
 *      output options.  All strings are in routine (outmenucmd)
 *  
 *  Author:     Paul J. Besl    (2-13-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           outmenu(output_options,string,
 *                          string_size,default_filenames)
 *  Modules used:   savstring(),    substr(),   ttyclrhom()
 *
 */

/* stop listing <stdio.h> */
/* #nolist */
#include <stdio.h>
/* #list */

#define NUMCMDS     56
#define GOBACK      55
#define RETURN      54
#define LIST        53
#define ALLOFF      52
#define ALLONN      51

int outmenu(outoptions,string,strsize,dfnames)
int     *outoptions,strsize;
char    *string,*dfnames[];
{
    /* default switch settings for output files */

    static int outinit[NUMCMDS] =
    {   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
        0, 0, 0, 0, 0, 0, 0, 1, 0, 0,
        0, 1, 0, 0, 0, 1, 0, 0, 0, 0,
        0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0                };

    static char onoff[2][4] = { "off","on" };

    int             nargs,icase,defcase,value;
    register int    i,j;

    /* set up default switch settings */

    for(i=0; i<NUMCMDS; ++i)
        outoptions[i] = outinit[i];
    icase = -1;
    defcase = LIST;

    /* Enter Command Prompt Loop */

enter_command:

    printf("\nEnter Output Option(%s)__ ",outmenucmd(defcase,0));
    if( fgets(string,strsize,stdin) == NULL )
    {
        printf("\n");
        exit(1);
    }

    /* Handle Default Case  (nargs=0 --> mnemonic entered) */

    if( strlen(string) <= 1 )
    {
        icase = defcase;
        nargs = 1;
    }
    else
        nargs = sscanf(string,"%d",&icase);

    /* Number Specification Case and Default Case */

    /* printf("nargs=%d icase=%d\n",nargs,icase); */
    if( nargs == 1 && ( icase > -1 && icase < NUMCMDS ) )
    {
        if( icase == LIST )
            goto list_menu;
        savstring(string,0,dfnames[0]);
        if( icase == RETURN )
            return(0);
        if( icase == GOBACK )
            return(-1);
        if( icase == ALLOFF )
        {
            value = 0;
            goto set_all;
        }
        if( icase == ALLONN )
        {
            value = 1;
            goto set_all;
        }
        outoptions[icase] = 1 - outoptions[icase];      /* toggle */
        printf("%s = %s\n",outmenucmd(icase,1),onoff[outoptions[icase]]);
        goto enter_command;
    }

    /* Mnemonic Specification */

    icase = -1;
    if( substr(string,"?") > -1 )
        goto list_menu;
    for(i=0; i< NUMCMDS; ++i)
    {
        if( substr(string,outmenucmd(i,0)) > -1 ) 
        {
            icase = i;          /* six lines of code duplicated below */
            if( icase == LIST )
                goto list_menu;
            savstring(string,0,dfnames[0]);
            if( icase == RETURN )
                return(0);
            if( icase == GOBACK )
                return(-1);
            if( icase == ALLOFF )
            {
                value = 0;
                goto set_all;
            }
            if( icase == ALLONN )
            {
                value = 1;
                goto set_all;
            }
            outoptions[icase] = 1 - outoptions[icase];  /* toggle */
            printf("%s = %s\n",outmenucmd(icase,1),onoff[outoptions[icase]]);
            goto enter_command;
        }
    }
    fprintf(stderr,"Unknown Command: %s",string);
    goto enter_command;

    /*==============================*/
    /* List Out Commands and Status */
    /*==============================*/

list_menu:

    ttyclrhom();
    printf("Enter All File Output Options by Number or Mnemonic\n");
    printf("Reverse Video Options are ON, Others OFF\n");
    i=0;
    while (i < NUMCMDS )
    {
        j=0;
        while( i < NUMCMDS && j < 3 )
        {
           if( outoptions[i] == 1 ) 
                printf("%cp%2d:%s%cq-%s",27,i,outmenucmd(i,0),27,outmenucmd(i,1));
            else 
               printf("%2d:%s-%s",i,outmenucmd(i,0),outmenucmd(i,1));
            ++i;
            ++j;
        }
        printf("\n");
    }
    goto enter_command;

    /*===============================*/
    /* Set All Commands to On or Off */
    /*===============================*/

set_all:

    for(i=0; i<NUMCMDS; ++i)
        outoptions[i] = value;
    goto enter_command;
}
