/*
 *  PICKFRGN
 *
 *      This Routine Choose Region and Surface Type For Next Fit Attempt.
 *      Use histogram and largest region counts to choose the next
 *      region to use for surface fitting.Returns Surface Type (surftype).
 *      Sets up HK-hklabl array
 *
 *  Author:     Paul J. Besl    (9-9-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           pickfrgn(stypimg,blakbord,nx,ny,hklabl,hkhist,hkrcnt,histogram,
 *                           mapcode,scode,minsz,errmax)
 *  Modules used:   imghist(), dumbsort(), rgnbrc()
 *
 */

/* skip listing <stdio.h> */
/* #nolist */
#include <stdio.h>
/* #list */

int pickfrgn(stypimg,blakbord,nx,ny,hklabl,hkhist,hkrcnt,histogram,
             mapcode,scode,minsz,errmax)    
unsigned char   *stypimg,*blakbord;
int             nx,ny,minsz;
int             hklabl[],hkhist[],hkrcnt[],histogram[],mapcode[];
char            *scode[];
float           *errmax;
{
    int             tstart(),rgnbrc();
    register int    i;
    int             largest = 0;
    int             surftype,tsecs;

    /* Check for Black Board Processing Mode (Exception) */

    if( hklabl[0] != 9 )
    {
        /* Standard Processing: Sort Histogram, Then Find
                                Largest Region of Any Type */

        hklabl[9] = 9;
        tsecs = tstart();
        imghist(stypimg,nx,ny,histogram);
        for(i=0;i<9;++i)
        {
            hklabl[i] = i;
            hkhist[i] = histogram[mapcode[i]];
        }
        dumbsort(hkhist,hklabl,9);
        for(i=0;i<9;++i)
        {
            if( hkhist[i] > largest )
            {
                hkrcnt[i] = rgnbrc(stypimg,nx,ny,mapcode[hklabl[i]],1);
                if( hkrcnt[i] > largest )
                    largest = hkrcnt[i];
            }
            else /* less than or equal to largest */
                hkrcnt[i] = hkhist[i];
            fprintf(stdout,"%d, ",hkrcnt[i]);
            fflush(stdout);
        }
        fprintf(stdout,"\n");
        dumbsort(hkrcnt,hklabl,9);
        tstop(tsecs,"Time to Compute Largest Region Info");
        printf("%d %s Pixels in Largest Region\n",
               hkrcnt[0],scode[hklabl[0]]);
        printf("%d %s Next Largest Pixel Count\n",
               hkrcnt[1],scode[hklabl[1]]);

        if( hkrcnt[0] < minsz )  /* check minimum initial size */
        {
            /* begin black board processing (9) */
            hklabl[0] = 9;
            surftype = 255;
            imgcpy(stypimg,blakbord,nx,ny);
            printf("\n*** Now POST-PROCESSING BlackBoard ***\n");
            (*errmax) *= 2.0;  /* double tolerance for last ditch efforts */
        }
        else
        {
            /* normal case */
            surftype = mapcode[ hklabl[0] ]; 
            printf("\n*** PROCESSING %s REGION of SIZE %d ***\n",
                    scode[ hklabl[0] ],hkrcnt[0]);
        }
    } /* End of Not-Yet Black Board If Statement */

    /* Return Surface Type to Calling Program */

    return(surftype);
}
