/*
 *  PLNSURFIT
 *
 *      This subroutine takes the (x,y,z) data arrays and the mean
 *      x,y values (xmean,ymean) fits a planar surface function with
 *      3 coefficients which are returned in (cvec) vector.
 *      Orthogonal Polynomial Coefficients Returned in (ovec)
 *
 *  Author:     Paul J. Besl    (4-7-85)
 *
 *  Revised:    Arnold. H. Chiu (7-26-88)
 *  
 *  Call:           plnsurfit(x_array,y_array,z_array,num_pixels,
 *                            x_mean,y_mean,errtol,x_size,y_size,
 *                            cvec,opt,ovec)
 *  Modules used:   linlsq(),   matcpyd()
 *
 */

#define ARSIZE  3

plnsurfit(xray,yray,zray,npixels,xmean,ymean,errtol,nx,ny,cvec,opt,ovec)
float   *xray,*yray,*zray,xmean,ymean,*errtol; 
int     npixels,nx,ny,opt;
double  cvec[],ovec[];
{
    unsigned char       *malloc();
    float               *xptr,*yptr,*zptr;
    double              *phiray,*zdblvec,error;
    register double     *phiptr,xval,yval,*zdblptr;
    register int        i;
    int                 rank,arsize = ARSIZE;
    double              a,b,c;

    if( npixels < arsize )
    {
        printf("=> %d Data Points needed for Surface Fit\n",arsize);
        exit(1);
    }

    /* Allocate memory for npixels x arsize double float array. */

    phiray  = (double *)malloc(8*arsize*npixels);
    zdblvec = (double *)malloc(8*npixels);

    /* Loop over all data points filling in PHI array */

    xptr = xray;
    yptr = yray;
    zptr = zray;
    phiptr = phiray;
    zdblptr = zdblvec;
    for(i=0;i<npixels;++i)
    {
        xval = (*xptr++) - xmean;
        yval = (*yptr++) - ymean;
        *zdblptr++ = *zptr++;
        *phiptr++ = 1.0;           /* 0 */
        *phiptr++ = xval;          /* 1 */
        *phiptr++ = yval;          /* 2 */
    }

    /* Compute Linear Least Squares Solution */

    rank = linlsq(cvec,phiray,zdblvec,npixels,arsize,opt,&error);
    matcpyd(ovec,cvec,arsize,1);
    a = cvec[0] - (cvec[1]*xmean + cvec[2]*ymean);
    b = cvec[1];
    c = cvec[2];

    /*
    printf("\nz(x,y) = a + bx + cy\n");
    printf(" a= %g   b= %g   c= %g\n",a,b,c);
    */
    cvec[0] = a;
    cvec[1] = b;
    cvec[2] = c;
    printf("[Planar_Fit] RMS_Error= %g\n",error);
    free( zdblvec );
    free( phiray );
    *errtol = error;
}
