/*
 *  RGNBRC
 *
 *      This routine finds the biggest region pixel count.
 *      It's part of finding 4-connected regions in binary
 *      image in two passes.  Reports number of pixels in biggest
 *      region.  Assumes that regions are non-zero regions.
 *      Given the pointer (imgptr) to a rectangular (nx x ny) image
 *      this subroutine computes the number of pixels in the largest
 *      region and returns that value.
 *      The region pixels have values equal to (rgnval) for
 *      (eqlflg=1), not equal to (rgnval) when (eqlflg=0).
 *
 *  Author:     Paul J. Besl    (8-18-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           rgnbrc(image_ptr,x_size,y_size,region_value,equal_flag)
 *  Modules used:   rgnprc2()       rgnlbln()       rgnlble()
 *                  imgbdrset()     dumbsort()
 *
 */
 
#define  EQUIVLABLSIZ  65536  /* unsigned short max */

int rgnbrc(imgptr,nx,ny,rgnval,eqlflg)
unsigned char   *imgptr;
int             nx,ny,rgnval,eqlflg;
{
    unsigned char                   *malloc();
    int                             *rgnprc2(),rgnlbln(),rgnlble();
    register int                    i,j;
    register unsigned short int     *lptr;
    unsigned short int              *imglabels,*equivlabl;
    int                             *regnlist;
    int                             npixels,eqlblsiz,iblack;
    int                             ntot,nregion,lenrlist,total,numpixtot,ilabel;

    /* allocate memory for neighbor marker image */

    ntot = nx*ny;
    imglabels = (unsigned short int *)malloc(sizeof(short)*ntot);
    lptr = imglabels;
    for(i=0; i<ntot; ++i)    /* zero out array */
        *lptr++ = 0;

    /* allocate memory for equivalent label list */

    eqlblsiz = EQUIVLABLSIZ;
    equivlabl = (unsigned short int *)malloc(sizeof(short)*eqlblsiz);
    lptr = equivlabl;
    for(i=0;i<eqlblsiz;++i)
        *lptr++ = i;

    /* black out border to insure algorithm assumptions */

    if( eqlflg == 1 && rgnval == 0 )
        iblack = 1;
    else
        iblack = 0;
    imgbdrset(imgptr,nx,ny,3,iblack);

    /* label regions as 1st pass of 2-pass algorithm */
    if( eqlflg == 1 )
        nregion = rgnlble(imgptr,nx,ny,rgnval,imglabels,equivlabl,eqlblsiz,&numpixtot);
    else  /* eqlflg == 0 */
        nregion = rgnlbln(imgptr,nx,ny,rgnval,imglabels,equivlabl,eqlblsiz,&numpixtot);
    if( nregion == 0 )
        return(0);

    /* process regions from labeled image:
       get number of pixels, (x,y)min, (x,y)max for each region */ 

    regnlist = rgnprc2(imglabels,nx,ny,equivlabl,nregion);

    /* Find Biggest Region */

    dumbsort(regnlist+1,equivlabl,nregion);
    npixels = regnlist[1];

    /* print out info and return */
    free(regnlist);
    free(equivlabl);
    free(imglabels);
    return(npixels);
}
