/*
 *  RGNFILERR
 *
 *      This routine fills in an already blanked image with
 *      a region's pixels.  It is a modified version to evaluate
 *      fit error parameters for region.
 *      Functions of two subroutines are combined here.
 *      Returns the number of pixels filled in.
 *
 *  Author:     Paul J. Besl    (9-5-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           rgnfilerr(region_list,region_num,offset,image_ptr,
 *                            x_size,y_size,org_image,c_vector.funeval,e1,e2,e8)
 *  Modules used:   none
 *
 */
 
#include <math.h>

int rgnfilerr(regnlist,regnum,offset,imgptr,nx,ny,orgimg,cvec,funeval,e1,e2,e8)
unsigned char   *orgimg,*imgptr;
int             regnum,offset,nx,ny;
double          (*funeval)(),cvec[]; 
double          *e1,*e2,*e8;
int             *regnlist;
{
    register short  int *pxlptr;
    register int    ix,iy,j,*rptr;
    int             nregns,npixels;
    double          xval,yval,zval,zfun;
    double          sqrerr,toterr,error,errmax;

    /* Fill In Region in Image with Value */

    nregns = *regnlist++;
    rptr = regnlist;
    npixels = *(rptr + 5*regnum);
    if( npixels <= 0 )
    {
        *e1 = *e2 = *e8 = 0.0;
        return(0);
    }
    pxlptr = (short int *)(rptr + 5*nregns + offset);
    sqrerr = 0.0;
    errmax = 0.0;
    toterr = 0.0;
    for(j=0; j<npixels; ++j)
    {
        xval = ix = *pxlptr++;
        yval = iy = *pxlptr++;
        zval = *(orgimg+iy*nx+ix);
        zfun = (*funeval)(xval,yval,cvec);
        *(imgptr+iy*nx+ix) = (int)(zfun+0.5);
        error = fabs( zfun - zval );
        if( error > errmax )
            errmax = error;
        toterr += error;
        sqrerr += error*error;
      }
    toterr /= (float)npixels; 
    sqrerr /= (float)npixels;
    sqrerr = sqrt( sqrerr );
    printf("e1=%g < e2=%g < e8= %g\n",toterr,sqrerr,errmax);
    *e1 = toterr;
    *e2 = sqrerr;
    *e8 = errmax;
    return(npixels);
}
