/*
 *  RGNGROWR2
 *
 *      This routine takes the (x,y,z) data arrays and the
 *      function definition (cvec,funeval) and computes the
 *      surface function at each pixel to find the predicted
 *      surface value. 
 *
 *  Author:     Paul J. Besl    (10-6-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           rgngrowr2(image_ptr,dsp_ptr,x_size,y_size,error_level,
 *                            c_vector,funeval,error_buffer,black_board)
 *  Modules used:   none
 *
 */
 
#include <math.h>

int rgngrowr2(imgptr,dspptr,nx,ny,errlevl,cvec,funeval,errbuf,blakbord)
unsigned char   *imgptr,*dspptr,*blakbord;
double          (*funeval)(),cvec[];
float           errlevl,*errbuf; 
int             nx,ny;
{
    register int            i,count,ntot;
    register unsigned char  *iptr,*dptr,*bptr;
    double                  error,xval,yval,zval,zfun;
    float                   *errptr;

    /* Loop over all data points */

    count = 0;
    ntot = nx*ny;  
    iptr = imgptr;
    dptr = dspptr;
    bptr = blakbord;
    errptr = errbuf;        /* new */
    for(i=0;i<ntot;++i)
    {
        xval = i % nx;
        yval = i / nx;
        zval = *iptr++;  
        zfun = (*funeval)(xval,yval,cvec);
        error = fabs( zfun - zval );

        /* error < threshold and either not claimed or better fit */
        if( error <= errlevl && ( *bptr == 255 || error < *errptr ) )
        {
            if( zfun < 0.5 )
                zfun = 0.5;
            else if( zfun > 255.4 )
                zfun = 255.4;
            *dptr++ = (int)(zfun+0.5);
            count++;
        }
        else
            *dptr++ = 0;
        errptr++;
        bptr++;
    }
    printf("Growth_Err_Tolrnc= %g --> %d Compat_Pxls\n",errlevl,count);
    printf("---------------------------------------------------\n");
    return(count);
}
