/*
 *  RGNLBLE
 *
 *      This routine finds connected regions in binary image in
 *      two passes.
 *      First pass labelling operation performed by this subroutine.
 *      Given the pointer (imgptr) to a rectangular (nx x ny) image
 *      this subroutine labels separate 4-connected regions
 *      which have (do not have) the value (regnvalue).
 *      in the unsigned short int image label array called (labels).
 *      Equivalent labels stored in (equivlabl) array.
 *      Memory is assumed already allocated for all arrays.
 *      (number of regions) is returned.
 *
 *  Author:     Paul J. Besl    (4-4-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           rgnlble(image_ptr,x_size,y_size,region_value,
 *                          labels,equiv_label,eq_size,num_pixel)
 *  Modules used:   none
 *
 */
 
int rgnlble(imgptr,nx,ny,regnvalue,labels,equivlabl,eqsize,npix)
unsigned char       *imgptr;
unsigned short int  *labels,*equivlabl;
int                 nx,ny,regnvalue,eqsize,*npix;
{
    register int                    i,j;
    register unsigned char          *iptr;
    register unsigned short int     *lptr,*uptr;
    register unsigned short int     leftlabl,upprlabl,ilabel;
    int                             ntot,regions,total,numpixtot,jtmp;

    /* label regions during 1st pass */

    ntot = nx*ny;
    ilabel = 1;
    total = 0;
    numpixtot = 0;
    iptr = imgptr;
    lptr = labels;
    uptr = lptr;
    uptr -= nx;
    for(i=0;i<ntot;++i)
    {
        if( *iptr == regnvalue )    /* pixel member of region */
        {
            /* check left and up */
            --lptr;
            leftlabl = *lptr++;
            upprlabl = *uptr;
            numpixtot++;
            if( upprlabl == 0 )
            {
                if( leftlabl == 0 )
                {
                    /* add newlabel to label array */

                    *lptr = ilabel++;
                    if( ilabel >= eqsize )
                    {
                        printf("Label Overflow:%d\n",ilabel);
                        exit(1);
                    }
                }
                else        /* use left's label */
                    *lptr = leftlabl;
            }
            else /* upprlabl != 0 */
            {
                if( leftlabl == 0 )     /* use upper's label */
                    *lptr = upprlabl;
                else        /* upper and left already labeled */
                {
                    if( upprlabl != leftlabl )
                    {
                        /* add equivalence of labels */
                        if( upprlabl > leftlabl ) /* swap */
                        {
                            jtmp = upprlabl;
                            upprlabl = leftlabl;
                            leftlabl = jtmp;
                        }
                        upprlabl = equivlabl[upprlabl];
                        *lptr = upprlabl;
                        total++;
                        if( equivlabl[leftlabl] == leftlabl )
                            equivlabl[leftlabl] = upprlabl;
                        else
                        {
                            jtmp = equivlabl[leftlabl];
                            if( jtmp  < upprlabl )
                            {
                                for(j=1;j<ilabel;++j)   /* search labels */
                                {
                                    if( equivlabl[j] == upprlabl )
                                        equivlabl[j] = jtmp;
                                }
                            }
                            else if( jtmp  > upprlabl )
                            {
                                for(j=1;j<ilabel;++j) /* search labels */
                                {
                                    if( equivlabl[j] == jtmp )
                                        equivlabl[j] = upprlabl;
                                }
                            }
                        }
                    }
                    else
                        *lptr = upprlabl;       /* labels equal case */
                }
            }
        }
        iptr++;
        lptr++;
        uptr++;
    }

    /* count number of regions */ 

    regions = 1;
    for(i=1;i<ilabel;i++)
    {
        if( i == equivlabl[i] )
            equivlabl[i] = regions++;
        else
            equivlabl[i] = equivlabl[equivlabl[i]];
    }
    --regions;
    --ilabel;       /* correct offset */

    /* Second pass is performed over label image to correct labels
       in (rgnprc) subroutine.
       Label Image has been completely processed; return data   */ 

    *npix = numpixtot;
    return(regions);
}
