/*
 *  RGNPRC
 *
 *      This routine processes labelled region image to
 *      obtain region by region description
 *      This subroutine accepts a image (labels) array of size
 *      (nx x ny) and the equivalent labels array (equivlabl)
 *      containing (nregns) as input.
 *      (numpixtot) is number of pixels in all regions.
 *      It generates a region list as output.
 *
 *  Author:     Paul J. Besl    (4-5-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           rgnprc(labels,x_size,y_size,equiv_label,
 *                         num_regions,num_pixel_total)
 *  Modules used:   none
 *
 */
 
int *rgnprc(labels,nx,ny,equivlabl,nregns,numpixtot)
unsigned short int  *labels,*equivlabl; 
int                 nx,ny,nregns,numpixtot; 
{
    int                             *imalloc();
    unsigned char                   *malloc();
    register int                    i,ix,iy,*rptr;
    register unsigned short int     *lptr;
    int                             *regnlist;
    int                             ntot,listsize;

    /* Allocate Memory for region list */

    listsize = 1 + 5 * nregns + numpixtot;
    regnlist = imalloc(listsize);
    /* printf("Region List Size= %d (%d,%d)\n",listsize,nregns,numpixtot); */
    rptr = regnlist;
    listsize = 1 + 5 * nregns; 
    for(i=0; i<listsize; ++i)
        *rptr++ = 0;
    *regnlist++ = nregns;

    /* perform second pass over label image to correct 
       image labels generated by regnlabl subroutines.
       counts no. of pixels per region, mins and maxs. */

    ntot = nx*ny;
    lptr = labels;
    for(i=0;i<ntot;++i)
    {
        if( *lptr != 0 ) 
        {
            *lptr = equivlabl[*lptr];
            ix = i % nx;
            iy = i / nx;
            rptr = regnlist + 5*(*lptr-1); 
            ++(*rptr);        /* increment pixel count region list */
            if( *rptr == 1 )  /* first pixel of this region */
            {
                rptr++;
                *rptr++ = ix;
                *rptr++ = iy;
                *rptr++ = ix;
                *rptr   = iy;
            }
            else
            {
                rptr++;
                if( ix < *rptr )
                    *rptr = ix;         /* x min */
                rptr++;
                if( iy < *rptr )
                    *rptr = iy;         /* y min */
                rptr++;
                if( ix > *rptr )
                    *rptr = ix;         /* x max */
                rptr++;
                if( iy > *rptr )
                    *rptr = iy;         /* y max */
                rptr;                   /* ????? */
            }
        }
        lptr++;
    }

    /* Return region list to calling program */

    regnlist--;
    return(regnlist);
}
