/*
 *  RGNPXL
 *
 *      This routine adds pixel locations to region list
 *      Given the pointer (labels) to a rectangular (nx x ny) 
 *      label image and the pointer to the region list (regnlist),
 *      this subroutine adds pixel locations to region list.
 *      Memory is assumed already allocated for all arrays.
 *      (short int) must be half the size of (int).
 *
 *  Author:     Paul J. Besl    (4-4-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           rgnpxl(labels,nx,ny,regnlist,numpixtot)
 *  Modules used:   none
 *
 */
 
#include <stdio.h>

void rgnpxl(labels,nx,ny,regnlist,numpixtot)
unsigned short int  *labels;
int                 nx,ny,numpixtot;
int                 *regnlist;
{
    register int                    i,j,k,*rptr;
    register unsigned short int     *lptr;
    register short int              *pxlptr;
    int                             jystart,jystop,ixstart,ixstop;
    int                             nregns,total,npixels,rpixls;

    /* initialize parameters */

    nregns = *regnlist++;
    rptr = regnlist;
    pxlptr = (short int *)(regnlist + 5*nregns);
    total = 0; 

    /* loop over the number of regions */

    for(k=1;k<=nregns;++k)
    {
        npixels = *rptr++;
        ixstart = *rptr++;
        jystart = *rptr++;
        ixstop  = *rptr++;
        jystop  = *rptr++;
        if( ixstart > ixstop )
            printf("ixstart=%d ixstop=%d region=%d\n",ixstart,ixstop,k);
        if( jystart > jystop )
            printf("jystart=%d jystop=%d region=%d\n",jystart,jystop,k);
        rpixls = 0;

        /* search for pixels in min/max box */
        
        for(j=jystart;j<=jystop;++j)
        {
            lptr = labels + j*nx + ixstart;
            for(i=ixstart;i<=ixstop;++i)
            {
                if( *lptr++ == k ) 
                {
                    *pxlptr++ = i;
                    *pxlptr++ = j;
                    rpixls++; }
                }
        }
        total += rpixls; 
        if( rpixls != npixels)  /* check just in case */
        {
            fprintf(stderr,"Error:rpixls=%d|npixls=%d|region=%d\n",rpixls,npixels,k);
            fprintf(stderr,"Error:(x,y)min=(%d,%d) (x,y)max=(%d,%d)\n",ixstart,jystart,ixstop,jystop);
        }
    }
    if( total != numpixtot )  /* check just in case */
        fprintf(stderr,"Error:total=%d|numpixtot=%d\n",total,numpixtot);
}
