/*
 *  RGNWT1
 *
 *      This routine prints run-length-encoded information for
 *      one region from region list to output file.
 *      Returns the number of pixels in written region.
 *
 *  Author:     Paul J. Besl    (8-5-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           rgnwt1(file_ptr,region_list,region_num,offset)
 *  Modules used:   none
 *      
 */     

#include <stdio.h>

int rgnwt1(filptr,regnlist,regnum,offset)
int     *regnlist,regnum,offset;
FILE    *filptr;
{
    register int        ix,iy,j,k;
    register int        *rptr;
    register short int  *pxlptr;
    int                 nregns,npixels,iylast,ixlast,notfirst;

    /* Set Up Pointers, etc. */

    nregns = *regnlist++;
    rptr = regnlist + 5*regnum;
    npixels = *rptr;
    pxlptr = (short int *)(regnlist + 5*nregns + offset);

    /* Write Out Region Header Information to File */

    fprintf(filptr,"\n===Region_# %d ===\n",regnum);
    fprintf(filptr,"%d Pixels\n",npixels);
    ix = *(rptr + 1);
    iy = *(rptr + 2);
    fprintf(filptr,"%d %d Min(Col,Row)\n",ix,iy);
    ix = *(rptr + 3);
    iy = *(rptr + 4);
    fprintf(filptr,"%d %d Max(Col,Row)\n",ix,iy);

    /* Write Out Run-Length-Encoded Pixels of Region */

    k=0;
    iylast = -9;
    ixlast = -9;
    notfirst = 0;
    for(j=0; j<npixels; ++j)
    {
        ix = *pxlptr++;
        iy = *pxlptr++;
        if( iy == iylast && ix == ixlast + 1 )
        {
            ixlast = ix;
            continue;
        }       /* in middle of run */
        else    /* start a new run */
        {
            if( notfirst ) /* wrap up last run */
            {
                fprintf(filptr,"-%-3d ",ixlast);
                if( k > 5 )
                {
                    fprintf(filptr,"\n");
                    k = 0;
                }
            } 
            fprintf(filptr,"%3d,%-3d",iy,ix);
            ++k;
            notfirst = 1;
            iylast = iy;
            ixlast = ix;
        }
    }
    fprintf(filptr,"-%-3d\n\n",ixlast);
    fflush(filptr);
    return(npixels);
}
