/*
 *  RGNXTR
 *
 *      This routine finds connected regions in binary image in two passes.
 *      It extracts regions from labelled image and puts them into a region list.
 *      Given the pointer (imgptr) to a rectangular (nx x ny) image
 *      this subroutine builds a region list array in
 *      the following format:		  V 5*nregns+1 (short int)
 *      nregns [ npxls rmin cmin rmax cmax]...[ ix1 iy1 ix2 iy2 ... ] ]
 *      The first portion of the list is integer.
 *      The pixel list is stored as short integer.
 *      The pointer to the region list is the returned parameter.
 *      Make Sure to FREE up region list space after use.
 *      Region : pixels != regnvalue if equalflag = 0
 *      Region : pixels == regnvalue if equalflag = 1
 *      Region : pixels <= regnvalue if equalflag = 2
 *      Region : pixels >= regnvalue if equalflag = 3
 *
 *  Author:     Paul J. Besl    (4-5-85)
 *
 *  Revised:    Arnold. H. Chiu (7-18-88)
 *  
 *  Call:           rgnxtr(image_ptr,x_size,y_size,region_value,equal_flag)
 *  Modules used:   none
 *      
 */     

#define  EQUIVLABLSIZ  65535  /* unsigned short max */

int *rgnxtr(imgptr,nx,ny,regnvalue,equalflag)
unsigned char   *imgptr,regnvalue;
int             nx,ny,equalflag;
{
    unsigned char                   *malloc();
    int                             *rgnprc(),rgnlbln(),rgnlble();

    register int                    i,ntot,*ptr;
    register unsigned short int     *lptr;
    unsigned short int              *imglabels,*equivlabl;
    int                             *regnlist;
    int                             eqlblsiz,nregion,numpixtot,tsecs,iblack;

    /* allocate memory for neighbor marker image */

    /*tsecs = tstart();*/
    ntot = nx*ny;
    imglabels = (unsigned short int *)malloc(sizeof(short)*ntot);
    ptr = (int *)imglabels;
    ntot /= 2;
    for(i=0; i<ntot; ++i)
        *ptr++ = 0;         /* zero out array */

    /* allocate memory for equivalent label list and initialize */
    eqlblsiz = EQUIVLABLSIZ;
    equivlabl = (unsigned short int *)malloc(sizeof(short)*eqlblsiz);
    lptr = equivlabl;
    for(i=0;i<eqlblsiz;++i)
        *lptr++ = i;

    /* label regions as 1st pass of 2-pass algorithm */

    switch( equalflag )
    {
        case 'n': /* not equal to */

        case 0: 
                iblack = regnvalue;
                imgbdrset(imgptr,nx,ny,3,iblack);
                nregion = rgnlbln(imgptr,nx,ny,regnvalue,imglabels,equivlabl,eqlblsiz,&numpixtot);
                break;

        case 'e': /* equal to */

        case 1:
                if( regnvalue == 0 )
                    iblack = 1;
                else
                    iblack = 0;
                imgbdrset(imgptr,nx,ny,3,iblack);
                nregion = rgnlble(imgptr,nx,ny,regnvalue,imglabels,equivlabl,eqlblsiz,&numpixtot);
                break;

        case 'l': /* less than or equal to */

        case 2:
                iblack = 255;
                imgbdrset(imgptr,nx,ny,3,iblack);
                nregion = rgnlbll(imgptr,nx,ny,regnvalue,imglabels,equivlabl,eqlblsiz,&numpixtot);
                break;

        case 'g': /* greater than or equal to */

        case 3:
                iblack = 0;
                imgbdrset(imgptr,nx,ny,3,iblack);
                nregion = rgnlblg(imgptr,nx,ny,regnvalue,imglabels,equivlabl,eqlblsiz,&numpixtot);
                break;

        default:
                printf("rgnxtr: equalflag= %d\n",equalflag);
                exit(1);
    }

    /* process regions from labeled image:
       get number of pixels, (x,y)min, (x,y)max for each region */ 

    regnlist = rgnprc(imglabels,nx,ny,equivlabl,nregion,numpixtot);

    /* add pixels into region list */

    rgnpxl(imglabels,nx,ny,regnlist,numpixtot);

    /* print out info and return */

    free(equivlabl);
    free(imglabels);
    printf("%d Pixels in %d Regions\n",numpixtot,nregion);
    /*tstop(tsecs,"Time to Compute 4-Connected Components");*/
    return(regnlist);
}
