/*
 *  ================================================================
 *  DEPTH MAP SURFACE CHARACTERIZATION AND OBJECT RECOGNITION SYSTEM
 *  ================================================================
 *
 *  SURFACE REFINEMENT MODULE (SREFINE)
 *
 *      This program accepts an unknown range_image and an object
 *      database as input.  It computes a rough surface characterization
 *      of the range_image first.  Then it computes a refined geometric
 *      structure representation.  This representation is used to match
 *      objects in the object database.
 *      Various output options for file output are available.
 *      Various compute options are also available.
 *
 *  Author:     Paul J. Besl    (7-17-85)
 *
 *  Revised:    Arnold. H. Chiu (8-31-88)
 *  
 *  Call:           srefine
 *  Modules used    getfildir(),    getrifn(),      cmpmenu_sr(),
 *                  outmenu(),      surfrefn()
 *
 */

#include <stdio.h>
 
#define DATAFILE        "dmorsys.data"
#define NUMOUTOPTNS     64
#define NUMCMPOPTNS     24
#define STRNGSIZE       128
#define NUMFILDIR       5

main(argc,argv)
int     argc;
char    *argv[];
{
    int     outoptions[NUMOUTOPTNS];    /* output option switches */
    int     cmpoptions[NUMCMPOPTNS];    /* compute options */
    char    string[STRNGSIZE];          /* scratch string */
    char    rangimg[STRNGSIZE];         /* range_image filename */
    int     iret;                       /* return_code */
    char    *names[NUMFILDIR];          /* file and directory names */
    int     img_size = 0;               /* image size entered */

    /* standard help info */

    if (argc<2)
        fprintf(stderr,"    {%s [range_image[image_size]]}\n\n", argv[0]);

    /* Retrieve image size if entered */

    if (argc==3)
        sscanf(argv[2],"%d",&img_size);

    /* Announce Program */

    /*set_sbrk_size(8400*8400);*/  /* -- this is Apollo specific call */
    /* this is obsolete, consult manual <AC> */

    printf("\nDEPTH MAP OBJECT RECOGNITION SYSTEM\n");
    printf("     Surface Refinement Module\n");
    printf("-----------------------------------\n");

    /* get default file directories */

    getfildir(DATAFILE,names,NUMFILDIR);

    /* Prompt for Valid Range_image File Name */

    get_file_name:

        getrifn(rangimg,string,argc,argv,names);

    /* Specify Compute Options */

    get_cmp_optns:

        iret = cmpmenu_sr(cmpoptions,string,STRNGSIZE,names,img_size);
        if( iret )
            goto get_file_name;

    /* Specify output options */

    get_out_optns:

        /* iret = outmenu(outoptions,string,STRNGSIZE,names); */
        if( iret )
            goto get_cmp_optns;

    /* Perform surface characterization operations */
    surfrefn(cmpoptions,outoptions,rangimg,string,names);
}
