/*
 *  SURFRFN5
 *
 *  Surface refinement subroutine with deanza display option.
 *
 *  Author:             Paul J. Besl        (7-17-85)
 *
 *  Cosmetic Surgery:   Arnold. H. Chiu     (7-13-88)
 *
 */

#include <stdio.h>
#include <math.h>

#define MINREGN         4
#define MAXI            100
#define SHRINKF         0.40
#define STRNGSZ         80
#define MAXSURF         254
#define FITMTHD         16
#define FITORD          17
#define NUMORDER        4

surfrefn(cmpoptions,outoptions,rangimg,string,dfnames)
int     *cmpoptions,*outoptions;
char    *rangimg,*string,*dfnames[];
{
    FILE            *gfopen();
    unsigned char   *malloc(),*getimage(),*calloc();
    int             *imalloc(),bbupdate(),hkupdate(),dilate(),funchk(),oktofit();
    int             *rgnxtr(),rgnbrc(),rgnfil(),rgnsiz(),rgnxyz(),regnrefn(),regngrower();
    int             imgrovlpf(),cntrctprf(),cntrctest();
    float           *fmalloc(),evalerr(),*errbuf,errsum();
    double          zplan(),zquad(),zcubc(),zqar4(),(*zfunc[NUMORDER])();

    static int      minfit[NUMORDER] = { 3, 6, 15, 15 };
    static int      mapcode[10] = { 168, 252, 42, 255, 1, 210, 84, 126, 220, -1 }; 
    static char     *stringcode[10] = { "Flat", "Ridge", "Valley", "Peak", "Cup",
                                        "Saddle Ridge", "Saddle Valley", "Minimal",
                                        "K>0:H=0", "Black_Board" };
    char            hksgnfnam[STRNGSZ],slogfnam[STRNGSZ];
    int             compat[MAXI],histogram[256],errlevl[MAXI];
    double          cvec[21],ovec[21];

    FILE            *filptr;                    /* pointer to session log file  */
    FILE            *plotfil;                   /* pointer to plot data file    */
    FILE            *binfil;
    register int    k;
    unsigned char   *imgptr;                    /* H-K sign map                 */
    unsigned char   *blakbord,glevel;
    unsigned char   *ucimg1,*ucimg2,*ucimg3,*ucimg4;
    unsigned char   *orgimg;                    /* pointer to original image */
    unsigned char   *stypimg,*rcnsimg,*lastimg,*surfimg,*rgnlabls;
    int             runflg,runlst,fitmethod,fitorder;
    int             *regnlist,*subrgnlst,maxbbrs;
    int             nsize,xsize,ysize,npixels,*regnptrs,*roffsets,nregns;
    int             *subrgnptr,*subrofset,nsbrgns,surftype;
    int             thisccnt,lastccnt,totime,tsecs,mem,fitmode,fitlast,surfadd;
    int             pixlcnts[64],ncontrcts,threshold1,threshold2,itime,totalpix; 
    int             limit,pxlcnt,nx,ny,numiter;
    int             conected[MAXI],thisconcnt,lastconcnt,ix,iy,first,ixlabl,iylabl;
    int             hkhist[10],maxiter,mininitsize,dzdisp,isurf,maxcntrct,npts;
    int             hklabl[10],mean,sdev,imax,imin,npoints,memark = 0;
    int             hkrcnt[10],badcntrct,thisnpts,lastnpts;
    int             fmlist[MAXI],iloop,rgnfound;
    int             n1fits,n2fits,n3fits,n4fits,binflg,firstx,lastx;
    float           *xptr,*yptr,*zptr,xmn,ymn,zmn,xsd,ysd,zsd,acceptf,errmrk,testval;
    float           *x2ptr,*y2ptr,*z2ptr;
    float           errtol,errmax,errmaxfix,errlim,errfact,errsav,rmse;
    double          e1,e2,e8,K2;

    totime = tstart();

    /* Initialize internal values (with command options) */

    dzdisp = cmpoptions[ 0 ];       mem = cmpoptions[ 1 ];
    mininitsize = cmpoptions[ 2 ];
    threshold2 = cmpoptions[ 3 ];   threshold1 = cmpoptions[ 4 ];
    maxiter = cmpoptions[ 5 ];      maxcntrct = cmpoptions[ 7 ];    maxbbrs = cmpoptions[ 10 ];
    errmaxfix = errmax = (float)cmpoptions[6]/100.0;
    errfact = (float)cmpoptions[8]/100.0;
    acceptf = (float)cmpoptions[9]/100.0;
    testval = (float)cmpoptions[11]/1000.0;
    fitmethod = cmpoptions[FITMTHD];
    fitorder  = cmpoptions[FITORD];

    if ( maxiter > MAXI )
        maxiter = MAXI;

    zfunc[0] = zplan;   zfunc[1] = zquad;   zfunc[2] = zcubc;   zfunc[3] = zqar4;
    totalpix = 0;       errmrk = 1024.0; 

    /* Open Auxiliary Files */

    strcpy(slogfnam,"file.slog");
    filptr = gfopen(slogfnam,"w");
    fprintf(filptr,"Surface Decomposition for '%s'\n\n",rangimg);
    plotfil = gfopen("plot.data","w");
    fprintf(plotfil,"%g %g %g <--(Max_Error,Err_Factor,Accept_Fact)\n",errmax,errfact,acceptf);
    unlink("bincoef");
    fclose(plotfil); 

    /* Get Original Image and Display It */

    orgimg = getimage(rangimg,&nsize);
    /*    if( dzdisp ) dspsub2(mem,orgimg,nsize);    */
    fprintf(filptr,"%d x %d Image\n",nsize,nsize);

    /* Get HK-Sign Image from Disk */

    cnvrtname(31,rangimg,hksgnfnam,dfnames);
    imgptr = getimage(hksgnfnam,&nsize);

    /* Check for SubImage Option, Fixing Pointers if Needed */

    xsize = ysize = nsize;
    chksubimg(cmpoptions,&orgimg,&imgptr,&xsize,&ysize,filptr);
    img0_1(orgimg,xsize,ysize);
    img0_1(imgptr,xsize,ysize);

    /* Allocate Memory for Various Purposes */

    limit = xsize*ysize;
    ucimg1   = malloc(limit);       ucimg2   = malloc(limit);
    ucimg3   = malloc(limit);       ucimg4   = malloc(limit);
    stypimg  = malloc(limit);       rcnsimg  = malloc(limit);
    blakbord = malloc(limit);       lastimg  = malloc(limit);
    x2ptr    = fmalloc(limit);      xptr     = fmalloc(limit/8);
    y2ptr    = fmalloc(limit);      yptr     = fmalloc(limit/8);
    z2ptr    = fmalloc(limit);      zptr     = fmalloc(limit/8);
    surfimg  = malloc(limit);       rgnlabls = calloc(limit,1);    /* new */
    errbuf   = fmalloc(limit);

    matset(errbuf,errmrk,xsize,ysize); 
    imgset(blakbord,255,xsize,ysize);
    imgset(rcnsimg,0,xsize,ysize);
    imgcpy(stypimg,imgptr,xsize,ysize);
    blkbdrfi(errbuf,xsize,ysize,3);
    /* if( nsize <= 128 )
        fc = 0; */
    errmrk--;
    /*    if( dzdisp ) dspsub2(mem,imgptr,nsize);    */

    /*
     *  ==============
     *   REGION LOOP I
     *  ==============
     *
     *  Process Regions By Size of Region Regardless of Surface type.
     *  While There is At Least One Big Surface Region of Some Type
     *  ( for variable order fitting )
     */

    glevel = 255;  hklabl[0] = 0;  isurf = 0;  iloop = 0;

    /* check if a fixed order fit was requested */

    if (fitorder > 0)
        goto fixorder_fit;

    /* Loop Until Jump Out at Break Point */

    printf("\n*** PERFORMING VARIABLE_ORDER_FIT ***\n");
    printf("    -----------------------------\n");
    while( isurf < MAXSURF )  
    {
        printf("\n*** WHILE_Big_Enough_Regions_Exist Loop *** (%d,%d)\n",iloop,isurf);
        itime = tstart(); 

        /* Get Largest Region Pixel Count; Check Each Surface Type */

        /*      if( dzdisp ) dspsub2(mem,stypimg,nsize);    */
        if ( hklabl[0] != 9 ) 
            surftype = pickfrgn(stypimg,blakbord,xsize,ysize,hklabl,hkhist,
                                hkrcnt,histogram,mapcode,stringcode,mininitsize,&errmax);

        /* Extract Basic Surface Type Regions and Sort by Size */ 

        regnlist = rgnxtr(stypimg,xsize,ysize,surftype,1);
        nregns = *regnlist;
        if ( nregns <= 0 )
        {
            printf("NO REGIONS\n");
            free(regnlist);
            break;
        }
        regnptrs = imalloc(2*nregns);
        roffsets = regnptrs + nregns;
        rgnsrt(regnlist+1,regnptrs,roffsets,nregns);

        /*
         * Process Largest Region with More Than X Pixels
         * Create Image with One Original Region in it; Check size
         */

        imgset(ucimg1,0,xsize,ysize);
        npixels = rgnfil(regnlist,regnptrs[0],roffsets[regnptrs[0]],
                         ucimg1,xsize,ysize,glevel);
        /*      if( dzdisp ) dspsub2(mem,ucimg1,nsize);    */
        rgnsiz(regnlist,regnptrs[0],roffsets[regnptrs[0]],&nx,&ny,&ix,&iy);
        free(regnptrs);
        free(regnlist);
        if( npixels < mininitsize )
        {
            printf("Less Than %d Pixels\n",mininitsize);
            if( hklabl[0] == 9 )
                break; /* BREAK */
            else
                continue; /* CONTINUE INTO BLACKBOARD MODE */
        }

        /* Determine Contraction Profile of Region */
    
        ncontrcts = cntrctprf(ucimg1,ucimg4,ucimg2,ucimg3,xsize,ysize,
                              pixlcnts,maxcntrct,threshold1,threshold2,
                              npixels,dzdisp,mem);
        imgcpy(ucimg2,ucimg4,xsize,ysize); 

        /* Check Contraction Profile Before Fitting */

        if( pixlcnts[0] < (acceptf*MINREGN) ) 
        {
            printf("\nFlagging Region for Bad Contraction Profile!!\n");
            badcntrct = 1;
        }
        else
            badcntrct = 0;

        /*
         * ===================================================
         *  ITERATIVE SURFACE FITTING and REGION GROWING LOOP
         * ===================================================
         * Uses 3 Different Modes of Surface Fitting.
         */

    surface_fit:

        n1fits = n2fits = n4fits = 0;
        imgset(lastimg,255,xsize,ysize);
        first = 0;          fitmode = 0;     fitlast = -1;    lastccnt = -2;   thisccnt = -1;
        lastconcnt = -2;    thisconcnt = -1; thisnpts = -1;   lastnpts = -2;
        errtol = 0.0;       numiter = 0;     runlst = 0;
        while( oktofit(lastccnt,thisccnt,lastconcnt,thisconcnt,errtol,errmax,
                       numiter,maxiter,thisnpts,lastnpts,fitmode,fitlast,
                       conected,compat) )
        {
            printf("~~~ S-Fit (Mode=%d) <#%d> ~~~\n",fitmode,numiter);
            subrgnlst = rgnxtr(ucimg2,xsize,ysize,0,0);
            nsbrgns = *subrgnlst;
            if ( nsbrgns == 0 )
            {
                thisccnt = -8;
                free(subrgnlst);
                break;
            }
            subrgnptr = imalloc(2*nsbrgns);
            subrofset = subrgnptr + nsbrgns;
            rgnsrt(subrgnlst+1,subrgnptr,subrofset,nsbrgns);

            /*
             * Find Largest Sub-Region of Seed Region Image 
             * that Overlaps the Last Image Used
             */ 

            rgnfound = 0;
            for(k=0;k<nsbrgns;++k)
            {
                npixels = rgnsiz(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],
                                 &nx,&ny,&ix,&iy);
                if( npixels <= MINREGN )
                {
                    thisccnt = -1;
                    break;
                }

                /* Ought to Use Min/Max Box to Check first for Overlap */

                /* Create Binary Image of Region */

                imgset(ucimg1,0,xsize,ysize);
                npixels = rgnfil(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],
                                 ucimg1,xsize,ysize,glevel);
                /*          if( dzdisp ) dspsub2(mem,ucimg1,nsize);    */

                /* Check if Region Overlaps with Last Version of Region */

                pxlcnt = imgrovlpf(ucimg1,lastimg,xsize,ysize);
                if( pxlcnt > 0 )
                {
                    rgnfound = 1;
                    break;
                }
                else
                    printf("No Overlap; Try Next Region (%d)\n",k);
            } /* ==== End of Subregion Loop ==== */

            /* Abort Fitting if Problems */

            if( npixels <= MINREGN ) 
              { printf("(1) Region Has Too Few Pixels!\n"); 
                thisccnt = -10; break;
              }
            if( rgnfound == 0 )
                printf("Region NOT Found!\n");
            if( lastccnt == -2 )
            {
                ixlabl = ix;
                iylabl = iy;
            } /*savelabelloc*/

            /* Build (x,y,z) Surface List from Region Description */

            npixels = rgnxyz(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],
                             orgimg,xsize,ysize,x2ptr,y2ptr,z2ptr,limit);
            if( npixels <= MINREGN ) 
            {
                printf("(2) Region Has Too Few Pixels! ***BUG***\n");
                break;
            }
            xyzstats(x2ptr,y2ptr,z2ptr,npixels,&xmn,&ymn,&zmn,&xsd,&ysd,&zsd);
            free(subrgnptr);
            free(subrgnlst);

            /* Compact Point List When Too Many Points Used */

            npoints = xyzcmpct(x2ptr,y2ptr,z2ptr,npixels,xptr,yptr,zptr);
            imgcpy(lastimg,ucimg1,xsize,ysize); /* save regn as last version */
            if( first == 0 )
                imgcpy(ucimg4,ucimg1,xsize,ysize);
            if( npoints <= MINREGN ) 
            {
                printf("(3) Region Has Too Few Pixels! ***BUG***\n");
                break;
            }

            /* Fit Surface to This Region */ 

        start_again:

            first = 1;
            if(npoints <= minfit[fitmode])
            {
                thisccnt = -6;
                break;
            }
            switch( fitmode )
            {
                case 0:     /* Planar Fit Mode ============== */
                            n1fits++;
                            tsecs = tstart();
                            plnsurfit(xptr,yptr,zptr,npoints,xmn,ymn,&errtol,
                                      nx,ny,cvec,fitmethod,ovec);
                            tstop(tsecs,"Time for Planar Fit");

                            if( errtol > errmax && npoints > minfit[1] )
                            {
                                fitmode=1;
                                printf("Incr_Fit_Order!\n");
                                goto start_again;
                            }
                            break;
     
                case 1:     /* Fit Biquadratic Surface ============== */
                            n2fits++;
                            tsecs = tstart();
                            bqdsurfit(xptr,yptr,zptr,npoints,xmn,ymn,&errtol,
                                      nx,ny,cvec,fitmethod,ovec);
                            tstop(tsecs,"Time for Biquadratic Fit");

                            /* On First Pass Seed Region May Be Too Big */

                            if( errtol > errmax && lastccnt == -2 ) 
                            {
                                for(npts=npoints/2; npts>MINREGN; npts /= 2)
                                {
                                    printf("#pts= %d  ",npts);
                                    plnsurfit(xptr,yptr,zptr,npts,xmn,ymn,&errtol,
                                              nx,ny,cvec,fitmethod,ovec);
                                    if( errtol <= errmax ) 
                                    {
                                        npixels = npoints = npts;
                                        fitmode = 0; 
                                        n1fits++;
                                        goto regn_grow;
                                    }
                                }
                                fitmode = 0;
                            }
                            if( badcntrct && errtol > acceptf*errmax )
                            {
                                thisccnt = -4;
                                goto final_steps;
                            }
                            if( errtol > 5.0*errmax)
                            {
                                thisccnt = -8;
                                goto final_steps;
                            }
                            if( errtol > errmax && npoints > minfit[2] )
                            {
                                fitmode=2;
                                printf("Incr_Fit_Order!\n");
                                goto start_again;
                            }
                            break;

                case 2:     /* Fit Bicubic Surface ============== */
                            n3fits++; 
                            if( fitlast <= 0 && numiter <= 2 ) /*indication of instability*/
                            {
                                thisccnt = -18;
                                printf("Two Consec. Order Jumps!!\n");
                                goto final_steps;
                            }

                            tsecs = tstart();
                            bcbsurfit(xptr,yptr,zptr,npoints,xmn,ymn,&errtol,
                                      nx,ny,cvec,fitmethod,ovec);
                            tstop(tsecs,"Time for Bicubic Fit");

                            if( badcntrct && errtol > acceptf*errmax )
                            {
                                thisccnt = -4;
                                goto final_steps;
                            }
                            if( errtol > 5.0*errmax)
                            {
                                thisccnt = -8;
                                goto final_steps;
                            }
                            if( errtol > errmax && npoints > minfit[2] )
                            {
                                fitmode=3;
                                printf("Incr_Fit_Order!\n");
                                goto start_again;
                            }
                            break;

                case 3:     /* Fit Biquartic Surface ============== */
                            n4fits++; 
                            if( fitlast <= 0 && numiter <= 2 ) /*indication of instability*/
                            {
                                thisccnt = -18;
                                printf("Two Consec. Order Jumps!!\n");
                                goto final_steps;
                            }

                            tsecs = tstart();
                            bq4surfit(xptr,yptr,zptr,npoints,xmn,ymn,&errtol,
                                      nx,ny,cvec,fitmethod,ovec);
                            tstop(tsecs,"Time for Biquartic Fit");
                            break;
            }

            /* Analyze Fit to Data; Flag if Heuristic Runs Test Not Satisfied. */

        regn_grow:

            if( fitmode == 0 || fitmode == 1 || fitmode == 2 ) 
            {
                runflg = funchk(x2ptr,y2ptr,z2ptr,npixels,cvec,zfunc[fitmode],
                                &e1,&e2,&e8,ucimg1,xsize,ysize,dzdisp,testval);
                errtol = e2;
            }
            else
                runflg = 0;

            /* Perform Region Growing and other Operations common to all methods */

            lastccnt = thisccnt;        lastnpts = thisnpts;    thisnpts = npoints;
            lastconcnt = thisconcnt;    thisconcnt = npixels;   fitlast = fitmode;
            if( errtol <= acceptf*errmax )
            {
                errlim = evalerr(errtol, errfact, errmax);
                thisccnt = rgngrowr2(orgimg,ucimg2,xsize,ysize,errlim,
                                     cvec,zfunc[fitmode],errbuf,blakbord);
            } 
            /*        if( dzdisp ) dspsub2(mem,ucimg2,nsize);    */

            /* Save Parameters from this iteration */

            fmlist[ numiter ] = fitmode;
            conected[ numiter ] = thisconcnt;
            compat[ numiter ] = thisccnt;
            errlevl[ numiter++ ] = errtol;
            rmse = errsav = errtol;
            vmemchk( &memark );                     /* check virtual memory allocation */
            if( thisconcnt < SHRINKF*lastconcnt )   /* check for shrinkage */
            {
                thisccnt = -14;
                break;
            }

            /* Use Heuristic 2-D Runs Test Results (2%) */

            if( runflg && errtol > 0.1 ) 
            {
                fitmode++;
                printf("*--> Increasing Fit Mode to %d\n",fitmode); 
            }
        } /* ===== End of Iterative Surf. Fit and Regn. Grow Loop ===== */

        /*
         * ===================================================
         * PROCESS ITERATED SURFACE RESULTS
         * ===================================================
         * Mark Pixels of Newly Grown Region in Blackboard,etc.
         * Requirements: Good Fit, 4-Connected Region, Low Error.
         */

    final_steps:

        if( errtol <= acceptf*errmax && 
            thisccnt > MINREGN && thisconcnt > MINREGN ) 
        {
            surfadd = 0;  /* still possible to reject surface */

            /* Refine Regions to Eliminate Isolated Pixels */

            pxlcnt = regnrefn(ucimg2,ucimg1,xsize,ysize,0);
            if( pxlcnt ) 
            {
                pxlcnt = regnrefn(ucimg1,ucimg2,xsize,ysize,0);
                if(pxlcnt)
                    regnrefn(ucimg2,ucimg1,xsize,ysize,0);
                else
                    imgcpy(ucimg1,ucimg2,xsize,ysize);
            }
            burrtrim(ucimg1,ucimg2,xsize,ysize,0);  /* Trim Off Burrs */
            burrtrim(ucimg2,ucimg1,xsize,ysize,0);
            /*          if( dzdisp ) dspsub2(mem,ucimg1,nsize);    */

            /* Extract Sorted Connected Component Regions */

            subrgnlst = rgnxtr(ucimg1,xsize,ysize,0,0);
            nsbrgns = *subrgnlst;
            subrgnptr = imalloc(2*nsbrgns);
            subrofset = subrgnptr + nsbrgns;
            rgnsrt(subrgnlst+1,subrgnptr,subrofset,nsbrgns);

            /* Check Size of Largest Region for Potential Rejection */
            npixels = *(subrgnlst+1+5*subrgnptr[0]);
            if( npixels <= MINREGN ) /* prelim check for stability */
            {
                hkupdate(stypimg,ucimg4,xsize,ysize);
                printf("\n*** Unstable Seed Region Discarded (%d) ***\n",npixels);
                free(subrgnptr);
                free(subrgnlst);
                goto end_while_loop;  /* reject */
            }
            isurf++;
            plotfil = gfopen("plot.data","a");
            iterprnt(plotfil,numiter,errlevl,compat,conected,fmlist,isurf);
            fclose(plotfil);
            binflg = 0;
            printf("\n*** Surface Primitive %d Interpreted! ***\n",isurf);
            printf("%d Sub-Regions for Surface_# %d\n",nsbrgns,isurf);

            /* For Each Compatible Sub-Region of Surface Until Too Small */

            for(k=0;k<nsbrgns;++k)
            {
                npixels = *(subrgnlst+1+5*subrgnptr[k]);
                if( npixels <= MINREGN && surfadd == 1 )
                    break; 
                imgset(ucimg2,0,xsize,ysize);
                npixels = rgnfilerr(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],
                                    ucimg2,xsize,ysize,orgimg,cvec,zfunc[fitlast],&e1,&e2,&e8);
                pxlcnt = imgrovlpf(ucimg2,lastimg,xsize,ysize);
                /*            if( dzdisp ) dspsub2(mem,ucimg2,nsize);    */

                /* Process Extra Compatible Regions; Continue if Not OK */

                if( pxlcnt <= 0 )
                {
                    if( npixels < mininitsize && surfadd == 1 )
                        break;
                    ncontrcts = cntrctest(ucimg2,ucimg1,ucimg3,xsize,ysize,pixlcnts,2);
                    if( ncontrcts < 2 )
                        continue; 
                    if( pixlcnts[1] <= MINREGN )
                        continue;
                    printf("%d Pixels Left after 2 Contractions\n",pixlcnts[1]);
                    if( e2 > acceptf*errmax )
                        goto error_buf_upd;
                    if( e2 > errmax && errsav < errmax )
                        goto error_buf_upd;
                    rgnsiz(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],&nx,&ny,&ixlabl,&iylabl);

                    /* I have assumed that if the first region is Compatible,
                       but is not the original region, it is ok to use the
                       center of the min/max box as a label, because the region
                       probably does not have any holes in it anyway. This 
                       whole problem could be corrected with a more elaborate
                       approach, but does not seem worth it just for labels.
                       This scheme should be right 99% of the time. */
                }
                else
                    errsav = e2;
                printf("\n Adding_Surface_Region_with %d Pixels\n",npixels);
                /*            if( dzdisp )      dspsub2(mem,ucimg2,nsize);    */

                if( dzdisp == 3 )
                    waitret();

                /* Write Out Region and Surface Information to File */

                fprintf(filptr,"@-------------( %d )( %d %d )\n",npixels,ixlabl,iylabl);
                fprintf(filptr,"%d %d <> Surface_# %d  of Type: %s (SR=%d)\n",
                        isurf,hklabl[0],isurf,stringcode[hklabl[0]],k);
                scoefprnt(filptr,fitlast,cvec);
                fprintf(filptr,"\n%g %g %g (e8,e2,e1), %g\n",e8,e2,e1,rmse);
                rgnwt1(filptr,subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]]);

                /* Write Coefficients to Binary Coefficients File */

                if( pxlcnt > 0 && binflg == 0 ) 
                {
                    binfil = gfopen("bincoef","a");
                    fwrite(&minfit[fitlast],sizeof(int),1,binfil);
                    fwrite(cvec,sizeof(double),minfit[fitlast],binfil);
                    errtol = e8;        fwrite(&errtol,sizeof(float),1,binfil);
                    errtol = e2;        fwrite(&errtol,sizeof(float),1,binfil);
                    errtol = e1;        fwrite(&errtol,sizeof(float),1,binfil);
                    fwrite(&rmse,sizeof(float),1,binfil);
                    fclose(binfil);
                    binflg = 1;
                }

                /* Update Blackboard Image */

                pxlcnt = bbupdate(blakbord,ucimg2,xsize,ysize,isurf); 
                printf("BlackBoard Display <%d|%d>\n",isurf,hklabl[0]);
                /*            if( dzdisp ) dspsub2(mem,blakbord,nsize);    */
                wrtimage("blakbord",blakbord,xsize,ysize);
                if( pxlcnt == 0 ) 
                    printf("bb: Infinite Loop Problem: Duplicate Surface\n");

                /* Update Reconstruction Image */

                imgor2(rcnsimg,ucimg2,xsize,ysize);
                printf("Reconstructed Image Display <%d|%d>\n",isurf,hklabl[0]);
                /*            if( dzdisp ) dspsub2(mem,rcnsimg,nsize);    */
                wrtimage("recons",rcnsimg,xsize,ysize);
                pxlcnt = hkupdate(stypimg,ucimg2,xsize,ysize);
                if( pxlcnt == 0 ) 
                    printf("hk: Infinite Loop Problem: Duplicate Surface\n");
                surfadd = 1;
                totalpix += npixels;

                /* Update Error Buffer: Unconditional Update,
                   Then 4 dilations, Then Conditional Update */

            error_buf_upd:

                tsecs = tstart();
                ebupdat1(orgimg,ucimg2,xsize,ysize,cvec,zfunc[fitlast],
                         errbuf,surfimg,rgnlabls,isurf); 
                imgith(ucimg1,ucimg2,xsize,ysize,0);
                firstx = -1;    lastx = -1;
                dilate(ucimg2,ucimg3,xsize,ysize,&firstx,&lastx);
                dilate(ucimg3,ucimg2,xsize,ysize,&firstx,&lastx);
                dilate(ucimg2,ucimg3,xsize,ysize,&firstx,&lastx);
                dilate(ucimg3,ucimg2,xsize,ysize,&firstx,&lastx);
                dilate(ucimg2,ucimg3,xsize,ysize,&firstx,&lastx);
                dilate(ucimg3,ucimg2,xsize,ysize,&firstx,&lastx);
                imgand2(ucimg2,ucimg1,xsize,ysize);
                /*         if( dzdisp ) dspsub2(mem,ucimg2,nsize);     */
                imgnotip(ucimg1,xsize,ysize);
                errlim = 5.0*errfact*evalerr( e2, errfact, errmax);
                ebupdat2(orgimg,ucimg2,ucimg1,ucimg3,xsize,ysize,cvec,
                         zfunc[fitlast],errbuf,surfimg,rgnlabls,isurf,errlim);
                tstop(tsecs,"Time for dilate & ebupdat1,2 Subroutines");
                /*         if( dzdisp ) dspsub2(mem,ucimg2,nsize);     */
                /*        if( dzdisp ) dspsub2(mem,surfimg,nsize);    */
                wrtimage("rgnlabl",rgnlabls,xsize,ysize);
                wrtimage("surfimg",surfimg,xsize,ysize);
                if( dzdisp == 3 )
                    waitret(); 
            }  /* -----End of Sub-Region Loop----------------------------- */

            free(subrgnptr);
            free(subrgnlst);
            if( surfadd == 0 ) 
            {
                hkupdate(stypimg,ucimg4,xsize,ysize);
                printf("\n*** Misleading Seed Region Discarded ***\n");
            }
            if( surfadd )  /* new section */
            {
                errtol = errsum(errbuf,xsize,ysize,errmrk);
                if( errtol <= errmaxfix )
                {
                    imgthr(ucimg1,blakbord,xsize,ysize,254); /* threshold */
                    contract(ucimg1,ucimg3,xsize,ysize);   /* contract */
                    pxlcnt = rgnbrc(ucimg3,xsize,ysize,255,1); /* rgn size */
                    /*              if( dzdisp ) dspsub2(mem,ucimg3,nsize);    */
                    if( pxlcnt < maxbbrs ) 
                    {
                        tstop(itime,"Time for This Iteration"); iloop++;
                        goto all_done;  /* Finished!! */
                    } 
                    printf("Check Region with %d Pixels(Cntrctd); Error OK\n",pxlcnt);
                    if( dzdisp == 3 ) waitret();
                }
            }
        }
        else
        {
            /* If BlackBoard Processing & Error Problem, Allow 2nd chance */

            if( hklabl[0] == 9 && thisccnt > MINREGN )
            {
                pxlcnt = contract(ucimg4,ucimg2,xsize,ysize);
                if( pxlcnt > MINREGN ) 
                {
                    printf("Giving Black_Board Region 2nd Chance\n");
                    goto surface_fit;
                }
            }

            /* do not mess with this seed region again */

            hkupdate(stypimg,ucimg4,xsize,ysize);
            printf("\n*** Bad Growth! This Seed Region Discarded ***\n");
            printf("Error= %g > Limit= %g (%d,%d)\n",errtol,acceptf*errmax,thisccnt,thisconcnt);
        }

    end_while_loop:

        tstop(itime,"Time for This Iteration");
        iloop++;
    } /* ===== End of Region Loop ===== */

fixorder_fit:   /* duplicated code from variable-order fitting steps below */

    /* Loop Until Jump Out at Break Point */

    printf("\n*** PERFORMING FIXED_ORDER_FIT===> %d ***\n",fitorder);
    printf("    --------------------------------\n");
    while( isurf < MAXSURF )  
    {
        printf("\n*** WHILE_Big_Enough_Regions_Exist Loop *** (%d,%d)\n",iloop,isurf);
        itime = tstart(); 

        /* Get Largest Region Pixel Count; Check Each Surface Type */

        /*      if( dzdisp ) dspsub2(mem,stypimg,nsize);    */
        if ( hklabl[0] != 9 ) 
            surftype = pickfrgn(stypimg,blakbord,xsize,ysize,hklabl,hkhist,
                                hkrcnt,histogram,mapcode,stringcode,mininitsize,&errmax);

        /* Extract Basic Surface Type Regions and Sort by Size */ 

        regnlist = rgnxtr(stypimg,xsize,ysize,surftype,1);
        nregns = *regnlist;
        if ( nregns <= 0 )
        {
            printf("NO REGIONS\n");
            free(regnlist);
            break;
        }
        regnptrs = imalloc(2*nregns);
        roffsets = regnptrs + nregns;
        rgnsrt(regnlist+1,regnptrs,roffsets,nregns);

        /*
         * Process Largest Region with More Than X Pixels
         * Create Image with One Original Region in it; Check size
         */

        imgset(ucimg1,0,xsize,ysize);
        npixels = rgnfil(regnlist,regnptrs[0],roffsets[regnptrs[0]],
                         ucimg1,xsize,ysize,glevel);
        /*      if( dzdisp ) dspsub2(mem,ucimg1,nsize);    */
        rgnsiz(regnlist,regnptrs[0],roffsets[regnptrs[0]],&nx,&ny,&ix,&iy);
        free(regnptrs);
        free(regnlist);
        if( npixels < mininitsize )
        {
            printf("Less Than %d Pixels\n",mininitsize);
            if( hklabl[0] == 9 )
                break; /* BREAK */
            else
                continue; /* CONTINUE INTO BLACKBOARD MODE */
        }

        /* Determine Contraction Profile of Region */
    
        ncontrcts = cntrctprf(ucimg1,ucimg4,ucimg2,ucimg3,xsize,ysize,
                              pixlcnts,maxcntrct,threshold1,threshold2,
                              npixels,dzdisp,mem);
        imgcpy(ucimg2,ucimg4,xsize,ysize); 

        /* Check Contraction Profile Before Fitting */

        if( pixlcnts[0] < (acceptf*MINREGN) ) 
        {
            printf("\nFlagging Region for Bad Contraction Profile!!\n");
            badcntrct = 1;
        }
        else
            badcntrct = 0;

        /*
         * ===================================================
         *  ITERATIVE SURFACE FITTING and REGION GROWING LOOP
         * ===================================================
         * Uses 3 Different Modes of Surface Fitting.
         */

    surface_fit_fix:

        n1fits = n2fits = n4fits = 0;
        imgset(lastimg,255,xsize,ysize);
        first = 0;          fitmode = fitorder - 1;              fitlast = -1;
        lastccnt = -2;      thisccnt = -1;
        lastconcnt = -2;    thisconcnt = -1;    thisnpts = -1;   lastnpts = -2;
        errtol = 0.0;       numiter = 0;        runlst = 0;
        while( oktofit(lastccnt,thisccnt,lastconcnt,thisconcnt,errtol,errmax,
                       numiter,maxiter,thisnpts,lastnpts,fitmode,fitlast,
                       conected,compat) )
        {
            printf("~~~ S-Fit (Mode=%d) <#%d> ~~~\n",fitmode,numiter);
            subrgnlst = rgnxtr(ucimg2,xsize,ysize,0,0);
            nsbrgns = *subrgnlst;
            if ( nsbrgns == 0 )
            {
                thisccnt = -8;
                free(subrgnlst);
                break;
            }
            subrgnptr = imalloc(2*nsbrgns);
            subrofset = subrgnptr + nsbrgns;
            rgnsrt(subrgnlst+1,subrgnptr,subrofset,nsbrgns);

            /*
             * Find Largest Sub-Region of Seed Region Image 
             * that Overlaps the Last Image Used
             */ 

            rgnfound = 0;
            for(k=0;k<nsbrgns;++k)
            {
                npixels = rgnsiz(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],
                                 &nx,&ny,&ix,&iy);
                if( npixels <= MINREGN )
                {
                    thisccnt = -1;
                    break;
                }

                /* Ought to Use Min/Max Box to Check first for Overlap */

                /* Create Binary Image of Region */

                imgset(ucimg1,0,xsize,ysize);
                npixels = rgnfil(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],
                                 ucimg1,xsize,ysize,glevel);
                /*          if( dzdisp ) dspsub2(mem,ucimg1,nsize);    */

                /* Check if Region Overlaps with Last Version of Region */

                pxlcnt = imgrovlpf(ucimg1,lastimg,xsize,ysize);
                if( pxlcnt > 0 )
                {
                    rgnfound = 1;
                    break;
                }
                else
                    printf("No Overlap; Try Next Region (%d)\n",k);
            } /* ==== End of Subregion Loop ==== */

            /* Abort Fitting if Problems */

            if( npixels <= MINREGN ) 
              { printf("(1) Region Has Too Few Pixels!\n"); 
                thisccnt = -10; break;
              }
            if( rgnfound == 0 )
                printf("Region NOT Found!\n");
            if( lastccnt == -2 )
            {
                ixlabl = ix;
                iylabl = iy;
            } /*savelabelloc*/

            /* Build (x,y,z) Surface List from Region Description */

            npixels = rgnxyz(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],
                             orgimg,xsize,ysize,x2ptr,y2ptr,z2ptr,limit);
            if( npixels <= MINREGN ) 
            {
                printf("(2) Region Has Too Few Pixels! ***BUG***\n");
                break;
            }
            xyzstats(x2ptr,y2ptr,z2ptr,npixels,&xmn,&ymn,&zmn,&xsd,&ysd,&zsd);
            free(subrgnptr);
            free(subrgnlst);

            /* Compact Point List When Too Many Points Used */

            npoints = xyzcmpct(x2ptr,y2ptr,z2ptr,npixels,xptr,yptr,zptr);
            imgcpy(lastimg,ucimg1,xsize,ysize); /* save regn as last version */
            if( first == 0 )
                imgcpy(ucimg4,ucimg1,xsize,ysize);
            if( npoints <= MINREGN ) 
            {
                printf("(3) Region Has Too Few Pixels! ***BUG***\n");
                break;
            }

            /* Fit Surface to This Region */ 

        start_again_fix:

            first = 1;
            if(npoints <= minfit[fitmode])
            {
                thisccnt = -6;
                break;
            }
            switch( fitmode )
            {
                case 0:     /* Planar Fit Mode ============== */
                            n1fits++;
                            tsecs = tstart();
                            plnsurfit(xptr,yptr,zptr,npoints,xmn,ymn,&errtol,
                                      nx,ny,cvec,fitmethod,ovec);
                            tstop(tsecs,"Time for Planar Fit");
                            break;
     
                case 1:     /* Fit Biquadratic Surface ============== */
                            n2fits++;
                            tsecs = tstart();
                            bqdsurfit(xptr,yptr,zptr,npoints,xmn,ymn,&errtol,
                                      nx,ny,cvec,fitmethod,ovec);
                            tstop(tsecs,"Time for Biquadratic Fit");
                            if( badcntrct && errtol > acceptf*errmax )
                            {
                                thisccnt = -4;
                                goto final_steps_fix;
                            }
                            if( errtol > 5.0*errmax)
                            {
                                thisccnt = -8;
                                goto final_steps_fix;
                            }
                            break;

                case 2:     /* Fit Bicubic Surface ============== */
                            n3fits++; 
                            if( fitlast <= 0 && numiter <= 2 ) /*indication of instability*/
                            {
                                thisccnt = -18;
                                printf("Two Consec. Order Jumps!!\n");
                                goto final_steps;
                            }

                            tsecs = tstart();
                            bcbsurfit(xptr,yptr,zptr,npoints,xmn,ymn,&errtol,
                                      nx,ny,cvec,fitmethod,ovec);
                            tstop(tsecs,"Time for Bicubic Fit");

                            if( badcntrct && errtol > acceptf*errmax )
                            {
                                thisccnt = -4;
                                goto final_steps;
                            }
                            if( errtol > 5.0*errmax)
                            {
                                thisccnt = -8;
                                goto final_steps;
                            }
                            if( errtol > errmax && npoints > minfit[2] )
                            {
                                fitmode=3;
                                printf("Incr_Fit_Order!\n");
                                goto start_again;
                            }
                            break;

                case 3:     /* Fit Biquartic Surface ============== */
                            n4fits++; 
                            tsecs = tstart();
                            bq4surfit(xptr,yptr,zptr,npoints,xmn,ymn,&errtol,
                                      nx,ny,cvec,fitmethod,ovec);
                            tstop(tsecs,"Time for Biquartic Fit");
                            break;
            }

            /* Analyze Fit to Data; Flag if Heuristic Runs Test Not Satisfied. */

        regn_grow_fix:

            if( fitmode == 0 || fitmode == 1 ) 
            {
                runflg = funchk(x2ptr,y2ptr,z2ptr,npixels,cvec,zfunc[fitmode],
                                &e1,&e2,&e8,ucimg1,xsize,ysize,dzdisp,testval);
                errtol = e2;
            }
            else
                runflg = 0;

            /* Perform Region Growing and other Operations common to all methods */

            lastccnt = thisccnt;        lastnpts = thisnpts;    thisnpts = npoints;
            lastconcnt = thisconcnt;    thisconcnt = npixels;   fitlast = fitmode;
            if( errtol <= acceptf*errmax )
            {
                errlim = evalerr(errtol, errfact, errmax);
                thisccnt = rgngrowr2(orgimg,ucimg2,xsize,ysize,errlim,
                                     cvec,zfunc[fitmode],errbuf,blakbord);
            } 
            /*        if( dzdisp ) dspsub2(mem,ucimg2,nsize);    */

            /* Save Parameters from this iteration */

            fmlist[ numiter ] = fitmode;
            conected[ numiter ] = thisconcnt;
            compat[ numiter ] = thisccnt;
            errlevl[ numiter++ ] = errtol;
            rmse = errsav = errtol;
            vmemchk( &memark );                     /* check virtual memory allocation */
            if( thisconcnt < SHRINKF*lastconcnt )   /* check for shrinkage */
            {
                thisccnt = -14;
                break;
            }

            /* Use Heuristic 2-D Runs Test Results (2%) */

            if( runflg && errtol > 0.1 ) 
            {
                thisccnt = -666;
                printf("*--> Stopping Fit at Current Fixed Mode\n"); 
                break;
            }
        } /* ===== End of Iterative Surf. Fit and Regn. Grow Loop ===== */

        /*
         * ===================================================
         * PROCESS ITERATED SURFACE RESULTS
         * ===================================================
         * Mark Pixels of Newly Grown Region in Blackboard,etc.
         * Requirements: Good Fit, 4-Connected Region, Low Error.
         */

    final_steps_fix:

        if( errtol <= acceptf*errmax && 
            thisccnt > MINREGN && thisconcnt > MINREGN ) 
        {
            surfadd = 0;  /* still possible to reject surface */

            /* Refine Regions to Eliminate Isolated Pixels */

            pxlcnt = regnrefn(ucimg2,ucimg1,xsize,ysize,0);
            if( pxlcnt ) 
            {
                pxlcnt = regnrefn(ucimg1,ucimg2,xsize,ysize,0);
                if(pxlcnt)
                    regnrefn(ucimg2,ucimg1,xsize,ysize,0);
                else
                    imgcpy(ucimg1,ucimg2,xsize,ysize);
            }
            burrtrim(ucimg1,ucimg2,xsize,ysize,0);  /* Trim Off Burrs */
            burrtrim(ucimg2,ucimg1,xsize,ysize,0);
            /*          if( dzdisp ) dspsub2(mem,ucimg1,nsize);    */

            /* Extract Sorted Connected Component Regions */

            subrgnlst = rgnxtr(ucimg1,xsize,ysize,0,0);
            nsbrgns = *subrgnlst;
            subrgnptr = imalloc(2*nsbrgns);
            subrofset = subrgnptr + nsbrgns;
            rgnsrt(subrgnlst+1,subrgnptr,subrofset,nsbrgns);

            /* Check Size of Largest Region for Potential Rejection */
            npixels = *(subrgnlst+1+5*subrgnptr[0]);
            if( npixels <= MINREGN ) /* prelim check for stability */
            {
                hkupdate(stypimg,ucimg4,xsize,ysize);
                printf("\n*** Unstable Seed Region Discarded (%d) ***\n",npixels);
                free(subrgnptr);
                free(subrgnlst);
                goto end_while_loop_fix;  /* reject */
            }
            isurf++;
            plotfil = gfopen("plot.data","a");
            iterprnt(plotfil,numiter,errlevl,compat,conected,fmlist,isurf);
            fclose(plotfil);
            binflg = 0;
            printf("\n*** Surface Primitive %d Interpreted! ***\n",isurf);
            printf("%d Sub-Regions for Surface_# %d\n",nsbrgns,isurf);

            /* For Each Compatible Sub-Region of Surface Until Too Small */

            for(k=0;k<nsbrgns;++k)
            {
                npixels = *(subrgnlst+1+5*subrgnptr[k]);
                if( npixels <= MINREGN && surfadd == 1 )
                    break; 
                imgset(ucimg2,0,xsize,ysize);
                npixels = rgnfilerr(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],
                                    ucimg2,xsize,ysize,orgimg,cvec,zfunc[fitlast],&e1,&e2,&e8);
                pxlcnt = imgrovlpf(ucimg2,lastimg,xsize,ysize);
                /*            if( dzdisp ) dspsub2(mem,ucimg2,nsize);    */

                /* Process Extra Compatible Regions; Continue if Not OK */

                if( pxlcnt <= 0 )
                {
                    if( npixels < mininitsize && surfadd == 1 )
                        break;
                    ncontrcts = cntrctest(ucimg2,ucimg1,ucimg3,xsize,ysize,pixlcnts,2);
                    if( ncontrcts < 2 )
                        continue; 
                    if( pixlcnts[1] <= MINREGN )
                        continue;
                    printf("%d Pixels Left after 2 Contractions\n",pixlcnts[1]);
                    if( e2 > acceptf*errmax )
                        goto error_buf_upd_fix;
                    if( e2 > errmax && errsav < errmax )
                        goto error_buf_upd_fix;
                    rgnsiz(subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]],&nx,&ny,&ixlabl,&iylabl);

                    /* I have assumed that if the first region is Compatible,
                       but is not the original region, it is ok to use the
                       center of the min/max box as a label, because the region
                       probably does not have any holes in it anyway. This 
                       whole problem could be corrected with a more elaborate
                       approach, but does not seem worth it just for labels.
                       This scheme should be right 99% of the time. */
                }
                else
                    errsav = e2;
                printf("\n Adding_Surface_Region_with %d Pixels\n",npixels);
                /*            if( dzdisp )      dspsub2(mem,ucimg2,nsize);    */

                if( dzdisp == 3 )
                    waitret();

                /* Write Out Region and Surface Information to File */

                fprintf(filptr,"@-------------( %d )( %d %d )\n",npixels,ixlabl,iylabl);
                fprintf(filptr,"%d %d <> Surface_# %d  of Type: %s (SR=%d)\n",
                        isurf,hklabl[0],isurf,stringcode[hklabl[0]],k);
                scoefprnt(filptr,fitlast,cvec);
                fprintf(filptr,"\n%g %g %g (e8,e2,e1), %g\n",e8,e2,e1,rmse);
                rgnwt1(filptr,subrgnlst,subrgnptr[k],subrofset[subrgnptr[k]]);

                /* Write Coefficients to Binary Coefficients File */

                if( pxlcnt > 0 && binflg == 0 ) 
                {
                    binfil = gfopen("bincoef","a");
                    fwrite(&minfit[fitlast],sizeof(int),1,binfil);
                    fwrite(cvec,sizeof(double),minfit[fitlast],binfil);
                    errtol = e8;        fwrite(&errtol,sizeof(float),1,binfil);
                    errtol = e2;        fwrite(&errtol,sizeof(float),1,binfil);
                    errtol = e1;        fwrite(&errtol,sizeof(float),1,binfil);
                    fwrite(&rmse,sizeof(float),1,binfil);
                    fclose(binfil);
                    binflg = 1;
                }

                /* Update Blackboard Image */

                pxlcnt = bbupdate(blakbord,ucimg2,xsize,ysize,isurf); 
                printf("BlackBoard Display <%d|%d>\n",isurf,hklabl[0]);
                /*            if( dzdisp ) dspsub2(mem,blakbord,nsize);    */
                wrtimage("blakbord",blakbord,xsize,ysize);
                if( pxlcnt == 0 ) 
                    printf("bb: Infinite Loop Problem: Duplicate Surface\n");

                /* Update Reconstruction Image */

                imgor2(rcnsimg,ucimg2,xsize,ysize);
                printf("Reconstructed Image Display <%d|%d>\n",isurf,hklabl[0]);
                /*            if( dzdisp ) dspsub2(mem,rcnsimg,nsize);    */
                wrtimage("recons",rcnsimg,xsize,ysize);
                pxlcnt = hkupdate(stypimg,ucimg2,xsize,ysize);
                if( pxlcnt == 0 ) 
                    printf("hk: Infinite Loop Problem: Duplicate Surface\n");
                surfadd = 1;
                totalpix += npixels;

                /* Update Error Buffer: Unconditional Update,
                   Then 4 dilations, Then Conditional Update */

    error_buf_upd_fix:

                tsecs = tstart();
                ebupdat1(orgimg,ucimg2,xsize,ysize,cvec,zfunc[fitlast],
                         errbuf,surfimg,rgnlabls,isurf); 
                imgith(ucimg1,ucimg2,xsize,ysize,0);
                firstx = -1;    lastx = -1;
                dilate(ucimg2,ucimg3,xsize,ysize,&firstx,&lastx);
                dilate(ucimg3,ucimg2,xsize,ysize,&firstx,&lastx);
                dilate(ucimg2,ucimg3,xsize,ysize,&firstx,&lastx);
                dilate(ucimg3,ucimg2,xsize,ysize,&firstx,&lastx);
                dilate(ucimg2,ucimg3,xsize,ysize,&firstx,&lastx);
                dilate(ucimg3,ucimg2,xsize,ysize,&firstx,&lastx);
                imgand2(ucimg2,ucimg1,xsize,ysize);
                /*         if( dzdisp ) dspsub2(mem,ucimg2,nsize);     */
                imgnotip(ucimg1,xsize,ysize);
                errlim = 5.0*errfact*evalerr( e2, errfact, errmax);
                ebupdat2(orgimg,ucimg2,ucimg1,ucimg3,xsize,ysize,cvec,
                         zfunc[fitlast],errbuf,surfimg,rgnlabls,isurf,errlim);
                tstop(tsecs,"Time for dilate & ebupdat1,2 Subroutines");
                /*         if( dzdisp ) dspsub2(mem,ucimg2,nsize);     */
                /*        if( dzdisp ) dspsub2(mem,surfimg,nsize);    */
                wrtimage("rgnlabl",rgnlabls,xsize,ysize);
                wrtimage("surfimg",surfimg,xsize,ysize);
                if( dzdisp == 3 )
                    waitret(); 
            }  /* -----End of Sub-Region Loop----------------------------- */

            free(subrgnptr);
            free(subrgnlst);
            if( surfadd == 0 ) 
            {
                hkupdate(stypimg,ucimg4,xsize,ysize);
                printf("\n*** Misleading Seed Region Discarded ***\n");
            }
            if( surfadd )  /* new section */
            {
                errtol = errsum(errbuf,xsize,ysize,errmrk);
                if( errtol <= errmaxfix )
                {
                    imgthr(ucimg1,blakbord,xsize,ysize,254); /* threshold */
                    contract(ucimg1,ucimg3,xsize,ysize);   /* contract */
                    pxlcnt = rgnbrc(ucimg3,xsize,ysize,255,1); /* rgn size */
                    /*              if( dzdisp ) dspsub2(mem,ucimg3,nsize);    */
                    if( pxlcnt < maxbbrs ) 
                    {
                        tstop(itime,"Time for This Iteration"); iloop++;
                        goto all_done;  /* Finished!! */
                    } 
                    printf("Check Region with %d Pixels(Cntrctd); Error OK\n",pxlcnt);
                    if( dzdisp == 3 ) waitret();
                }
            }
        }
        else
        {
            /* If BlackBoard Processing & Error Problem, Allow 2nd chance */

            if( hklabl[0] == 9 && thisccnt > MINREGN )
            {
                pxlcnt = contract(ucimg4,ucimg2,xsize,ysize);
                if( pxlcnt > MINREGN ) 
                {
                    printf("Giving Black_Board Region 2nd Chance\n");
                    goto surface_fit_fix;
                }
            }

            /* do not mess with this seed region again */

            hkupdate(stypimg,ucimg4,xsize,ysize);
            printf("\n*** Bad Growth! This Seed Region Discarded ***\n");
            printf("Error= %g > Limit= %g (%d,%d)\n",errtol,acceptf*errmax,thisccnt,thisconcnt);
        }

    end_while_loop_fix:

        tstop(itime,"Time for This Iteration");
        iloop++;
    } /* ===== End of Region Loop ===== */

    /* Re-Display Reconstructed Image and Original and Difference */

all_done:

    printf("=== All Done! ===\n");
    /*    if( dzdisp ) { dspsub2(mem,rcnsimg,nsize); waitret(); }    */
    /*    if( dzdisp ) { dspsub2(mem,orgimg,nsize);  waitret(); }    */
    /*    if( dzdisp ) { dspsub2(mem,surfimg,nsize); waitret(); }    */
    fimglim(errbuf,xsize,ysize,64.0);
    fltucimg(ucimg1,errbuf,xsize,ysize,&e1,&e2);
    /*    if( dzdisp ) { dspsub2(mem,ucimg1,nsize);  }    */
    imgsub2(orgimg,surfimg,xsize,ysize);
    /*    if( dzdisp ) { dspsub2(mem,orgimg,nsize);  }    */
    imgmsd(orgimg,xsize,ysize,&mean,&sdev,&imax,&imin);
    wrtimage("errimg",orgimg,xsize,ysize);
    printf("Image Error: %d  %d  %d  %d (Mean,Sdev,Max,Min)\n",mean,sdev,imax,imin);
    imgthr(ucimg3,orgimg,xsize,ysize,sdev);
    /*    if( dzdisp ) { dspsub2(mem,ucimg3,nsize);  }    */
    pxlcnt = rgnbri(ucimg3,ucimg2,xsize,ysize,255,1);
    /*    if( dzdisp ) { dspsub2(mem,ucimg2,nsize);  }    */

    tstop(totime,"Total Time to Compute Surfaces");
    printf("%d Region Loops --> %d Surfaces\n",iloop,isurf);
    printf("Total: %d Pixels in Surface Regions\n",totalpix);

    fclose(filptr);
}
